@@ -2,24 +2,25 @@
 // the GNU General Public License (See COPYING for details).
 // Copyright (C) 2001 Alistair Riddoch
 
-#ifndef CREATOR_CLIENT_H
-#define CREATOR_CLIENT_H
+#ifndef CLIENT_CREATOR_CLIENT_H
+#define CLIENT_CREATOR_CLIENT_H
 
 #include "CharacterClient.h"
 
 class Entity;
 
+/// \brief Class to implement a creator entity in an admin client
 class CreatorClient : public CharacterClient {
   private:
-    Entity * sendLook(RootOperation & op);
+    Entity * sendLook(const Operation & op);
   public:
     CreatorClient(const std::string&, const std::string&, ClientConnection&);
 
-    Entity * make(const Element &);
-    void sendSet(const std::string &, const Element &);
+    Entity * make(const Atlas::Objects::Entity::RootEntity &);
+    void sendSet(const std::string &, const Atlas::Objects::Entity::RootEntity &);
     Entity * look(const std::string &);
-    Entity * lookFor(const Element &);
+    Entity * lookFor(const Atlas::Objects::Entity::RootEntity &);
     int runScript(const std::string & package, const std::string & function);
 };
 
-#endif // CREATOR_CLIENT_H
+#endif // CLIENT_CREATOR_CLIENT_H
