@@ -1,21 +1,22 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2000,2001 Alistair Riddoch
+// Copyright (C) 2005 Alistair Riddoch
 
 #ifndef COMMON_CHOP_H
 #define COMMON_CHOP_H
 
-#include <Atlas/Objects/Operation/Action.h>
+#include <Atlas/Objects/Generic.h>
 
 namespace Atlas { namespace Objects { namespace Operation {
 
-class Chop : public Action {
-  protected:
-    Chop(const char *, const char *);
+extern int CHOP_NO;
+
+class Chop : public Generic
+{
   public:
-    Chop();
-    virtual ~Chop();
-    static Chop Class();
+    Chop() {
+        (*this)->setType("chop", CHOP_NO);
+    }
 };
 
 } } }
