@@ -1,26 +1,27 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2001-2003 Alistair Riddoch
+// Copyright (C) 2000-2004 Alistair Riddoch
 
 #include "Database.h"
 
 #include "log.h"
 #include "debug.h"
 #include "globals.h"
-#include "stringstream.h"
+#include "compose.hpp"
 
-#include <Atlas/Message/Encoder.h>
+#include <Atlas/Message/MEncoder.h>
+#include <Atlas/Message/Element.h>
 #include <Atlas/Codecs/XML.h>
 
 #include <varconf/Config.h>
 
 #include <iostream>
+#include <sstream>
 
 #include <cassert>
 
 using Atlas::Message::Element;
 using Atlas::Message::MapType;
-using Atlas::Message::ListType;
 
 typedef Atlas::Codecs::XML Serialiser;
 
@@ -86,7 +87,7 @@ int Database::initConnection(bool createDatabase)
 
     std::string dbname = "cyphesis";
     if (global_conf->findItem("cyphesis", "dbname")) {
-        dbname = std::string(global_conf->getItem("cyphesis", "dbname"));
+        dbname = global_conf->getItem("cyphesis", "dbname").as_string();
     }
     conninfos << "dbname=" << dbname << " ";
 
@@ -100,7 +101,7 @@ int Database::initConnection(bool createDatabase)
     }
 
     if (global_conf->findItem("cyphesis", "dbpasswd")) {
-        conninfos << "password=" << std::string(global_conf->getItem("cyphesis", "dbpasswd")) << " ";
+        conninfos << "password=" << global_conf->getItem("cyphesis", "dbpasswd").as_string() << " ";
     }
 
     const std::string cinfo = conninfos.str();
@@ -113,8 +114,13 @@ int Database::initConnection(bool createDatabase)
 
     m_connection = PQconnectdb(cinfo.c_str());
 
-    if ((m_connection == NULL) || (PQstatus(m_connection) != CONNECTION_OK)) {
-        log(ERROR, "Database connection failed");
+    if (m_connection == NULL) {
+        log(ERROR, "Database connection failed with unknown error");
+        return -1;
+    }
+
+    if (PQstatus(m_connection) != CONNECTION_OK) {
+        log(ERROR, String::compose("Connection to database failed: \"%1\"", PQerrorMessage(m_connection)).c_str());
         return -1;
     }
 
@@ -179,8 +185,8 @@ bool Database::decodeObject(const std::string & data,
 
     std::stringstream str(data, std::ios::in);
 
-    Serialiser codec(str, &m_d);
-    Atlas::Message::Encoder enc(&codec);
+    Serialiser codec(str, m_d);
+    Atlas::Message::Encoder enc(codec);
 
     // Clear the decoder
     m_d.get();
@@ -201,11 +207,11 @@ bool Database::encodeObject(const MapType & o,
 {
     std::stringstream str;
 
-    Serialiser codec(str, &m_d);
-    Atlas::Message::Encoder enc(&codec);
+    Serialiser codec(str, m_d);
+    Atlas::Message::Encoder enc(codec);
 
     codec.streamBegin();
-    enc.streamMessage(o);
+    enc.streamMessageElement(o);
     codec.streamEnd();
 
     data = str.str();
@@ -265,17 +271,18 @@ bool Database::putObject(const std::string & table,
                     << std::endl << std::flush;);
     std::stringstream str;
 
-    Serialiser codec(str, &m_d);
-    Atlas::Message::Encoder enc(&codec);
+    Serialiser codec(str, m_d);
+    Atlas::Message::Encoder enc(codec);
 
     codec.streamBegin();
-    enc.streamMessage(o);
+    enc.streamMessageElement(o);
     codec.streamEnd();
 
     debug(std::cout << "Encoded to: " << str.str().c_str() << " "
                << str.str().size() << std::endl << std::flush;);
     std::string query = std::string("INSERT INTO ") + table + " VALUES ('" + key;
-    for (StringVector::const_iterator I = c.begin(); I != c.end(); ++I) {
+    StringVector::const_iterator Iend = c.end();
+    for (StringVector::const_iterator I = c.begin(); I != Iend; ++I) {
         query += "', '";
         query += *I;
     }
@@ -293,11 +300,11 @@ bool Database::updateObject(const std::string & table,
                     << std::endl << std::flush;);
     std::stringstream str;
 
-    Serialiser codec(str, &m_d);
-    Atlas::Message::Encoder enc(&codec);
+    Serialiser codec(str, m_d);
+    Atlas::Message::Encoder enc(codec);
 
     codec.streamBegin();
-    enc.streamMessage(o);
+    enc.streamMessageElement(o);
     codec.streamEnd();
 
     std::string query = std::string("UPDATE ") + table + " SET contents = '" +
@@ -391,7 +398,7 @@ bool Database::getTable(const std::string & table, MapType &o)
     }
 
     MapType t;
-    for(int i = 0; i < results; i++) {
+    for(int i = 0; i < results; ++i) {
         const char * key = PQgetvalue(res, i, id_column);
         const char * data = PQgetvalue(res, i, contents_column);
         debug(std::cout << "Got record " << key << " from database, value "
@@ -437,7 +444,7 @@ const DatabaseResult Database::runSimpleSelectQuery(const std::string & query)
     clearPendingQuery();
     int status = PQsendQuery(m_connection, query.c_str());
     if (!status) {
-        log(ERROR, "Database query error.");
+        log(ERROR, "runSimpleSelectQuery(): Database query error.");
         reportError();
         return DatabaseResult(0);
     }
@@ -470,7 +477,7 @@ bool Database::runCommandQuery(const std::string & query)
     clearPendingQuery();
     int status = PQsendQuery(m_connection, query.c_str());
     if (!status) {
-        log(ERROR, "Database query error.");
+        log(ERROR, "runCommandQuery(): Database query error.");
         reportError();
         return false;
     }
@@ -527,7 +534,7 @@ bool Database::registerRelation(std::string & tablename,
     clearPendingQuery();
     int status = PQsendQuery(m_connection, query.c_str());
     if (!status) {
-        log(ERROR, "Database query error.");
+        log(ERROR, "registerRelation(): Database query error.");
         reportError();
         return false;
     }
@@ -627,8 +634,8 @@ bool Database::registerSimpleTable(const std::string & name,
     createquery += name;
     query += " WHERE id = 0";
     createquery += " (id integer UNIQUE PRIMARY KEY";
-    MapType::const_iterator I = row.begin();
-    for(; I != row.end(); ++I) {
+    MapType::const_iterator Iend = row.end();
+    for (MapType::const_iterator I = row.begin(); I != Iend; ++I) {
         query += " AND ";
         createquery += ", ";
         const std::string & column = I->first;
@@ -637,7 +644,7 @@ bool Database::registerSimpleTable(const std::string & name,
         const Element & type = I->second;
         if (type.isString()) {
             query += " LIKE 'foo'";
-            int size = type.asString().size();
+            int size = type.String().size();
             if (size == 0) {
                 createquery += " text";
             } else {
@@ -662,7 +669,7 @@ bool Database::registerSimpleTable(const std::string & name,
     clearPendingQuery();
     int status = PQsendQuery(m_connection, query.c_str());
     if (!status) {
-        log(ERROR, "Database query error.");
+        log(ERROR, "registerSimpleTable(): Database query error.");
         reportError();
         return false;
     }
@@ -759,7 +766,7 @@ bool Database::registerEntityIdGenerator()
     clearPendingQuery();
     int status = PQsendQuery(m_connection, "SELECT * FROM entity_ent_id_seq");
     if (!status) {
-        log(ERROR, "Database query error.");
+        log(ERROR, "registerEntityIdGenerator(): Database query error.");
         reportError();
         return false;
     }
@@ -780,7 +787,7 @@ bool Database::newId(std::string & id)
     int status = PQsendQuery(m_connection,
                              "SELECT nextval('entity_ent_id_seq')");
     if (!status) {
-        log(ERROR, "Database query error.");
+        log(ERROR, "newId(): Database query error.");
         reportError();
         return false;
     }
@@ -849,8 +856,8 @@ bool Database::registerEntityTable(const std::string & classname,
     if (parent.empty()) {
         createquery += "id integer UNIQUE PRIMARY KEY, ";
     }
-    MapType::const_iterator I = row.begin();
-    for(; I != row.end(); ++I) {
+    MapType::const_iterator Iend = row.end();
+    for (MapType::const_iterator I = row.begin(); I != Iend; ++I) {
         if (I != row.begin()) {
             query += " AND ";
             createquery += ", ";
@@ -861,7 +868,7 @@ bool Database::registerEntityTable(const std::string & classname,
         const Element & type = I->second;
         if (type.isString()) {
             query += " LIKE 'foo'";
-            int size = type.asString().size();
+            int size = type.String().size();
             if (size == 0) {
                 createquery += " text";
             } else {
@@ -891,7 +898,7 @@ bool Database::registerEntityTable(const std::string & classname,
     clearPendingQuery();
     int status = PQsendQuery(m_connection, query.c_str());
     if (!status) {
-        log(ERROR, "Database query error.");
+        log(ERROR, "registerEntityTable(): Database query error.");
         reportError();
         return false;
     }
@@ -1097,7 +1104,8 @@ bool Database::registerArrayTable(const std::string & name,
         indexquery += array_axes[i];
     }
 
-    for (MapType::const_iterator I = row.begin(); I != row.end(); ++I) {
+    MapType::const_iterator Iend = row.end();
+    for (MapType::const_iterator I = row.begin(); I != Iend; ++I) {
         const std::string & column = I->first;
 
         query += " AND ";
@@ -1110,7 +1118,7 @@ bool Database::registerArrayTable(const std::string & name,
 
         if (type.isString()) {
             query += " LIKE 'foo'";
-            int size = type.asString().size();
+            int size = type.String().size();
             if (size == 0) {
                 createquery += " text";
             } else {
@@ -1135,7 +1143,7 @@ bool Database::registerArrayTable(const std::string & name,
     clearPendingQuery();
     int status = PQsendQuery(m_connection, query.c_str());
     if (!status) {
-        log(ERROR, "Database query error.");
+        log(ERROR, "registerArrayTable(): Database query error.");
         reportError();
         return false;
     }
@@ -1194,24 +1202,27 @@ bool Database::createArrayRow(const std::string & name,
     for (unsigned int i = 0; i < key.size(); ++i) {
         query << ", " << array_axes[i];
     }
-    for (MapType::const_iterator I = data.begin(); I != data.end(); ++I) {
+    MapType::const_iterator Iend = data.end();
+    for (MapType::const_iterator I = data.begin(); I != Iend; ++I) {
         query << ", " << I->first;
     }
     query << " ) VALUES ( " << id;
-    for(std::vector<int>::const_iterator I = key.begin(); I != key.end(); ++I) {
-        query << ", " << *I;
+    std::vector<int>::const_iterator Jend = key.end();
+    for (std::vector<int>::const_iterator J = key.begin(); J != Jend; ++J) {
+        query << ", " << *J;
     }
-    for (MapType::const_iterator I = data.begin(); I != data.end(); ++I) {
+    // We assume data has not been modified, so Iend is still valid
+    for (MapType::const_iterator I = data.begin(); I != Iend; ++I) {
         const Element & e = I->second;
         switch (e.getType()) {
           case Element::TYPE_INT:
-            query << ", " << e.asInt();
+            query << ", " << e.Int();
             break;
           case Element::TYPE_FLOAT:
-            query << ", " << e.asFloat();
+            query << ", " << e.Float();
             break;
           case Element::TYPE_STRING:
-            query << ", " << e.asString();
+            query << ", " << e.String();
             break;
           default:
             log(ERROR, "Bad type constructing array database row for insert");
@@ -1237,7 +1248,8 @@ bool Database::updateArrayRow(const std::string & name,
     std::stringstream query;
 
     query << "UPDATE " << name << " SET ";
-    for (MapType::const_iterator I = data.begin(); I != data.end(); ++I) {
+    MapType::const_iterator Iend = data.end();
+    for (MapType::const_iterator I = data.begin(); I != Iend; ++I) {
         if (I != data.begin()) {
             query << ", ";
         }
@@ -1245,13 +1257,13 @@ bool Database::updateArrayRow(const std::string & name,
         const Element & e = I->second;
         switch (e.getType()) {
           case Element::TYPE_INT:
-            query << e.asInt();
+            query << e.Int();
             break;
           case Element::TYPE_FLOAT:
-            query << e.asFloat();
+            query << e.Float();
             break;
           case Element::TYPE_STRING:
-            query << "'" << e.asString() << "'";
+            query << "'" << e.String() << "'";
             break;
           default:
             log(ERROR, "Bad type constructing array database row for update");
@@ -1384,8 +1396,8 @@ bool Database::runMaintainance(int command)
     // FIXME VACUUM and REINDEX tables from a common store
     if ((command & MAINTAIN_REINDEX) == MAINTAIN_REINDEX) {
         std::string query("REINDEX TABLE ");
-        TableSet::const_iterator I = allTables.begin();
-        for(; I != allTables.end(); ++I) {
+        TableSet::const_iterator Iend = allTables.end();
+        for (TableSet::const_iterator I = allTables.begin(); I != Iend; ++I) {
             debug(std::cout << (query + *I) << std::endl << std::flush;);
             scheduleCommand(query + *I);
         }
@@ -1398,8 +1410,8 @@ bool Database::runMaintainance(int command)
         if ((command & MAINTAIN_VACUUM_FULL) == MAINTAIN_VACUUM_FULL) {
             query += "FULL ";
         }
-        TableSet::const_iterator I = allTables.begin();
-        for(; I != allTables.end(); ++I) {
+        TableSet::const_iterator Iend = allTables.end();
+        for(TableSet::const_iterator I = allTables.begin(); I != Iend; ++I) {
             debug(std::cout << (query + *I) << std::endl << std::flush;);
             scheduleCommand(query + *I);
         }
