@@ -1,21 +1,22 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2000,2001 Alistair Riddoch
+// Copyright (C) 2005 Alistair Riddoch
 
 #ifndef COMMON_EAT_H
 #define COMMON_EAT_H
 
-#include <Atlas/Objects/Operation/Action.h>
+#include <Atlas/Objects/Generic.h>
 
 namespace Atlas { namespace Objects { namespace Operation {
 
-class Eat : public Action {
-  protected:
-    Eat(const char *, const char *);
+extern int EAT_NO;
+
+class Eat : public Generic
+{
   public:
-    Eat();
-    virtual ~Eat();
-    static Eat Class();
+    Eat() {
+        (*this)->setType("eat", EAT_NO);
+    }
 };
 
 } } }
