@@ -1,21 +1,22 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2000,2001 Alistair Riddoch
+// Copyright (C) 2005 Alistair Riddoch
 
 #ifndef COMMON_NOURISH_H
 #define COMMON_NOURISH_H
 
-#include <Atlas/Objects/Operation/Action.h>
+#include <Atlas/Objects/Generic.h>
 
 namespace Atlas { namespace Objects { namespace Operation {
 
-class Nourish : public Action {
-  protected:
-    Nourish(const char *, const char *);
+extern int NOURISH_NO;
+
+class Nourish : public Generic
+{
   public:
-    Nourish();
-    virtual ~Nourish();
-    static Nourish Class();
+    Nourish() {
+        (*this)->setType("nourish", NOURISH_NO);
+    }
 };
 
 } } }
