@@ -1,9 +1,9 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2000,2001 Alistair Riddoch
+// Copyright (C) 2000-2004 Alistair Riddoch
 
-#ifndef SERVER_OOG_THING_H
-#define SERVER_OOG_THING_H
+#ifndef COMMON_OOG_THING_H
+#define COMMON_OOG_THING_H
 
 #include "BaseEntity.h"
 
@@ -16,7 +16,7 @@ class OOGThing : public BaseEntity {
     explicit OOGThing(const std::string & id);
   public:
     virtual ~OOGThing();
-    virtual void OtherOperation(const RootOperation & op, OpVector &);
+    virtual void OtherOperation(const Operation & op, OpVector &);
 };
 
-#endif // SERVER_OOG_THING_H
+#endif // COMMON_OOG_THING_H
