@@ -1,21 +1,22 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2000,2001 Alistair Riddoch
+// Copyright (C) 2005 Alistair Riddoch
 
 #ifndef COMMON_SETUP_H
 #define COMMON_SETUP_H
 
-#include <Atlas/Objects/Operation/RootOperation.h>
+#include <Atlas/Objects/Generic.h>
 
 namespace Atlas { namespace Objects { namespace Operation {
 
-class Setup : public RootOperation {
-  protected:
-    Setup(const char *, const char *);
+extern int SETUP_NO;
+
+class Setup : public Generic
+{
   public:
-    Setup();
-    virtual ~Setup();
-    static Setup Class();
+    Setup() {
+        (*this)->setType("setup", SETUP_NO);
+    }
 };
 
 } } }
