@@ -1,6 +1,6 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2000,2001 Alistair Riddoch
+// Copyright (C) 2000-2003 Alistair Riddoch
 
 #include "log.h"
 #include "globals.h"
@@ -9,6 +9,7 @@
 
 extern "C" {
   #include <syslog.h>
+  #include <errno.h>
 }
 
 void initLogger()
@@ -76,3 +77,13 @@ void log(LogLevel lvl, const char * msg)
         std::cerr << type << ": " << msg << std::endl << std::flush;
     }
 }
+
+void logSysError(LogLevel lvl)
+{
+    char * err = strerror(errno);
+    if (err != NULL) {
+        log(lvl, err);
+    } else {
+        log(ERROR, "Error getting error message from system.");
+    }
+}
