@@ -9,16 +9,18 @@
 
 typedef Character Creator_parent;
 
+/// \brief This is a class for an in-game entity used by administrators
+/// and world builders to manipulate the world
 class Creator : public Creator_parent {
   public:
     explicit Creator(const std::string & id);
 
-    void sendExternalMind(const RootOperation & msg, OpVector &);
+    void sendExternalMind(const Operation & op, OpVector &);
 
-    virtual void operation(const RootOperation & op, OpVector &);
-    virtual void externalOperation(const RootOperation & op);
+    virtual void operation(const Operation & op, OpVector &);
+    virtual void externalOperation(const Operation & op);
 
-    virtual void mindLookOperation(const Look & op, OpVector &);
+    virtual void mindLookOperation(const Operation & op, OpVector &);
 };
 
 #endif // RULESETS_CREATOR_H
