@@ -1,6 +1,6 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2000,2001 Alistair Riddoch
+// Copyright (C) 2000-2005 Alistair Riddoch
 
 #include "MemEntity.h"
 #include "MemMap.h"
@@ -8,14 +8,19 @@
 
 #include "common/log.h"
 #include "common/debug.h"
-#include "common/stringstream.h"
 
-#include <Atlas/Objects/Operation/Look.h>
+#include <Atlas/Objects/Operation.h>
+#include <Atlas/Objects/Anonymous.h>
+
+#include <sstream>
 
 static const bool debug_flag = false;
 
-// This is a template which requires debug flag to be declared.
-#include "Entity_getLocation.h"
+using Atlas::Message::Element;
+using Atlas::Message::MapType;
+using Atlas::Objects::Operation::Look;
+using Atlas::Objects::Entity::RootEntity;
+using Atlas::Objects::Entity::Anonymous;
 
 MemEntity * MemMap::addEntity(MemEntity * entity)
 {
@@ -32,57 +37,64 @@ MemEntity * MemMap::addEntity(MemEntity * entity)
     m_checkIterator = m_entities.find(next);
 
     debug( std::cout << this << std::endl << std::flush;);
-    std::vector<std::string>::const_iterator I;
-    for(I = m_addHooks.begin(); I != m_addHooks.end(); I++) {
+    std::vector<std::string>::const_iterator I = m_addHooks.begin();
+    std::vector<std::string>::const_iterator Iend = m_addHooks.end();
+    for (; I != Iend; ++I) {
         m_script->hook(*I, entity);
     }
     return entity;
 }
 
-void MemMap::readEntity(MemEntity * entity, const MapType & entmap)
+void MemMap::readEntity(MemEntity * entity, const RootEntity & ent)
 // Read the contents of an Atlas message into an entity
 {
-    MapType::const_iterator I = entmap.find("name");
-    if (I != entmap.end() && I->second.isString()) {
-        entity->setName(I->second.asString());
-    }
-    I = entmap.find("type");
-    if (I != entmap.end() && I->second.isString()) {
-        entity->setType(I->second.asString());
-    }
-    entity->merge(entmap);
-    I = entmap.find("loc");
-    if ((I != entmap.end()) && I->second.isString()) {
-        getAdd(I->second.asString());
-    }
-    Entity * old_loc = entity->m_location.m_loc;
-    entity->getLocation(entmap, m_entities);
-    if (old_loc != entity->m_location.m_loc) {
-        if (old_loc != 0) {
-            old_loc->m_contains.erase(entity);
+    if (ent->hasAttrFlag(Atlas::Objects::PARENTS_FLAG)) {
+        const std::list<std::string> & parents = ent->getParents();
+        if (!parents.empty()) {
+            if (entity->getType() == "entity") {
+                entity->setType(parents.front());
+            } else if (entity->getType() != parents.front()) {
+                debug(std::cout << "Attempting to mutate " << entity->getType()
+                                << " into " << parents.front()
+                                << std::endl << std::flush;);
+            }
         }
-        entity->m_location.m_loc->m_contains.insert(entity);
     }
-    addContents(entmap);
+    entity->merge(ent->asMessage());
+    if (ent->hasAttrFlag(Atlas::Objects::Entity::LOC_FLAG)) {
+        Entity * old_loc = entity->m_location.m_loc;
+        const std::string & new_loc_id = ent->getLoc();
+        // Has LOC been changed?
+        if (old_loc == 0 || new_loc_id != old_loc->getId()) {
+            entity->m_location.m_loc = getAdd(new_loc_id);
+            assert(old_loc != entity->m_location.m_loc);
+            if (old_loc != 0) {
+                old_loc->m_contains.erase(entity);
+            }
+            entity->m_location.m_loc->m_contains.insert(entity);
+        }
+        entity->m_location.readFromEntity(ent);
+    }
+    addContents(ent);
 }
 
-void MemMap::updateEntity(MemEntity * entity, const MapType & entmap)
+void MemMap::updateEntity(MemEntity * entity, const RootEntity & ent)
 // Update contents of entity an Atlas message.
 {
     assert(entity != 0);
 
     debug( std::cout << " got " << entity << std::endl << std::flush;);
 
-    readEntity(entity, entmap);
+    readEntity(entity, ent);
 
-    std::vector<std::string>::const_iterator K;
-    for(K = m_updateHooks.begin(); K != m_updateHooks.end(); K++) {
+    std::vector<std::string>::const_iterator K = m_updateHooks.begin();
+    std::vector<std::string>::const_iterator Kend = m_updateHooks.end();
+    for (; K != Kend; ++K) {
         m_script->hook(*K, entity);
     }
 }
 
-MemEntity * MemMap::newEntity(const std::string & id,
-                              const MapType & entmap)
+MemEntity * MemMap::newEntity(const std::string & id, const RootEntity & ent)
 // Create a new entity from an Atlas message.
 {
     assert(!id.empty());
@@ -90,7 +102,7 @@ MemEntity * MemMap::newEntity(const std::string & id,
 
     MemEntity * entity = new MemEntity(id);
 
-    readEntity(entity, entmap);
+    readEntity(entity, ent);
 
     return addEntity(entity);
 }
@@ -99,17 +111,16 @@ MemMap::MemMap(Script *& s) : m_checkIterator(m_entities.begin()), m_script(s)
 {
 }
 
-RootOperation * MemMap::lookId()
+Atlas::Objects::Operation::RootOperation MemMap::lookId()
 // Generate a look operation to look at an entity we are interested in
 {
     debug( std::cout << "MemMap::lookId" << std::endl << std::flush;);
     if (!m_additionsById.empty()) {
         const std::string & id = m_additionsById.front();
-        Look * l = new Look();
-        MapType m;
-        m["id"] = id;
-        l->setArgs(ListType(1, m));
-        l->setTo(id);
+        Look l;
+        Anonymous m;
+        m->setId(id);
+        l->setArgs1(m);
         m_additionsById.pop_front();
         return l;
     }
@@ -144,30 +155,34 @@ void MemMap::del(const std::string & id)
 
         // Handling re-parenting is done very similarly to Entity::destroy,
         // but is slightly different as we tolerate LOC being null.
-        Entity * mloc = ent->m_location.m_loc;
-        if (mloc != 0) {
+        Entity * ent_loc = ent->m_location.m_loc;
+        if (ent_loc != 0) {
             // Remove deleted entity from its parents contains
-            mloc->m_contains.erase(ent);
+            ent_loc->m_contains.erase(ent);
         }
+        // FIXME This is required until MemMap uses parent refcounting
+        ent->m_location.m_loc = 0;
 
         // Add deleted entities children into its parents contains
         EntitySet::const_iterator K = ent->m_contains.begin();
-        for (; K != ent->m_contains.end(); ++K) {
-            Entity * cent = *K;
-            cent->m_location.m_loc = mloc;
+        EntitySet::const_iterator Kend = ent->m_contains.end();
+        for (; K != Kend; ++K) {
+            Entity * child_ent = *K;
+            child_ent->m_location.m_loc = ent_loc;
             // FIXME adjust pos and:
             // FIXME take account of orientation
-            if (mloc != 0) {
-                mloc->m_contains.insert(cent);
+            if (ent_loc != 0) {
+                ent_loc->m_contains.insert(child_ent);
             }
         }
 
         m_checkIterator = m_entities.find(next);
-        std::vector<std::string>::const_iterator J;
-        for(J = m_deleteHooks.begin(); J != m_deleteHooks.end(); J++) {
+        std::vector<std::string>::const_iterator J = m_deleteHooks.begin();
+        std::vector<std::string>::const_iterator Jend = m_deleteHooks.end();
+        for(; J != Jend; ++J) {
             m_script->hook(*J, ent);
         }
-        delete ent;
+        ent->decRef();
     }
 }
 
@@ -204,29 +219,21 @@ MemEntity * MemMap::getAdd(const std::string & id)
     return addId(id);
 }
 
-void MemMap::addContents(const MapType & entmap)
+void MemMap::addContents(const RootEntity & ent)
 // Iterate over the contains attribute of a message, looking at all the contents
 {
-    MapType::const_iterator I = entmap.find("contains");
-    if (I == entmap.end()) {
+    if (!ent->hasAttrFlag(Atlas::Objects::Entity::CONTAINS_FLAG)) {
         return;
     }
-    if (!I->second.isList()) {
-        log(ERROR, "MemMap::addContents, malformed contains is not list");
-        return;
-    }
-    const ListType & contlist = I->second.asList();
-    ListType::const_iterator J = contlist.begin();
-    for(;J != contlist.end(); J++) {
-        if (!J->isString()) {
-            log(ERROR, "MemMap::addContents, malformed non-string in contains");
-            continue;
-        }
-        getAdd(J->asString());
+    const std::list<std::string> & contlist = ent->getContains();
+    std::list<std::string>::const_iterator Jend = contlist.end();
+    std::list<std::string>::const_iterator J = contlist.begin();
+    for (; J != Jend; ++J) {
+        getAdd(*J);
     }
 }
 
-MemEntity * MemMap::updateAdd(const MapType & entmap, const double & d)
+MemEntity * MemMap::updateAdd(const RootEntity & ent, const double & d)
 // Update an entity in our memory, from an Atlas message
 // The mind code relies on this function never sending a Sight to
 // be sure that seeing something created does not imply that the created
@@ -234,27 +241,22 @@ MemEntity * MemMap::updateAdd(const MapType & entmap, const double & d)
 // creator.
 {
     debug( std::cout << "MemMap::update" << std::endl << std::flush;);
-    MapType::const_iterator I = entmap.find("id");
-    if (I == entmap.end()) {
+    if (!ent->hasAttrFlag(Atlas::Objects::ID_FLAG)) {
         log(ERROR, "MemMap::update, Missing id in updated entity");
         return NULL;
     }
-    if (!I->second.isString()) {
-        log(ERROR, "MemMap::update, Malformed non-string id in updated entity");
-        return NULL;
-    }
-    const std::string & id = I->second.asString();
+    const std::string & id = ent->getId();
     if (id.empty()) {
         log(ERROR, "MemMap::update, Empty id in updated entity");
         return NULL;
     }
-    MemEntityDict::const_iterator J = m_entities.find(id);
+    MemEntityDict::const_iterator I = m_entities.find(id);
     MemEntity * entity;
-    if (J == m_entities.end()) {
-        entity = newEntity(id, entmap);
+    if (I == m_entities.end()) {
+        entity = newEntity(id, ent);
     } else {
-        entity = J->second;
-        updateEntity(entity, entmap);
+        entity = I->second;
+        updateEntity(entity, ent);
     }
     entity->update(d);
     return entity;
@@ -264,11 +266,12 @@ MemEntityVector MemMap::findByType(const std::string & what)
 // Find an entity in our memory of a certain type
 {
     MemEntityVector res;
-    MemEntityDict::const_iterator I;
-    for(I = m_entities.begin(); I != m_entities.end(); I++) {
+    
+    MemEntityDict::const_iterator Iend = m_entities.end();
+    for (MemEntityDict::const_iterator I = m_entities.begin(); I != Iend; ++I) {
         MemEntity * item = I->second;
         debug( std::cout << "F" << what << ":" << item->getType() << ":" << item->getId() << std::endl << std::flush;);
-        if (item->getType() == what) {
+        if (item->isVisible() && item->getType() == what) {
             res.push_back(I->second);
         }
     }
@@ -277,27 +280,33 @@ MemEntityVector MemMap::findByType(const std::string & what)
 
 MemEntityVector MemMap::findByLocation(const Location & loc, double radius)
 // Find an entity in our memory in a certain place
+// FIXME Don't return by value
 {
     MemEntityVector res;
-    MemEntityDict::const_iterator I;
-    for(I = m_entities.begin(); I != m_entities.end(); I++) {
+    MemEntityDict::const_iterator Iend = m_entities.end();
+    for (MemEntityDict::const_iterator I = m_entities.begin(); I != Iend; ++I) {
+        MemEntity * item = I->second;
+        if (!item->isVisible()) {
+            continue;
+        }
         const Location & oloc = I->second->m_location;
         if (!loc.isValid() || !oloc.isValid()) {
             continue;
         }
         if ((oloc.m_loc->getId() == loc.m_loc->getId()) &&
-            (squareDistance(loc.m_pos, oloc.m_pos) < (radius * radius))) {
+            (squareDistance(loc.pos(), oloc.pos()) < (radius * radius))) {
             res.push_back(I->second);
         }
     }
     return res;
 }
 
-const Element MemMap::asObject()
+const Element MemMap::asMessage()
 {
     MapType omap;
-    MemEntityDict::const_iterator I = m_entities.begin();
-    for(;I != m_entities.end(); I++) {
+    
+    MemEntityDict::const_iterator Iend = m_entities.end();
+    for (MemEntityDict::const_iterator I = m_entities.begin(); I != Iend; ++I) {
         I->second->addToMessage((omap[I->first] = MapType()).asMap());
     }
     return Element(omap);
@@ -323,10 +332,13 @@ void MemMap::check(const double & time)
             if (me->m_location.m_loc != 0) {
                 me->m_location.m_loc->m_contains.erase(me);
             }
-            // m_checkIterator = m_entities.begin();
+            
+            // FIXME This is required until MemMap uses parent refcounting
+            me->m_location.m_loc = 0;
+
             m_checkIterator = m_entities.find(next);
             // attribute of its its parent.
-            delete me;
+            me->decRef();
         } else {
             debug(std::cout << me->getId() << "|" << me->getType() << "|"
                             << me->lastSeen() << "|" << me->isVisible()
@@ -340,8 +352,11 @@ void MemMap::flush()
 {
     debug(std::cout << "Flushing memory with " << m_entities.size()
                     << " memories" << std::endl << std::flush;);
-    MemEntityDict::const_iterator I = m_entities.begin();
-    for (; I != m_entities.end(); I++) {
-        delete I->second;
+    
+    MemEntityDict::const_iterator Iend = m_entities.end();
+    for (MemEntityDict::const_iterator I = m_entities.begin(); I != Iend; ++I) {
+        // FIXME This is required until MemMap uses parent refcounting
+        I->second->m_location.m_loc = 0;
+        I->second->decRef();
     }
 }
