@@ -5,16 +5,10 @@
 #ifndef RULESETS_MEM_MAP_H
 #define RULESETS_MEM_MAP_H
 
-#include "common/types.h"
-
 #include <string>
 #include <list>
 
-namespace Atlas {
-  namespace Objects { namespace Operation {
-    class RootOperation;
-  } }
-}
+#include <Atlas/Objects/ObjectsFwd.h>
 
 class MemEntity;
 class Script;
@@ -23,6 +17,7 @@ class Location;
 typedef std::vector<MemEntity *> MemEntityVector;
 typedef std::map<std::string, MemEntity *> MemEntityDict;
 
+/// \brief Class to handle the basic entity memory of a mind
 class MemMap {
   private:
     friend class BaseMind;
@@ -36,15 +31,15 @@ class MemMap {
     Script *& m_script;
 
     MemEntity * addEntity(MemEntity *);
-    void readEntity(MemEntity *, const MapType &);
-    void updateEntity(MemEntity *, const MapType &);
-    MemEntity * newEntity(const std::string &, const MapType &);
-    void addContents(const MapType &);
+    void readEntity(MemEntity *, const Atlas::Objects::Entity::RootEntity &);
+    void updateEntity(MemEntity *, const Atlas::Objects::Entity::RootEntity &);
+    MemEntity * newEntity(const std::string &, const Atlas::Objects::Entity::RootEntity &);
+    void addContents(const Atlas::Objects::Entity::RootEntity &);
     MemEntity * addId(const std::string & id);
   public:
     explicit MemMap(Script *& s);
 
-    bool find(const std::string & id) {
+    bool find(const std::string & id) const {
         return (m_entities.find(id) != m_entities.end());
     }
 
@@ -52,16 +47,16 @@ class MemMap {
         return m_entities;
     }
 
-    Atlas::Objects::Operation::RootOperation * lookId();
+    Atlas::Objects::Operation::RootOperation lookId();
     void del(const std::string & id);
     MemEntity * get(const std::string & id) const;
     MemEntity * getAdd(const std::string & id);
-    MemEntity * updateAdd(const MapType &, const double &);
+    MemEntity * updateAdd(const Atlas::Objects::Entity::RootEntity &, const double &);
 
     MemEntityVector findByType(const std::string & what);
     MemEntityVector findByLocation(const Location & where, double radius);
 
-    const Element asObject();
+    const Atlas::Message::Element asMessage();
 
     void check(const double &);
     void flush();
