@@ -5,48 +5,50 @@
 #ifndef RULESETS_MOVEMENT_H
 #define RULESETS_MOVEMENT_H
 
-#include <physics/Vector3D.h>
-#include <physics/Quaternion.h>
+#include "physics/Vector3D.h"
+#include "physics/Quaternion.h"
+
+#include "common/types.h"
 
 class Entity;
-class Character;
 class Location;
 
-namespace Atlas { namespace Objects { namespace Operation {
-  class Move;
-} } }
-
+/// \brief Base class for handling Character movement
+///
+/// This class should be replaced by a base class for handling all entity
+/// movement.
 class Movement {
   protected:
     Entity & m_body;
     double m_lastMovementTime;
     Point3D m_targetPos;
     Point3D m_updatedPos;
-    Vector3D m_velocity;
-    Quaternion m_orient;
     int m_serialno;
     Point3D m_collPos;
     Entity * m_collEntity;
-    bool m_collRefChange;
+    bool m_collLocChange;
     Vector3D m_collNormal;
-    int m_collAxis;
 
     bool updateNeeded(const Location & location) const;
     void checkCollisions(const Location & loc);
-
-    friend class Character;
   public:
     explicit Movement(Entity & body);
     virtual ~Movement();
 
-    void reset();
-    bool moving() const;
+    int serialno() const {
+        return m_serialno;
+    }
 
-    virtual double getTickAddition(const Point3D & coordinates) const = 0;
-    virtual Atlas::Objects::Operation::Move * genFaceOperation() = 0;
-    virtual Atlas::Objects::Operation::Move * genMoveUpdate(Location *) = 0;
-    virtual Atlas::Objects::Operation::Move * genMoveOperation(Location *,
-                                                        const Location &) = 0;
+    void setTarget(const Point3D & target) {
+        m_targetPos = target;
+    }
+
+    void reset();
+
+    virtual double getTickAddition(const Point3D & coordinates,
+                                   const Vector3D & velocity) const = 0;
+    virtual int getUpdatedLocation(Location &) = 0;
+    virtual Operation generateMove(const Location&) = 0;
 };
 
 #endif // RULESETS_MOVEMENT_H
