@@ -9,17 +9,17 @@
 
 class Entity;
 
+/// \brief Class for handling movement of a Pedestrian Character
 class Pedestrian : public Movement {
     // friend class Character;
   public:
     explicit Pedestrian(Entity & body);
     virtual ~Pedestrian();
 
-    double getTickAddition(const Point3D & coordinates) const;
-    Atlas::Objects::Operation::Move * genFaceOperation();
-    Atlas::Objects::Operation::Move * genMoveUpdate(Location *);
-    Atlas::Objects::Operation::Move * genMoveOperation(Location *,
-                                                 const Location &);
+    double getTickAddition(const Point3D & coordinates,
+                           const Vector3D & velocity) const;
+    int getUpdatedLocation(Location &);
+    Operation generateMove(const Location &);
 };
 
 #endif // RULESETS_PEDESTRIAN_H
