@@ -25,6 +25,8 @@ class Plant : public Plant_parent {
     double m_sizeAdult; // chance of growing fruit
     std::string m_fruitName;
 
+    double m_nourishment;
+
     static const int m_speed = 20; // Number of basic_ticks per tick
     static const int m_minuDrop = 0; // min fruit dropped
     static const int m_maxuDrop = 2; // max fruit dropped
@@ -40,12 +42,10 @@ class Plant : public Plant_parent {
     const int getFruitChance() const { return m_fruitChance; }
     const double getSizeAdult() const { return m_sizeAdult; }
 
-    virtual bool get(const std::string &, Element &) const;
-    virtual void set(const std::string &, const Element &);
-    virtual void addToMessage(MapType & obj) const;
-
-    virtual void TickOperation(const Tick &, OpVector &);
-    virtual void TouchOperation(const Touch &, OpVector &);
+    virtual void ChopOperation(const Operation &, OpVector &);
+    virtual void NourishOperation(const Operation &, OpVector &);
+    virtual void TickOperation(const Operation &, OpVector &);
+    virtual void TouchOperation(const Operation &, OpVector &);
 };
 
 #endif // RULESETS_PLANT_H
