@@ -3,6 +3,7 @@
 // Copyright (C) 2000 Alistair Riddoch
 
 #include "Py_Mind.h"
+#include "Py_Thing.h"
 #include "Py_Map.h"
 #include "Py_Object.h"
 #include "Py_Vector3D.h"
@@ -12,6 +13,12 @@
 #include "Py_WorldTime.h"
 #include "BaseMind.h"
 
+using Atlas::Message::Element;
+using Atlas::Message::MapType;
+
+// This method is almost profoundly pointless. Remove it soon, once we
+// sort out something to do with the one placed it is used in the python
+// mind code.
 static PyObject * Mind_as_entity(PyMind * self)
 {
 #ifndef NDEBUG
@@ -30,25 +37,7 @@ static PyObject * Mind_as_entity(PyMind * self)
     return (PyObject *)ret;
 }
 
-static PyObject * Mind_get_xyz(PyMind * self)
-{
-#ifndef NDEBUG
-    if (self->m_mind == NULL) {
-        PyErr_SetString(PyExc_AssertionError, "invalid mind get_xyz");
-        return NULL;
-    }
-#endif // NDEBUG
-    PyPoint3D * ret = newPyPoint3D();
-    if (ret == NULL) {
-        return NULL;
-    }
-    ret->coords = self->m_mind->getXyz();
-    assert(ret->coords.isValid());
-    return (PyObject *)ret;
-}
-
 static PyMethodDef Mind_methods[] = {
-        {"get_xyz",        (PyCFunction)Mind_get_xyz,  METH_NOARGS},
         {"as_entity",        (PyCFunction)Mind_as_entity,  METH_NOARGS},
         {NULL,          NULL}           /* sentinel */
 };
@@ -102,6 +91,25 @@ static PyObject * Mind_getattr(PyMind *self, char *name)
         worldtime->time = self->m_mind->getTime();
         return (PyObject *)worldtime;
     }
+    if (strcmp(name, "contains") == 0) {
+        PyObject * list = PyList_New(0);
+        if (list == NULL) {
+            return NULL;
+        }
+        EntitySet::const_iterator I = self->m_mind->m_contains.begin();
+        EntitySet::const_iterator Iend = self->m_mind->m_contains.end();
+        for (; I != Iend; ++I) {
+            PyEntity * child = newPyEntity();
+            if (child == NULL) {
+                Py_DECREF(list);
+                return NULL;
+            }
+            // FIXME Do we need to increment the reference count on this?
+            child->m_entity = *I;
+            PyList_Append(list, (PyObject*)child);
+        }
+        return list;
+    }
     if (self->Mind_attr != NULL) {
         PyObject *v = PyDict_GetItemString(self->Mind_attr, name);
         if (v != NULL) {
