@@ -5,13 +5,13 @@
 #ifndef RULESETS_PY_OBJECT_H
 #define RULESETS_PY_OBJECT_H
 
-#include <Atlas/Message/Element.h>
-
 #include <Python.h>
 
+#include <Atlas/Message/Element.h>
+
+/// \brief Wrapper for Atlas::Message::Element in Python
 typedef struct {
     PyObject_HEAD
-    PyObject * Object_attr;  // Attributes dictionary
     Atlas::Message::Element * m_obj;
 } PyMessageElement;
 
@@ -32,8 +32,8 @@ PyMessageElement * newPyMessageElement();
 // PyObject * MapType_asPyObject(const Atlas::Message::MapType & map);
 // PyObject * ListType_asPyObject(const Atlas::Message::ListType & list);
 PyObject * MessageElement_asPyObject(const Atlas::Message::Element & obj);
-Atlas::Message::ListType PyListObject_asElementList(PyObject * list);
-Atlas::Message::MapType PyDictObject_asElementMap(PyObject * dict);
+Atlas::Message::Element PyListObject_asElement(PyObject * list);
+Atlas::Message::Element PyDictObject_asElement(PyObject * dict);
 Atlas::Message::Element PyObject_asMessageElement(PyObject * o);
 
 #endif // RULESETS_PY_OBJECT_H
