@@ -3,14 +3,18 @@
 // Copyright (C) 2000 Alistair Riddoch
 
 #include "Py_Operation.h"
+#include "Py_RootEntity.h"
 #include "Py_Oplist.h"
-#include "Py_Optime.h"
 #include "Py_Object.h"
 #include "Py_Thing.h"
 
-#include "common/utility.h"
+#include "common/log.h"
 
-#include <Atlas/Objects/Operation/RootOperation.h>
+using Atlas::Message::Element;
+using Atlas::Message::ListType;
+using Atlas::Objects::Root;
+using Atlas::Objects::Operation::RootOperation;
+using Atlas::Objects::Entity::RootEntity;
 
 /*
  * Beginning of Operation section.
@@ -27,7 +31,7 @@
 static PyObject * Operation_setSerialno(PyOperation * self, PyObject * args)
 {
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.setSerialno");
         return NULL;
     }
@@ -47,7 +51,7 @@ static PyObject * Operation_setRefno(PyOperation * self, PyObject * args)
 {
     // Takes integer, returns none
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.setRefno");
         return NULL;
     }
@@ -66,7 +70,7 @@ static PyObject * Operation_setFrom(PyOperation * self, PyObject * args)
 {
     // Takes string, returns none
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.setFrom");
         return NULL;
     }
@@ -85,7 +89,7 @@ static PyObject * Operation_setTo(PyOperation * self, PyObject * args)
 {
     // Takes string, returns none
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.setTo");
         return NULL;
     }
@@ -104,7 +108,7 @@ static PyObject * Operation_setSeconds(PyOperation * self, PyObject * args)
 {
     // Takes float, returns none
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.setSeconds");
         return NULL;
     }
@@ -123,7 +127,7 @@ static PyObject * Operation_setFutureSeconds(PyOperation * self, PyObject * args
 {
     // Takes float, returns none
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.setFutureSeconds");
         return NULL;
     }
@@ -138,34 +142,18 @@ static PyObject * Operation_setFutureSeconds(PyOperation * self, PyObject * args
     return Py_None;
 }
 
-static PyObject * Operation_setTimeString(PyOperation * self, PyObject * args)
-{
-    // Takes string, returns none
-#ifndef NDEBUG
-    if (self->operation == NULL) {
-        PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.setTimeString");
-        return NULL;
-    }
-#endif // NDEBUG
-    char * timestring;
-    if (!PyArg_ParseTuple(args, "s", &timestring)) {
-        return NULL;
-    }
-    self->operation->setTimeString(timestring);
-
-    Py_INCREF(Py_None);
-    return Py_None;
-}
-
 static PyObject * Operation_setArgs(PyOperation * self, PyObject * args)
 {
     // Takes List, returns none
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.setArgs");
         return NULL;
     }
 #endif // NDEBUG
+    // FIXME This is a big mess - either get rid of it in entirity, or
+    // make something better.
+
     PyObject * args_object;
     if (!PyArg_ParseTuple(args, "O", &args_object)) {
         return NULL;
@@ -180,13 +168,13 @@ static PyObject * Operation_setArgs(PyOperation * self, PyObject * args)
         if (PyMessageElement_Check(item)) {
             argslist.push_back(*((PyMessageElement*)item)->m_obj);
         } else if (PyOperation_Check(item)) {
-            argslist.push_back(((PyOperation*)item)->operation->asObject());
+            argslist.push_back(((PyOperation*)item)->operation->asMessage());
         } else {
             PyErr_SetString(PyExc_TypeError,"args contains non Atlas Object");
             return NULL;
         }
     }
-    self->operation->setArgs(argslist);
+    self->operation->setArgsAsList(argslist);
 
     Py_INCREF(Py_None);
     return Py_None;
@@ -196,7 +184,7 @@ static PyObject * Operation_getSerialno(PyOperation * self)
 {
     // Returns int
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.getSerialno");
         return NULL;
     }
@@ -208,7 +196,7 @@ static PyObject * Operation_getRefno(PyOperation * self)
 {
     // Returns int
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.getRefno");
         return NULL;
     }
@@ -220,7 +208,7 @@ static PyObject * Operation_getFrom(PyOperation * self)
 {
     // Returns string
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.getFrom");
         return NULL;
     }
@@ -232,7 +220,7 @@ static PyObject * Operation_getTo(PyOperation * self)
 {
     // Returns string
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.getTo");
         return NULL;
     }
@@ -244,7 +232,7 @@ static PyObject * Operation_getSeconds(PyOperation * self)
 {
     // Returns float
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.getSeconds");
         return NULL;
     }
@@ -256,7 +244,7 @@ static PyObject * Operation_getFutureSeconds(PyOperation * self)
 {
     // Returns float
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.getFutureSeconds");
         return NULL;
     }
@@ -264,41 +252,37 @@ static PyObject * Operation_getFutureSeconds(PyOperation * self)
     return PyFloat_FromDouble(self->operation->getFutureSeconds());
 }
 
-static PyObject * Operation_getTimeString(PyOperation * self)
-{
-    // Returns string
-#ifndef NDEBUG
-    if (self->operation == NULL) {
-        PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.getTimeString");
-        return NULL;
-    }
-#endif // NDEBUG
-    return PyString_FromString(self->operation->getTimeString().c_str());
-}
-
 static PyObject * Operation_getArgs(PyOperation * self)
 {
     // Returns list
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.getArgs");
         return NULL;
     }
 #endif // NDEBUG
     // Here we go:- 
-    ListType & args_list = self->operation->getArgs();
+
+    // FIXME
+    // Here we need a generic way to make a Root object into a python object,
+    // either by wrapping as a Operation, or as an Entity. New Entity binding
+    // thus required. Once written this code should also be used in
+    // Operation_seq_item(), and the code there is a better starting point.
+
+    const std::vector<Root> & args_list = self->operation->getArgs();
     PyObject * args_pylist = PyList_New(args_list.size());
-    ListType::const_iterator I;
     int j = 0;
     PyMessageElement * item;
-    for(I = args_list.begin();I != args_list.end(); I++, j++) {
+    std::vector<Root>::const_iterator Iend = args_list.end();
+    std::vector<Root>::const_iterator I = args_list.begin();
+    for (; I != Iend; ++I, ++j) {
         item = newPyMessageElement();
         if (item == NULL) {
             PyErr_SetString(PyExc_TypeError,"error creating list");
             Py_DECREF(args_pylist);
             return NULL;
         }
-        item->m_obj = new Element(*I);
+        item->m_obj = new Element((*I)->asMessage());
         PyList_SetItem(args_pylist, j, (PyObject *)item);
     }
     return args_pylist;
@@ -307,7 +291,7 @@ static PyObject * Operation_getArgs(PyOperation * self)
 static PyObject* Operation_get_name(PyOperation * self)
 {
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.get_name");
         return NULL;
     }
@@ -322,7 +306,7 @@ static PyObject* Operation_get_name(PyOperation * self)
 static int Operation_seq_length(PyOperation * self)
 {
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.seq_length");
         return 0;
     }
@@ -333,34 +317,35 @@ static int Operation_seq_length(PyOperation * self)
 static PyObject * Operation_seq_item(PyOperation * self, int item)
 {
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError,"NULL Operation in Operation.seq_item");
         return 0;
     }
 #endif // NDEBUG
-    ListType & args_list = self->operation->getArgs();
-    ListType::const_iterator I = args_list.begin();
-    int i;
-    for(i = 0; i < item && I != args_list.end(); i++, I++);
+    const std::vector<Root> & args_list = self->operation->getArgs();
+    std::vector<Root>::const_iterator I = args_list.begin();
+    std::vector<Root>::const_iterator Iend = args_list.end();
+    for(int i = 0; i < item && I != Iend; ++i, ++I);
     if (I == args_list.end()) {
-        PyErr_SetString(PyExc_TypeError,"Not enought op arguments");
+        PyErr_SetString(PyExc_TypeError,"Operation.[]: Not enought op arguments");
         return 0;
     }
-    if (!I->isMap()) {
-        PyErr_SetString(PyExc_TypeError,"op argument is not a map");
-        return 0;
-    }
-    const MapType & obj = I->asMap();
-    RootOperation op;
-    bool isOp = utility::Object_asOperation(obj, op);
-    if (isOp) {
+    const Root & arg = *I;
+    RootOperation op = Atlas::Objects::smart_dynamic_cast<RootOperation>(arg);
+    if (op.isValid()) {
         PyOperation * ret_op = newPyOperation();
-        ret_op->operation = new RootOperation(op);
-        ret_op->own = 1;
+        ret_op->operation = op;
         return (PyObject *)ret_op;
     }
+    RootEntity ent = Atlas::Objects::smart_dynamic_cast<RootEntity>(arg);
+    if (ent.isValid()) {
+        PyRootEntity * ret_ent = newPyRootEntity();
+        ret_ent->entity = ent;
+        return (PyObject *)ret_ent;
+    }
+    log(WARNING, "Non operation or entity being returned as arg of operation");
     PyMessageElement * ret = newPyMessageElement();
-    ret->m_obj = new Element(obj);
+    ret->m_obj = new Element(arg->asMessage());
     return (PyObject *)ret;
 }
 
@@ -368,7 +353,7 @@ static PyObject * Operation_num_add(PyOperation *self, PyObject *other)
 {
     fflush(stdout);
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError, "NULL Operation in Operation.num_add");
         fflush(stdout);
         return NULL;
@@ -378,7 +363,6 @@ static PyObject * Operation_num_add(PyOperation *self, PyObject *other)
         PyOplist * res = newPyOplist();
         res->ops = new OpVector();
         res->ops->push_back(self->operation);
-        self->own = 0;
         fflush(stdout);
         return (PyObject*)res;
     }
@@ -394,14 +378,13 @@ static PyObject * Operation_num_add(PyOperation *self, PyObject *other)
         }
         res->ops = new OpVector(*opl->ops);
         res->ops->push_back(self->operation);
-        self->own = 0;
         fflush(stdout);
         return (PyObject*)res;
     }
     if (PyOperation_Check(other)) {
         PyOperation * op = (PyOperation*)other;
 #ifndef NDEBUG
-        if (op->operation == NULL) {
+        if (!op->operation.isValid()) {
             PyErr_SetString(PyExc_AssertionError, "NULL Operation in other of Operation.num_add");
         }
 #endif // NDEBUG
@@ -411,9 +394,7 @@ static PyObject * Operation_num_add(PyOperation *self, PyObject *other)
         }
         res->ops = new OpVector();
         res->ops->push_back(op->operation);
-        op->own = 0;
         res->ops->push_back(self->operation);
-        self->own = 0;
         fflush(stdout);
         return (PyObject*)res;
     }
@@ -494,7 +475,6 @@ PyMethodDef RootOperation_methods[] = {
     {"setTo",           (PyCFunction)Operation_setTo,           METH_VARARGS},
     {"setSeconds",      (PyCFunction)Operation_setSeconds,      METH_VARARGS},
     {"setFutureSeconds",(PyCFunction)Operation_setFutureSeconds,METH_VARARGS},
-    {"setTimeString",   (PyCFunction)Operation_setTimeString,   METH_VARARGS},
     {"setArgs",         (PyCFunction)Operation_setArgs,         METH_VARARGS},
     {"getSerialno",     (PyCFunction)Operation_getSerialno,     METH_NOARGS},
     {"getRefno",        (PyCFunction)Operation_getRefno,        METH_NOARGS},
@@ -502,7 +482,6 @@ PyMethodDef RootOperation_methods[] = {
     {"getTo",           (PyCFunction)Operation_getTo,           METH_NOARGS},
     {"getSeconds",      (PyCFunction)Operation_getSeconds,      METH_NOARGS},
     {"getFutureSeconds",(PyCFunction)Operation_getFutureSeconds,METH_NOARGS},
-    {"getTimeString",   (PyCFunction)Operation_getTimeString,   METH_NOARGS},
     {"getArgs",         (PyCFunction)Operation_getArgs,         METH_NOARGS},
     {"get_name",        (PyCFunction)Operation_get_name,        METH_NOARGS},
     {NULL,          NULL}
@@ -515,7 +494,6 @@ PyMethodDef ConstRootOperation_methods[] = {
     {"getTo",           (PyCFunction)Operation_getTo,           METH_NOARGS},
     {"getSeconds",      (PyCFunction)Operation_getSeconds,      METH_NOARGS},
     {"getFutureSeconds",(PyCFunction)Operation_getFutureSeconds,METH_NOARGS},
-    {"getTimeString",   (PyCFunction)Operation_getTimeString,   METH_NOARGS},
     {"getArgs",         (PyCFunction)Operation_getArgs,         METH_NOARGS},
     {"get_name",        (PyCFunction)Operation_get_name,        METH_NOARGS},
     {NULL,          NULL}
@@ -528,11 +506,8 @@ PyMethodDef ConstRootOperation_methods[] = {
 
 static void Operation_dealloc(PyOperation *self)
 {
-        if ((self->own != 0) && (self->operation != NULL)) {
-            // Can't delete until I have sorted out bugs with own flag
-            delete self->operation;
-        }
-        PyMem_DEL(self);
+    self->operation.~RootOperation();
+    PyMem_DEL(self);
 }
 
 static inline PyObject * findMethod(PyOperation * self, char * name)
@@ -545,72 +520,26 @@ static inline PyObject * findMethod(PyConstOperation * self, char * name)
     return Py_FindMethod(ConstRootOperation_methods, (PyObject *)self, name);
 }
 
-static inline PyObject * handleTime(PyOperation * self)
-{
-    PyOptime * time_obj = newPyOptime();
-    if (time_obj == NULL) {
-        return NULL;
-    }
-    time_obj->operation = self->operation;
-    return (PyObject *)time_obj;
-}
-
-static inline PyObject * handleTime(PyConstOperation * self)
-{
-    // If it becomes necessary for python scripts to query the time
-    // of ops, this can be adapted to provide them with that information
-    PyErr_SetString(PyExc_TypeError, "cannot get time on const ops");
-    return NULL;
-}
-
 template <typename T>
 static PyObject * getattr(T * self, char * name)
 {
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError, "NULL Operation in Operation.getattr");
         return NULL;
     }
 #endif // NDEBUG
     if (strcmp(name, "from_") == 0) {
-        if (self->from != NULL) {
-            PyEntity * thing_obj = newPyEntity();
-            if (thing_obj == NULL) {
-                return NULL;
-            }
-            thing_obj->m_entity = self->from;
-            return (PyObject *)thing_obj;
-        } else {
-            PyMessageElement * obj = newPyMessageElement();
-            MapType omap;
-            omap["id"] = Element(self->operation->getFrom());
-            obj->m_obj = new Element(omap);
-            return (PyObject *)obj;
-        }
+        return PyString_FromString(self->operation->getFrom().c_str());
     } else if (strcmp(name, "to") == 0) {
-        if (self->to != NULL) {
-            PyEntity * thing_obj = newPyEntity();
-            if (thing_obj == NULL) {
-                return NULL;
-            }
-            thing_obj->m_entity = self->to;
-            return (PyObject *)thing_obj;
-        } else {
-            PyMessageElement * obj = newPyMessageElement();
-            MapType omap;
-            omap["id"] = Element(self->operation->getTo());
-            obj->m_obj = new Element(omap);
-            return (PyObject *)obj;
-        }
-    } else if (strcmp(name, "time") == 0) {
-        return handleTime(self);
+        return PyString_FromString(self->operation->getTo().c_str());
     } else if (strcmp(name, "id") == 0) {
-        const ListType & parents = self->operation->getParents();
-        if ((parents.empty()) || (!parents.front().isString())) {
+        const std::list<std::string> & parents = self->operation->getParents();
+        if (parents.empty()) {
             PyErr_SetString(PyExc_TypeError, "Operation has no parents");
             return NULL;
         }
-        return PyString_FromString(parents.front().asString().c_str());
+        return PyString_FromString(parents.front().c_str());
     }
     return findMethod(self, name);
 }
@@ -629,7 +558,7 @@ static PyObject * ConstOperation_getattr(PyConstOperation * self, char * name)
 static int Operation_setattr(PyOperation *self, char *name, PyObject *v)
 {
 #ifndef NDEBUG
-    if (self->operation == NULL) {
+    if (!self->operation.isValid()) {
         PyErr_SetString(PyExc_AssertionError, "NULL Operation in Operation.setattr");
         return -1;
     }
@@ -643,10 +572,6 @@ static int Operation_setattr(PyOperation *self, char *name, PyObject *v)
             }
             return -1;
         }
-        if (((PyTypeObject*)PyObject_Type(v) == &PyEntity_Type) &&
-            (((PyEntity *)v)->m_entity != NULL)) {
-            self->from = ((PyEntity *)v)->m_entity;
-        }
         self->operation->setFrom(PyString_AsString(thing_id));
         Py_DECREF(thing_id);
         return 0;
@@ -660,10 +585,6 @@ static int Operation_setattr(PyOperation *self, char *name, PyObject *v)
             }
             return -1;
         }
-        if (((PyTypeObject*)PyObject_Type(v) == &PyEntity_Type) &&
-            (((PyEntity *)v)->m_entity != NULL)) {
-            self->to = ((PyEntity *)v)->m_entity;
-        }
         self->operation->setTo(PyString_AsString(thing_id));
         Py_DECREF(thing_id);
         return 0;
@@ -720,10 +641,7 @@ PyOperation * newPyOperation()
     if (self == NULL) {
         return NULL;
     }
-    self->operation = NULL;
-    self->from = NULL;
-    self->to = NULL;
-    self->own = 0;
+    new (&(self->operation)) RootOperation(NULL);
     return self;
 }
 
@@ -734,9 +652,6 @@ PyConstOperation * newPyConstOperation()
     if (self == NULL) {
         return NULL;
     }
-    self->operation = NULL;
-    self->from = NULL;
-    self->to = NULL;
-    self->own = 0;
+    new (&(self->operation)) RootOperation(NULL);
     return self;
 }
