@@ -5,30 +5,23 @@
 #ifndef RULESETS_PY_OPERATION_H
 #define RULESETS_PY_OPERATION_H
 
-#include "common/operations.h"
-
 #include <Python.h>
 
-namespace Atlas { namespace Objects { namespace Operation {
-  class RootOperation;
-} } }
+#include <Atlas/Objects/RootOperation.h>
+#include <Atlas/Objects/SmartPtr.h>
 
 class Entity;
 
+/// \brief Wrapper for all Atlas operations in Python
 typedef struct {
     PyObject_HEAD
-    Atlas::Objects::Operation::RootOperation * operation;
-    int own;
-    Entity * from;
-    Entity * to;
+    Atlas::Objects::Operation::RootOperation operation;
 } PyOperation;
 
+/// \brief Wrapper for read only Atlas operations in Python
 typedef struct {
     PyObject_HEAD
-    const Atlas::Objects::Operation::RootOperation * operation;
-    int own;
-    Entity * from;
-    Entity * to;
+    Atlas::Objects::Operation::RootOperation operation;
 } PyConstOperation;
 
 extern PyTypeObject PyOperation_Type;
