@@ -19,7 +19,6 @@ static PyObject* Oplist_append(PyOplist * self, PyObject * args)
     }
     if (PyOperation_Check(op)) {
         self->ops->push_back(op->operation);
-        op->own = 0;
     } else if ((PyObject*)op != Py_None) {
         PyErr_SetString(PyExc_TypeError, "Append must be an op");
         return NULL;
@@ -67,7 +66,8 @@ static PyObject * Oplist_num_add(PyOplist *self, PyObject *other)
             return NULL;
         }
         res->ops = new OpVector(*self->ops);
-        for(OpVector::const_iterator I = opl->ops->begin(); I != opl->ops->end(); I++) {
+        OpVector::const_iterator Iend = opl->ops->end();
+        for (OpVector::const_iterator I = opl->ops->begin(); I != Iend; ++I) {
             res->ops->push_back(*I);
         }
         return (PyObject*)res;
@@ -75,7 +75,7 @@ static PyObject * Oplist_num_add(PyOplist *self, PyObject *other)
     if (PyOperation_Check(other)) {
         PyOperation * op = (PyOperation*)other;
 #ifndef NDEBUG
-        if (op->operation == NULL) {
+        if (!op->operation.isValid()) {
             PyErr_SetString(PyExc_AssertionError, "NULL Operation in other of Oplist.num_add");
         }
 #endif // NDEBUG
@@ -86,7 +86,6 @@ static PyObject * Oplist_num_add(PyOplist *self, PyObject *other)
         res->ops = new OpVector(*self->ops);
         // res->ops->merge(*self->ops);
         res->ops->push_back(op->operation);
-        op->own = 0;
         return (PyObject*)res;
     }
     return NULL;
