@@ -10,6 +10,9 @@
 #include "Py_World.h"
 #include "Entity.h"
 
+using Atlas::Message::Element;
+using Atlas::Message::MapType;
+
 static PyObject * Entity_as_entity(PyEntity * self)
 {
 #ifndef NDEBUG
@@ -27,25 +30,7 @@ static PyObject * Entity_as_entity(PyEntity * self)
     return (PyObject *)ret;
 }
 
-static PyObject * Entity_get_xyz(PyEntity * self)
-{
-#ifndef NDEBUG
-    if (self->m_entity == NULL) {
-        PyErr_SetString(PyExc_AssertionError, "invalid entity get_xyz");
-        return NULL;
-    }
-#endif // NDEBUG
-    PyPoint3D * ret = newPyPoint3D();
-    if (ret == NULL) {
-        return NULL;
-    }
-    ret->coords = self->m_entity->getXyz();
-    assert(ret->coords.isValid());
-    return (PyObject *)ret;
-}
-
 static PyMethodDef Entity_methods[] = {
-        {"get_xyz",        (PyCFunction)Entity_get_xyz,  METH_NOARGS},
         {"as_entity",        (PyCFunction)Entity_as_entity,  METH_NOARGS},
         {NULL,          NULL}           /* sentinel */
 };
@@ -104,6 +89,25 @@ static PyObject * Entity_getattr(PyEntity *self, char *name)
         world->world = self->m_entity->m_world;
         return (PyObject *)world;
     }
+    if (strcmp(name, "contains") == 0) {
+        PyObject * list = PyList_New(0);
+        if (list == NULL) {
+            return NULL;
+        }
+        EntitySet::const_iterator I = self->m_entity->m_contains.begin();
+        EntitySet::const_iterator Iend = self->m_entity->m_contains.end();
+        for (; I != Iend; ++I) {
+            PyEntity * child = newPyEntity();
+            if (child == NULL) {
+                Py_DECREF(list);
+                return NULL;
+            }
+            // FIXME Do we need to increment the reference count on this?
+            child->m_entity = *I;
+            PyList_Append(list, (PyObject*)child);
+        }
+        return list;
+    }
     if (self->Entity_attr != NULL) {
         PyObject *v = PyDict_GetItemString(self->Entity_attr, name);
         if (v != NULL) {
