@@ -105,7 +105,12 @@ static PyObject * Vector3D_unit_vector(PyVector3D * self)
         return NULL;
     }
     ret->coords = self->coords;
-    ret->coords.normalize();
+    WFMath::CoordType the_mag = ret->coords.mag();
+    if (!the_mag > 0) {
+        PyErr_SetString(PyExc_ZeroDivisionError, "Attempt to normalize a vector with zero magnitude");
+        return NULL;
+    }
+    ret->coords /= the_mag;
     return (PyObject *)ret;
 }
 
@@ -116,7 +121,7 @@ static PyObject *Vector3D_unit_vector_to(PyVector3D * self, PyObject * args)
         return NULL;
     }
     if (!PyVector3D_Check(other)) {
-        PyErr_SetString(PyExc_TypeError, "Can get unit vector to Vector3D");
+        PyErr_SetString(PyExc_TypeError, "Argument must be a Vector3D");
         return NULL;
     }
     PyVector3D * ret = newPyVector3D();
@@ -124,7 +129,12 @@ static PyObject *Vector3D_unit_vector_to(PyVector3D * self, PyObject * args)
         return NULL;
     }
     ret->coords = (other->coords - self->coords);
-    ret->coords.normalize();
+    WFMath::CoordType the_mag = ret->coords.mag();
+    if (!the_mag > 0) {
+        PyErr_SetString(PyExc_ZeroDivisionError, "Attempt to normalize a vector with zero magnitude");
+        return NULL;
+    }
+    ret->coords /= the_mag;
     return (PyObject *)ret;
 }
 
@@ -149,6 +159,21 @@ static void Vector3D_dealloc(PyVector3D *self)
     PyMem_DEL(self);
 }
 
+static int Vector3D_print(PyVector3D * self, FILE * fp, int)
+{
+    // if (flags & Py_PRINT_RAW) {
+    // }
+    fprintf(fp, "(%lf %lf %lf", self->coords.x(), self->coords.y(), self->coords.z());
+    return 0;
+}
+
+static PyObject* Vector3D_repr(PyVector3D * self)
+{
+	char buf[64];
+	::snprintf(buf, 64, "(%f, %f, %f)", self->coords.x(), self->coords.y(), self->coords.z());
+	return PyString_FromString(buf);
+}
+
 static PyObject * Vector3D_getattr(PyVector3D *self, char *name)
 {
     //if (!self->coords) {
@@ -164,6 +189,25 @@ static PyObject * Vector3D_getattr(PyVector3D *self, char *name)
 
 static int Vector3D_setattr(PyVector3D *self, char *name, PyObject *v)
 {
+    float val;
+    if (PyInt_Check(v)) {
+        val = PyInt_AsLong(v);
+    } else if (PyFloat_Check(v)) {
+        val = PyFloat_AsDouble(v);
+    } else {
+        PyErr_SetString(PyExc_TypeError, "Vector3D attributes must be numeric");
+        return -1;
+    }
+    if (strcmp(name, "x") == 0) {
+        self->coords.x() = val;
+    } else if (strcmp(name, "y") == 0) {
+        self->coords.y() = val;
+    } else if (strcmp(name, "z") == 0) {
+        self->coords.z() = val;
+    } else {
+        PyErr_SetString(PyExc_AttributeError, "Vector3D attribute does not exist");
+        return -1;
+    }
     return 0;
 }
 
@@ -285,11 +329,11 @@ PyTypeObject PyVector3D_Type = {
         0,                              /*tp_itemsize*/
         /* methods */
         (destructor)Vector3D_dealloc,   /*tp_dealloc*/
-        0,                              /*tp_print*/
+        (printfunc)Vector3D_print,      /*tp_print*/
         (getattrfunc)Vector3D_getattr,  /*tp_getattr*/
         (setattrfunc)Vector3D_setattr,  /*tp_setattr*/
         (cmpfunc)Vector3D_compare,      /*tp_compare*/
-        0,                              /*tp_repr*/
+        (reprfunc)Vector3D_repr,        /*tp_repr*/
         &Vector3D_num,                  /*tp_as_number*/
         0,                              /*tp_as_sequence*/
         0,                              /*tp_as_mapping*/
