@@ -9,14 +9,17 @@
 
 class BaseMind;
 
+/// \brief Script class for Python scripts attached to a character Mind
 class PythonMindScript : public PythonScript {
   private:
     BaseMind & mind;
   public:
     PythonMindScript(PyObject *, BaseMind &);
     virtual ~PythonMindScript();
-    virtual bool Operation(const std::string &, const RootOperation &,
-                           OpVector &, RootOperation * sub_op = 0);
+    virtual bool operation(const std::string &,
+                           const Atlas::Objects::Operation::RootOperation &,
+                           OpVector &,
+                           const Atlas::Objects::Operation::RootOperation * = 0);
     virtual void hook(const std::string &, Entity *);
 };
 
