@@ -5,12 +5,13 @@
 #ifndef RULESETS_PYTHON_SCRIPT_H
 #define RULESETS_PYTHON_SCRIPT_H
 
-#include "Script.h"
-
 #include <Python.h>
 
+#include "Script.h"
+
 class Entity;
 
+/// \brief Base Script class for Python scripts
 class PythonScript : public Script {
   protected:
     PyObject * scriptObject;
@@ -18,8 +19,10 @@ class PythonScript : public Script {
   public:
     PythonScript(PyObject *, Entity &);
     virtual ~PythonScript();
-    virtual bool Operation(const std::string &, const RootOperation &,
-                           OpVector &, RootOperation * sub_op = 0) = 0;
+    virtual bool operation(const std::string &,
+                           const Atlas::Objects::Operation::RootOperation &,
+                           OpVector &,
+                           const Atlas::Objects::Operation::RootOperation * = 0) = 0;
     virtual void hook(const std::string &, Entity *) = 0;
 };
 
