@@ -9,14 +9,17 @@
 
 class Entity;
 
+/// \brief Script class for Python scripts attached to an Entity
 class PythonEntityScript : public PythonScript {
   private:
     Entity & m_entity;
   public:
     PythonEntityScript(PyObject *, Entity &);
     virtual ~PythonEntityScript();
-    virtual bool Operation(const std::string &, const RootOperation &,
-                           OpVector &, RootOperation * sub_op = 0);
+    virtual bool operation(const std::string &,
+                           const Atlas::Objects::Operation::RootOperation &,
+                           OpVector &,
+                           const Atlas::Objects::Operation::RootOperation * = 0);
     virtual void hook(const std::string &, Entity *);
 };
 
