@@ -1,9 +1,13 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2000 Alistair Riddoch
+// Copyright (C) 2000-2005 Alistair Riddoch
+
+#include "Python.h"
 
 #include "Python_API.h"
+#include "Python_Script_Utils.h"
 
+#include "Py_BBox.h"
 #include "Py_Object.h"
 #include "Py_Thing.h"
 #include "Py_Mind.h"
@@ -15,8 +19,8 @@
 #include "Py_WorldTime.h"
 #include "Py_World.h"
 #include "Py_Operation.h"
+#include "Py_RootEntity.h"
 #include "Py_Oplist.h"
-#include "Py_Optime.h"
 
 #include "PythonThingScript.h"
 #include "PythonMindScript.h"
@@ -24,34 +28,23 @@
 #include "Entity.h"
 #include "BaseMind.h"
 
+#include "common/inheritance.h"
 #include "common/globals.h"
 #include "common/const.h"
 #include "common/debug.h"
 #include "common/log.h"
 
-#include "common/Tick.h"
-#include "common/Burn.h"
-#include "common/Chop.h"
-#include "common/Cut.h"
-#include "common/Setup.h"
-#include "common/Eat.h"
-#include "common/Nourish.h"
+#include <Atlas/Objects/Operation.h>
+#include <Atlas/Objects/Anonymous.h>
 
-#include "common/Generic.h"
-
-#include <Atlas/Objects/Operation/Create.h>
-#include <Atlas/Objects/Operation/Look.h>
-#include <Atlas/Objects/Operation/Move.h>
-#include <Atlas/Objects/Operation/Set.h>
-#include <Atlas/Objects/Operation/Sight.h>
-#include <Atlas/Objects/Operation/Talk.h>
-#include <Atlas/Objects/Operation/Touch.h>
-#include <Atlas/Objects/Operation/Info.h>
-
-#include <cstdio>
+using Atlas::Message::Element;
+using Atlas::Objects::Root;
+using Atlas::Objects::Operation::RootOperation;
+using Atlas::Objects::Entity::Anonymous;
 
 static const bool debug_flag = false;
 
+/// \brief Python wrapper for C++ functions to be exposed to Python
 typedef struct {
     PyObject_HEAD
 } FunctionObject;
@@ -91,6 +84,11 @@ PyTypeObject log_debug_type = {
 // Logger replaces sys.stdout and sys.stderr so the nothing goes to output
 //////////////////////////////////////////////////////////////////////////
 
+/// \brief Python struct to handle output from python scripts
+///
+/// In instance of this struct is used to replace sys.stdout and sys.stderr
+/// in the Python interpreter so that all script output goes to the cyphesis
+/// log subsystem
 typedef struct {
     PyObject_HEAD
 } PyLogger;
@@ -111,7 +109,7 @@ static void python_log(LogLevel lvl, const char * msg)
     }
 }
 
-static PyObject * PyLogger_write(PyObject * self, PyObject * args)
+static PyObject * PyOutLogger_write(PyObject * self, PyObject * args)
 {
     char * mesg;
     if (!PyArg_ParseTuple(args, "s", &mesg)) {
@@ -124,7 +122,7 @@ static PyObject * PyLogger_write(PyObject * self, PyObject * args)
     return Py_None;
 }
 
-static PyObject * PyErrorLogger_write(PyObject * self, PyObject * args)
+static PyObject * PyErrLogger_write(PyObject * self, PyObject * args)
 {
     char * mesg;
     if (!PyArg_ParseTuple(args, "s", &mesg)) {
@@ -137,14 +135,14 @@ static PyObject * PyErrorLogger_write(PyObject * self, PyObject * args)
     return Py_None;
 }
 
-static PyMethodDef PyLogger_methods[] = {
-    {"write",     PyLogger_write,       METH_VARARGS},
-    {NULL,        NULL}
+static PyMethodDef PyOutLogger_methods[] = {
+    {"write",       PyOutLogger_write,          METH_VARARGS},
+    {NULL,          NULL}                       /* Sentinel */
 };
 
-static PyMethodDef PyErrorLogger_methods[] = {
-    {"write",     PyErrorLogger_write,       METH_VARARGS},
-    {NULL,        NULL}
+static PyMethodDef PyErrLogger_methods[] = {
+    {"write",       PyErrLogger_write,          METH_VARARGS},
+    {NULL,          NULL}                       /* Sentinel */
 };
 
 static void PyLogger_dealloc(PyObject * self)
@@ -152,26 +150,26 @@ static void PyLogger_dealloc(PyObject * self)
     PyMem_DEL(self);
 }
 
-static PyObject * PyLogger_getattr(PyObject * self, char *name)
+static PyObject * PyOutLogger_getattr(PyObject * self, char *name)
 {
-    return Py_FindMethod(PyLogger_methods, self, name);
+    return Py_FindMethod(PyOutLogger_methods, self, name);
 }
 
-static PyObject * PyErrorLogger_getattr(PyObject * self, char *name)
+static PyObject * PyErrLogger_getattr(PyObject * self, char *name)
 {
-    return Py_FindMethod(PyErrorLogger_methods, self, name);
+    return Py_FindMethod(PyErrLogger_methods, self, name);
 }
 
-PyTypeObject PyLogger_Type = {
+PyTypeObject PyOutLogger_Type = {
         PyObject_HEAD_INIT(&PyType_Type)
         0,                   // ob_size
-        "Logger",            // tp_name
+        "OutLogger",         // tp_name
         sizeof(PyLogger),    // tp_basicsize
         0,                   // tp_itemsize
         //  methods 
         PyLogger_dealloc,    // tp_dealloc
         0,                   // tp_print
-        PyLogger_getattr,    // tp_getattr
+        PyOutLogger_getattr, // tp_getattr
         0,                   // tp_setattr
         0,                   // tp_compare
         0,                   // tp_repr
@@ -181,16 +179,16 @@ PyTypeObject PyLogger_Type = {
         0,                   // tp_hash
 };
 
-PyTypeObject PyErrorLogger_Type = {
+PyTypeObject PyErrLogger_Type = {
         PyObject_HEAD_INIT(&PyType_Type)
         0,                   // ob_size
-        "ErrorLogger",            // tp_name
+        "ErrLogger",         // tp_name
         sizeof(PyLogger),    // tp_basicsize
         0,                   // tp_itemsize
         //  methods 
         PyLogger_dealloc,    // tp_dealloc
         0,                   // tp_print
-        PyErrorLogger_getattr,    // tp_getattr
+        PyErrLogger_getattr, // tp_getattr
         0,                   // tp_setattr
         0,                   // tp_compare
         0,                   // tp_repr
@@ -323,46 +321,47 @@ PyTypeObject dictlist_add_value_type = {
 };
 
 static PyObject * Get_PyClass(const std::string & package,
-                              const std::string & _type)
+                              const std::string & type)
 {
-    std::string type = _type;
-    type[0] = toupper(type[0]);
+    std::string classname(type);
+    classname[0] = toupper(classname[0]);
     PyObject * package_name = PyString_FromString((char *)package.c_str());
-    PyObject * mod_dict = PyImport_Import(package_name);
+    PyObject * module = PyImport_Import(package_name);
     Py_DECREF(package_name);
-    if (mod_dict == NULL) {
+    if (module == NULL) {
         std::string msg = std::string("Missing python module ") + package;
         log(ERROR, msg.c_str());
         PyErr_Print();
         return NULL;
     }
-    PyObject * my_class = PyObject_GetAttrString(mod_dict, (char *)type.c_str());
-    Py_DECREF(mod_dict);
-    if (my_class == NULL) {
-        std::string msg = std::string("Missing class ") + package + "." + type;
+    PyObject * pyClass = PyObject_GetAttrString(module, (char *)classname.c_str());
+    Py_DECREF(module);
+    if (pyClass == NULL) {
+        std::string msg = std::string("Could not find python class ")
+                        + package + "." + classname;
         log(ERROR, msg.c_str());
         PyErr_Print();
         return NULL;
     }
-    if (PyCallable_Check(my_class) == 0) {
+    if (PyCallable_Check(pyClass) == 0) {
         std::string msg = std::string("Could not instance python class ")
-                        + package + "." + type;
+                        + package + "." + classname;
         log(ERROR, msg.c_str());
-        Py_DECREF(my_class);
+        Py_DECREF(pyClass);
         return NULL;
     }
 #if 0
     // In later versions of python using PyType_* will become the right thing to do.
-    if (PyType_Check(my_class) == 0) {
+    if (PyType_Check(pyClass) == 0) {
         std::cerr << "PyCallable_Check returned true, but PyType_Check returned false " << package << "." << type << std::endl << std::flush;
     } else {
         std::cerr << "PyType_Check returned true" << std::endl << std::flush;
     }
 #endif
-    return my_class;
+    return pyClass;
 }
 
-static PyObject * Create_PyScript(PyObject * pyEntity, PyObject * pyClass)
+PyObject * Create_PyScript(PyObject * pyEntity, PyObject * pyClass)
 {
     PyObject * pyob = PyEval_CallFunction(pyClass,"(O)", pyEntity);
     
@@ -374,15 +373,14 @@ static PyObject * Create_PyScript(PyObject * pyEntity, PyObject * pyClass)
             PyErr_Print();
         }
     }
-    Py_DECREF(pyClass);
     Py_DECREF(pyEntity);
     return pyob;
 }
 
-template<class T>
-void Subscribe_Script(T * entity, PyObject * pyclass,
+void Subscribe_Script(Entity * entity, PyObject * pyclass,
                       const std::string& package)
 {
+#if 0
     PyObject * dmap = PyObject_GetAttrString(pyclass, "__dict__");
     if (dmap == NULL) {
         std::string msg = std::string( "Python class for ") + package
@@ -397,6 +395,9 @@ void Subscribe_Script(T * entity, PyObject * pyclass,
         return;
     }
     PyObject * keys = PyDict_Keys(dmap);
+#else
+    PyObject * keys = PyObject_Dir(pyclass);
+#endif
     if (keys == NULL) {
         std::string msg = std::string("Error getting attribute list of Python class for ") + package;
         log(ERROR, msg.c_str());
@@ -420,28 +421,31 @@ void Subscribe_Script(T * entity, PyObject * pyclass,
 }
 
 void Create_PyEntity(Entity * entity, const std::string & package,
-                                    const std::string & _type)
+                                      const std::string & type)
 {
-    PyObject * c = Get_PyClass(package, _type);
-    if (c == NULL) { return; }
+    PyObject * pyClass = Get_PyClass(package, type);
+    if (pyClass == NULL) { return; }
     PyEntity * pyEntity = newPyEntity();
     pyEntity->m_entity = entity;
-    Subscribe_Script(entity, c, package);
-    PyObject * o = Create_PyScript((PyObject *)pyEntity, c);
+    Subscribe_Script(entity, pyClass, package);
+    PyObject * o = Create_PyScript((PyObject *)pyEntity, pyClass);
+    Py_DECREF(pyClass);
+
     if (o != NULL) {
         entity->setScript(new PythonEntityScript(o, *entity));
     }
 }
 
 void Create_PyMind(BaseMind * mind, const std::string & package,
-                                    const std::string & _type)
+                                    const std::string & type)
 {
-    PyObject * c = Get_PyClass(package, _type);
-    if (c == NULL) { return; }
+    PyObject * pyClass = Get_PyClass(package, type);
+    if (pyClass == NULL) { return; }
     PyMind * pyMind = newPyMind();
     pyMind->m_mind = mind;
-    Subscribe_Script(mind, c, package);
-    PyObject * o = Create_PyScript((PyObject *)pyMind, c);
+    Subscribe_Script(mind, pyClass, package);
+    PyObject * o = Create_PyScript((PyObject *)pyMind, pyClass);
+    Py_DECREF(pyClass);
 
     if (o != NULL) {
         mind->setScript(new PythonMindScript(o, *mind));
@@ -462,26 +466,6 @@ static PyObject * is_location(PyObject * self, PyObject * args)
     return Py_False;
 }
 
-static PyObject * distance_to(PyObject * self, PyObject * args)
-{
-    PyObject * near, * other;
-    if (!PyArg_ParseTuple(args, "OO", &near, &other)) {
-        return NULL;
-    }
-    if (!PyLocation_Check(near) || !PyLocation_Check(other)) {
-        PyErr_SetString(PyExc_TypeError, "Arg Location required");
-        return NULL;
-    }
-    PyLocation * sloc = (PyLocation *)near,
-               * oloc = (PyLocation *)other;
-    PyVector3D * ret = newPyVector3D();
-    if (ret == NULL) {
-        return NULL;
-    }
-    ret->coords = distanceTo(*sloc->location, *oloc->location);
-    return (PyObject *)ret;
-}
-
 static PyObject * location_new(PyObject * self, PyObject * args)
 {
     PyLocation *o;
@@ -557,6 +541,74 @@ static PyObject * location_new(PyObject * self, PyObject * args)
     return (PyObject *)o;
 }
 
+static PyObject * distance_to(PyObject * self, PyObject * args)
+{
+    PyObject * near, * other;
+    if (!PyArg_ParseTuple(args, "OO", &near, &other)) {
+        return NULL;
+    }
+    if (!PyLocation_Check(near) || !PyLocation_Check(other)) {
+        PyErr_SetString(PyExc_TypeError, "Arg Location required");
+        return NULL;
+    }
+    PyLocation * sloc = (PyLocation *)near,
+               * oloc = (PyLocation *)other;
+#ifndef NDEBUG
+    if (sloc->location == NULL || oloc == NULL) {
+        PyErr_SetString(PyExc_AssertionError, "Null location pointer");
+        return NULL;
+    }
+#endif // NDEBUG
+    PyVector3D * ret = newPyVector3D();
+    if (ret == NULL) {
+        return NULL;
+    }
+    ret->coords = distanceTo(*sloc->location, *oloc->location);
+    return (PyObject *)ret;
+}
+
+static PyObject * square_distance(PyObject * self, PyObject * args)
+{
+    PyObject * near, * other;
+    if (!PyArg_ParseTuple(args, "OO", &near, &other)) {
+        return NULL;
+    }
+    if (!PyLocation_Check(near) || !PyLocation_Check(other)) {
+        PyErr_SetString(PyExc_TypeError, "Arg Location required");
+        return NULL;
+    }
+    PyLocation * sloc = (PyLocation *)near,
+               * oloc = (PyLocation *)other;
+#ifndef NDEBUG
+    if (sloc->location == NULL || oloc == NULL) {
+        PyErr_SetString(PyExc_AssertionError, "Null location pointer");
+        return NULL;
+    }
+#endif // NDEBUG
+    return PyFloat_FromDouble(squareDistance(*sloc->location, *oloc->location));
+}
+
+static PyObject * square_horizontal_distance(PyObject * self, PyObject * args)
+{
+    PyObject * near, * other;
+    if (!PyArg_ParseTuple(args, "OO", &near, &other)) {
+        return NULL;
+    }
+    if (!PyLocation_Check(near) || !PyLocation_Check(other)) {
+        PyErr_SetString(PyExc_TypeError, "Arg Location required");
+        return NULL;
+    }
+    PyLocation * sloc = (PyLocation *)near,
+               * oloc = (PyLocation *)other;
+#ifndef NDEBUG
+    if (sloc->location == NULL || oloc == NULL) {
+        PyErr_SetString(PyExc_AssertionError, "Null location pointer");
+        return NULL;
+    }
+#endif // NDEBUG
+    return PyFloat_FromDouble(squareHorizontalDistance(*sloc->location, *oloc->location));
+}
+
 static PyObject * vector3d_new(PyObject * self, PyObject * args)
 {
         PyVector3D *o;
@@ -578,6 +630,13 @@ static PyObject * vector3d_new(PyObject * self, PyObject * args)
                         val[i] = (float)PyInt_AsLong(item);
                     } else if (PyFloat_Check(item)) {
                         val[i] = PyFloat_AsDouble(item);
+                    } else if (PyMessageElement_Check(item)) {
+                        PyMessageElement * mitem = (PyMessageElement*)item;
+                        if (!mitem->m_obj->isNum()) {
+                            PyErr_SetString(PyExc_TypeError, "Vector3D() must take list of floats, or ints");
+                            return NULL;
+                        }
+                        val[i] = mitem->m_obj->asNum();
                     } else {
                         PyErr_SetString(PyExc_TypeError, "Vector3D() must take list of floats, or ints");
                         return NULL;
@@ -634,6 +693,13 @@ static PyObject * point3d_new(PyObject * self, PyObject * args)
                         val[i] = (float)PyInt_AsLong(item);
                     } else if (PyFloat_Check(item)) {
                         val[i] = PyFloat_AsDouble(item);
+                    } else if (PyMessageElement_Check(item)) {
+                        PyMessageElement * mitem = (PyMessageElement*)item;
+                        if (!mitem->m_obj->isNum()) {
+                            PyErr_SetString(PyExc_TypeError, "Point3D() must take list of floats, or ints");
+                            return NULL;
+                        }
+                        val[i] = mitem->m_obj->asNum();
                     } else {
                         PyErr_SetString(PyExc_TypeError, "Point3D() must take list of floats, or ints");
                         return NULL;
@@ -669,6 +735,79 @@ static PyObject * point3d_new(PyObject * self, PyObject * args)
         return (PyObject *)o;
 }
 
+static PyObject * bbox_new(PyObject * self, PyObject * args)
+{
+    std::vector<float> val;
+
+    PyObject * clist;
+    int tuple_size = PyTuple_Size(args);
+    int clist_size;
+    switch(tuple_size) {
+        case 0:
+            break;
+        case 1:
+            clist = PyTuple_GetItem(args, 0);
+            clist_size = PyList_Size(clist);
+            if (!PyList_Check(clist) || (clist_size != 3 && clist_size != 6)) {
+                PyErr_SetString(PyExc_TypeError, "BBox() from single value must a list 3 or 6 long");
+                return NULL;
+            }
+            
+            val.resize(clist_size);
+            for(int i = 0; i < clist_size; i++) {
+                PyObject * item = PyList_GetItem(clist, i);
+                if (PyInt_Check(item)) {
+                    val[i] = (float)PyInt_AsLong(item);
+                } else if (PyFloat_Check(item)) {
+                    val[i] = PyFloat_AsDouble(item);
+                } else if (PyMessageElement_Check(item)) {
+                    PyMessageElement * mitem = (PyMessageElement*)item;
+                    if (!mitem->m_obj->isNum()) {
+                        PyErr_SetString(PyExc_TypeError, "BBox() must take list of floats, or ints");
+                        return NULL;
+                    }
+                    val[i] = mitem->m_obj->asNum();
+                } else {
+                    PyErr_SetString(PyExc_TypeError, "BBox() must take list of floats, or ints");
+                    return NULL;
+                }
+            }
+            break;
+        case 3:
+        case 6:
+            val.resize(tuple_size);
+            for(int i = 0; i < tuple_size; i++) {
+                PyObject * item = PyTuple_GetItem(args, i);
+                if (PyInt_Check(item)) {
+                    val[i] = (float)PyInt_AsLong(item);
+                } else if (PyFloat_Check(item)) {
+                    val[i] = PyFloat_AsDouble(item);
+                } else {
+                    PyErr_SetString(PyExc_TypeError, "BBox() must take list of floats, or ints");
+                    return NULL;
+                }
+            }
+            break;
+        default:
+            PyErr_SetString(PyExc_TypeError, "Point3D must take list of floats, or ints, 3 ints or 3 floats");
+            return NULL;
+            break;
+    }
+        
+    PyBBox * o = newPyBBox();
+    if ( o == NULL ) {
+            return NULL;
+    }
+    if (val.size() == 3) {
+        o->box = BBox(WFMath::Point<3>(0.f, 0.f, 0.f),
+                      WFMath::Point<3>(val[0], val[1], val[2]));
+    } else {
+        o->box = BBox(WFMath::Point<3>(val[0], val[1], val[2]),
+                      WFMath::Point<3>(val[3], val[4], val[5]));
+    }
+    return (PyObject *)o;
+}
+
 static PyObject * quaternion_new(PyObject * self, PyObject * args)
 {
         PyQuaternion *o;
@@ -704,7 +843,7 @@ static PyObject * quaternion_new(PyObject * self, PyObject * args)
             case 2:
                 {
                 PyObject * v1 = PyTuple_GetItem(args, 0);
-                PyObject * v2 = PyTuple_GetItem(args, 0);
+                PyObject * v2 = PyTuple_GetItem(args, 1);
                 if (!PyVector3D_Check(v1) || !PyVector3D_Check(v2)) {
                     PyErr_SetString(PyExc_TypeError, "Quaternion(a,b) must take two vectors");
                     return NULL;
@@ -768,7 +907,6 @@ static inline void addToOplist(PyOperation * op, PyOplist * o)
     if (op != NULL) {
        if (PyOperation_Check(op)) {
            o->ops->push_back(op->operation);
-           op->own = 0;
        } else if ((PyObject*)op != Py_None) {
            PyErr_SetString(PyExc_TypeError, "Argument must be an op");
            return;
@@ -797,27 +935,16 @@ static PyObject * oplist_new(PyObject * self, PyObject * args)
         return (PyObject *)o;
 }
 
-static PyObject * object_new(PyObject * self)
-{
-        PyMessageElement * o = newPyMessageElement();
-        if (o == NULL) {
-                return NULL;
-        }
-        o->m_obj = new Element(MapType());
-        return (PyObject *)o;
-}
-
 static PyObject * entity_new(PyObject * self, PyObject * args, PyObject * kwds)
 {
-    PyMessageElement *o;
     char * id = NULL;
     
     if (!PyArg_ParseTuple(args, "|s", &id)) {
         return NULL;
     }
-    MapType omap;
+    Anonymous ent;
     if (id != NULL) {
-        omap["id"] = std::string(id);
+        ent->setId(id);
     }
     if ((kwds != NULL) && (PyDict_Check(kwds))) {
         PyObject * keys = PyDict_Keys(kwds);
@@ -832,13 +959,14 @@ static PyObject * entity_new(PyObject * self, PyObject * args, PyObject * kwds)
             PyObject * val = PyList_GetItem(vals, i);
             if ((strcmp(key, "location") == 0) && (PyLocation_Check(val))) {
                 PyLocation * loc = (PyLocation*)val;
-                loc->location->addToMessage(omap);
+                loc->location->addToEntity(ent);
             } else if (strcmp(key, "xyz") == 0) {
-                omap["pos"] = PyObject_asMessageElement(val);
+                ent->setAttr("pos", PyObject_asMessageElement(val));
             } else if ((strcmp(key, "parent") == 0) && (PyString_Check(val))) {
-                omap["loc"] = PyString_AsString(val);
+                ent->setLoc(PyString_AsString(val));
             } else if ((strcmp(key, "type") == 0) && (PyString_Check(val))) {
-                omap["parents"] = ListType(1,std::string(PyString_AsString(val)));
+                ent->setParents(std::list<std::string>(1, PyString_AsString(val)));
+                ent->setObjtype("obj");
             } else {
                 Element val_obj = PyObject_asMessageElement(val);
                 if (val_obj.getType() == Element::TYPE_NONE) {
@@ -847,60 +975,52 @@ static PyObject * entity_new(PyObject * self, PyObject * args, PyObject * kwds)
                     PyErr_SetString(PyExc_TypeError, "Arg has no type.");
                     return NULL;
                 }
-                omap[key] = val_obj;
+                ent->setAttr(key, val_obj);
             }
         }
         Py_DECREF(keys);
         Py_DECREF(vals);
     }
 
-    o = newPyMessageElement();
+    PyRootEntity * o = newPyRootEntity();
     if ( o == NULL ) {
         return NULL;
     }
-    o->m_obj = new Element(omap);
+    o->entity = ent;
     return (PyObject *)o;
 }
 
-static PyObject * cppthing_new(PyObject * self)
+static inline void addToArgs(std::vector<Root> & args, PyObject * arg)
 {
-        PyEntity * o = newPyEntity();
-        if ( o == NULL ) {
-                return NULL;
-        }
-        return (PyObject *)o;
-}
-
-static inline void addToArgs(ListType & args, PyObject * ent)
-{
-    if (ent == NULL) {
+    if (arg == NULL) {
         return;
     }
-    if (PyMessageElement_Check(ent)) {
-        PyMessageElement * obj = (PyMessageElement*)ent;
+    if (PyMessageElement_Check(arg)) {
+        PyMessageElement * obj = (PyMessageElement*)arg;
         if (obj->m_obj == NULL) {
             log(ERROR, "Operation() Null element object added to new operation arguments.");
             return;
         }
-        Element o(*obj->m_obj);
-        if (o.isMap() && (obj->Object_attr != NULL)) {
-            MapType & ent = o.asMap();
-            MapType ent2 = PyDictObject_asElementMap(obj->Object_attr);
-            MapType::const_iterator I = ent2.begin();
-            for(; I != ent2.end(); I++) {
-                if (ent.find(I->first) != ent.end()) {
-                    ent[I->first] = I->second;
-                }
-            }
+        const Element & o = *obj->m_obj;
+        if (o.isMap()) {
+            args.push_back(Atlas::Objects::Factories::instance()->createObject(o.asMap()));
+        } else {
+            log(ERROR, "Operation() Non-map element object added to new operation arguments."); // FIXME perhaps this should raise a python exception?
         }
-        args.push_back(o);
-    } else if (PyOperation_Check(ent)) {
-        PyOperation * op = (PyOperation*)ent;
-        if (op->operation == NULL) {
+    } else if (PyOperation_Check(arg)) {
+        PyOperation * op = (PyOperation*)arg;
+        if (!op->operation.isValid()) {
             log(ERROR, "Operation() Null operation object added to new operation arguments.");
             return;
         }
-        args.push_back(op->operation->asObject());
+        args.push_back(op->operation);
+    } else if (PyRootEntity_Check(arg)) {
+        PyRootEntity * ent = (PyRootEntity*)arg;
+        if (!ent->entity.isValid()) {
+            log(ERROR, "Operation() Null operation object added to new operation arguments.");
+            return;
+        }
+        args.push_back(ent->entity);
     } else {
         log(ERROR, "Operation() Unknown object added to operation arguments.");
     }
@@ -922,47 +1042,19 @@ static PyObject * operation_new(PyObject * self, PyObject * args, PyObject * kwd
     if (op == NULL) {
         return NULL;
     }
-    if (strcmp(type, "tick") == 0) {
-        op->operation = new Tick();
-    } else if (strcmp(type, "sight") == 0) {
-        op->operation = new Sight();
-    } else if (strcmp(type, "set") == 0) {
-        op->operation = new Set();
-    } else if (strcmp(type, "burn") == 0) {
-        op->operation = new Burn();
-    } else if (strcmp(type, "action") == 0) {
-        op->operation = new Action();
-    } else if (strcmp(type, "chop") == 0) {
-        op->operation = new Chop();
-    } else if (strcmp(type, "cut") == 0) {
-        op->operation = new Cut();
-    } else if (strcmp(type, "create") == 0) {
-        op->operation = new Create();
-    } else if (strcmp(type, "setup") == 0) {
-        op->operation = new Setup();
-    } else if (strcmp(type, "look") == 0) {
-        op->operation = new Look();
-    } else if (strcmp(type, "move") == 0) {
-        op->operation = new Move();
-    } else if (strcmp(type, "talk") == 0) {
-        op->operation = new Talk();
-    } else if (strcmp(type, "touch") == 0) {
-        op->operation = new Touch();
-    } else if (strcmp(type, "eat") == 0) {
-        op->operation = new Eat();
-    } else if (strcmp(type, "nourish") == 0) {
-        op->operation = new Nourish();
-    } else if (strcmp(type, "info") == 0) {
-        op->operation = new Info();
-    } else if (strcmp(type, "thought") == 0 ||
-               strcmp(type, "goal_info") == 0) {
+    if (strcmp(type, "thought") == 0 || strcmp(type, "goal_info") == 0) {
         Py_DECREF(op);
         Py_INCREF(Py_None);
         return Py_None;
     } else {
-        op->operation = new Generic(type);
+        Root r = Atlas::Objects::Factories::instance()->createObject(type);
+        op->operation = Atlas::Objects::smart_dynamic_cast<RootOperation>(r);
+        if (!op->operation.isValid()) {
+            Py_DECREF(op);
+            PyErr_SetString(PyExc_TypeError, "Operation() unknown operation type requested");
+            return NULL;
+        }
     }
-    op->own = 1;
     if (PyMapping_HasKeyString(kwds, "to")) {
         PyObject * to = PyMapping_GetItemString(kwds, "to");
         PyObject * to_id = 0;
@@ -1004,11 +1096,10 @@ static PyObject * operation_new(PyObject * self, PyObject * args, PyObject * kwd
         op->operation->setFrom(PyString_AsString(from_id));
         Py_DECREF(from_id);
     }
-    ListType args_list;
+    std::vector<Root> & args_list = op->operation->modifyArgs();
     addToArgs(args_list, arg1);
     addToArgs(args_list, arg2);
     addToArgs(args_list, arg3);
-    op->operation->setArgs(args_list);
     return (PyObject *)op;
 }
 
@@ -1077,54 +1168,64 @@ list_contains_it:
 }
 
 static PyMethodDef atlas_methods[] = {
-    /* {"system",       spam_system, METH_VARARGS}, */
     {"Operation",  (PyCFunction)operation_new,  METH_VARARGS|METH_KEYWORDS},
     {"isLocation", is_location,                 METH_VARARGS},
-    {"distance_to",distance_to,                 METH_VARARGS},
     {"Location",   location_new,                METH_VARARGS},
-    {"Object",     (PyCFunction)object_new,     METH_NOARGS},
     {"Entity",     (PyCFunction)entity_new,     METH_VARARGS|METH_KEYWORDS},
     {"Message",    oplist_new,                  METH_VARARGS},
-    {"cppEntity",  (PyCFunction)cppthing_new,   METH_NOARGS},
-    {NULL,              NULL}                           /* Sentinel */
+    {NULL,          NULL}                       /* Sentinel */
+};
+
+static PyMethodDef physics_methods[] = {
+    {"distance_to",distance_to,                 METH_VARARGS},
+    {"square_distance",square_distance,         METH_VARARGS},
+    {"square_horizontal_distance",
+      square_horizontal_distance,               METH_VARARGS},
+    {NULL,          NULL}                       /* Sentinel */
 };
 
 static PyMethodDef vector3d_methods[] = {
-        {"Vector3D",    vector3d_new,   METH_VARARGS},
-        {NULL,          NULL}                           /* Sentinel */
+    {"Vector3D",    vector3d_new,               METH_VARARGS},
+    {NULL,          NULL}                       /* Sentinel */
 };
 
 static PyMethodDef point3d_methods[] = {
-        {"Point3D",    point3d_new,   METH_VARARGS},
-        {NULL,          NULL}                           /* Sentinel */
+    {"Point3D",     point3d_new,                 METH_VARARGS},
+    {NULL,          NULL}                       /* Sentinel */
+};
+
+static PyMethodDef bbox_methods[] = {
+    {"BBox",        bbox_new,                 METH_VARARGS},
+    {NULL,          NULL}                       /* Sentinel */
 };
 
 static PyMethodDef quaternion_methods[] = {
-        {"Quaternion",  quaternion_new, METH_VARARGS},
-        {NULL,          NULL}                           /* Sentinel */
+    {"Quaternion",  quaternion_new,             METH_VARARGS},
+    {NULL,          NULL}                       /* Sentinel */
 };
 
 static PyMethodDef server_methods[] = {
-        {"WorldTime",   worldtime_new,  METH_VARARGS},
-        {NULL,          NULL}                           /* Sentinel */
+    {"WorldTime",   worldtime_new,              METH_VARARGS},
+    {NULL,          NULL}                       /* Sentinel */
 };
 
 static PyMethodDef common_methods[] = {
-        //{"null",      null_new,       METH_VARARGS},
-        {NULL,          NULL}                           /* Sentinel */
+    //{"null",      null_new,                   METH_VARARGS},
+    {NULL,          NULL}                       /* Sentinel */
 };
 
 static PyMethodDef misc_methods[] = {
-        {"set_kw",      set_kw,         METH_VARARGS},
-        {NULL,          NULL}                           /* Sentinel */
+    {"set_kw",      set_kw,                     METH_VARARGS},
+    {NULL,          NULL}                       /* Sentinel */
 };
 
 void init_python_api()
 {
     std::string importCmd("ruleset_import_hooks.install([");
-    std::vector<std::string>::const_iterator I;
-    for(I = rulesets.begin(); I != rulesets.end(); I++) {
-        if (I != rulesets.begin()) {
+    std::vector<std::string>::const_iterator Ibeg = rulesets.begin();
+    std::vector<std::string>::const_iterator Iend = rulesets.end();
+    for (std::vector<std::string>::const_iterator I = Ibeg; I != Iend; ++I) {
+        if (I != Ibeg) {
             importCmd = importCmd + ",";
         }
         importCmd = importCmd + "\"" + *I + "\"";
@@ -1142,18 +1243,19 @@ void init_python_api()
         return;
     }
 
-    PyObject * logger = (PyObject*)PyObject_NEW(PyLogger, &PyLogger_Type);
-    PyObject_SetAttrString(sys_module, "stdout", logger);
-    Py_DECREF(logger);
-    PyObject * errorLogger = (PyObject*)PyObject_NEW(PyLogger,
-                                                     &PyErrorLogger_Type);
-    PyObject_SetAttrString(sys_module, "stderr", errorLogger);
-    Py_DECREF(errorLogger);
+    PyObject * outLogger = (PyObject*)PyObject_NEW(PyLogger, &PyOutLogger_Type);
+    PyObject_SetAttrString(sys_module, "stdout", outLogger);
+    Py_DECREF(outLogger);
+
+    PyObject * errLogger = (PyObject*)PyObject_NEW(PyLogger, &PyErrLogger_Type);
+    PyObject_SetAttrString(sys_module, "stderr", errLogger);
+    Py_DECREF(errLogger);
 
     PyObject * sys_path = PyObject_GetAttrString(sys_module, "path");
     if (sys_path != 0) {
         if (PyList_Check(sys_path)) {
-            for(I = rulesets.begin(); I != rulesets.end(); I++) {
+            std::vector<std::string>::const_iterator I = Ibeg;
+            for (; I != Iend; ++I) {
                 std::string p = share_directory + "/cyphesis/rulesets/" + *I;
                 PyObject * path = PyString_FromString(p.c_str());
                 PyList_Append(sys_path, path);
@@ -1172,13 +1274,23 @@ void init_python_api()
         return;
     }
 
+    if (Py_InitModule("physics", physics_methods) == NULL) {
+        log(CRITICAL, "Python init failed to create physics module\n");
+        return;
+    }
+
     if (Py_InitModule("Vector3D", vector3d_methods) == NULL) {
         log(CRITICAL, "Python init failed to create Vector3D module\n");
         return;
     }
 
     if (Py_InitModule("Point3D", point3d_methods) == NULL) {
-        log(CRITICAL, "Python init failed to create Vector3D module\n");
+        log(CRITICAL, "Python init failed to create Point3D module\n");
+        return;
+    }
+
+    if (Py_InitModule("BBox", bbox_methods) == NULL) {
+        log(CRITICAL, "Python init failed to create BBox module\n");
         return;
     }
 
