@@ -12,8 +12,10 @@ Script::~Script()
 {
 }
 
-bool Script::Operation(const std::string & opname, const RootOperation & op,
-                       OpVector & res, RootOperation * sub_op)
+bool Script::operation(const std::string & opname,
+                       const Atlas::Objects::Operation::RootOperation & op,
+                       OpVector & res,
+                       const Atlas::Objects::Operation::RootOperation * sub_op)
 {
    return false;
 }
