@@ -26,13 +26,8 @@ class Stackable : public Stackable_parent {
     explicit Stackable(const std::string & id);
     virtual ~Stackable();
 
-    virtual bool get(const std::string &, Element &) const;
-    virtual void set(const std::string &, const Element &);
-
-    virtual void addToMessage(MapType &) const;
-
-    virtual void CombineOperation(const Combine &, OpVector &);
-    virtual void DivideOperation(const Divide &, OpVector &);
+    virtual void CombineOperation(const Operation &, OpVector &);
+    virtual void DivideOperation(const Operation &, OpVector &);
 };
 
 #endif // RULESETS_STACKABLE_H
