@@ -9,6 +9,7 @@
 
 namespace Mercator {
     class Terrain;
+    class TileShader;
 }
 
 typedef Thing World_parent;
@@ -23,46 +24,52 @@ class World : public World_parent {
   protected:
     /// Terrain manager for the world.
     Mercator::Terrain & m_terrain;
+    /// Terrain shader tracking surface type.
+    Mercator::TileShader & m_tileShader;
     /// Set of terrain points which have been changed.
     PointSet m_modifiedTerrain;
     /// Set of terrain points which have been added.
     PointSet m_createdTerrain;
 
-    void getTerrain(MapType &) const;
-    void setTerrain(const MapType &);
   public:
     explicit World(const std::string & id);
     virtual ~World();
 
+    /// \brief Accessor for terrain manager
     const Mercator::Terrain & terrain() {
         return m_terrain;
     }
 
+    /// \brief Accessor for set of terrain points which have been changed
     const PointSet & modifiedTerrain() {
         return m_modifiedTerrain;
     }
 
+    /// \brief Accessor for set of terrain points which have been added
     const PointSet & createdTerrain() {
         return m_createdTerrain;
     }
 
+    /// \brief Clear the sets used to track terrain modifications
     void clearTerrainFlags() {
         m_modifiedTerrain.clear();
         m_createdTerrain.clear();
     }
 
     float getHeight(float x, float y);
+    int getSurface(const Point3D &,  int &);
 
-    virtual bool get(const std::string &, Element &) const;
-    virtual void set(const std::string &, const Element &);
+    void delveOperation(const Operation &, OpVector &);
+    void digOperation(const Operation &, OpVector &);
+    void mowOperation(const Operation &, OpVector &);
 
-    virtual void addToMessage(MapType &) const;
-
-    virtual void LookOperation(const Look & op, OpVector &);
-    virtual void BurnOperation(const Burn & op, OpVector &);
-    virtual void DeleteOperation(const Delete & op, OpVector &);
-    virtual void MoveOperation(const Move & op, OpVector &);
-    virtual void SetOperation(const Set & op, OpVector &);
+    virtual void EatOperation(const Operation &, OpVector &);
+    virtual void LookOperation(const Operation &, OpVector &);
+    virtual void BurnOperation(const Operation &, OpVector &);
+    virtual void DeleteOperation(const Operation &, OpVector &);
+    virtual void MoveOperation(const Operation &, OpVector &);
+    virtual void SetOperation(const Operation &, OpVector &);
+    virtual void OtherOperation(const Operation &, OpVector &);
 };
 
 #endif // RULESETS_WORLD_H
