@@ -2,6 +2,8 @@
 #Copyright (C) 1999 Aloril (See the file COPYING for details).
 
 from atlas import *
+from physics import *
+from Quaternion import Quaternion
 from common import const
 from types import *
 
@@ -14,10 +16,10 @@ else:
     from server import WorldTime
     BaseMind = Thing
 
-from mind.MemMap import MemMap
 from mind.Memory import Memory
 from mind.Knowledge import Knowledge
 from mind.panlingua import interlinguish,ontology
+from mind.compass import vector_to_compass
 from common import log,const
 from server import dictlist
 import mind.goals
@@ -56,6 +58,7 @@ class NPCMind(BaseMind):
         #print "Map update",obj
         foo_lst = self.things.get('Foo',[])
         for foo in foo_lst[:]: #us copy in loop, because it might get modified
+            print "Oh MY GOD! We have a Foo thing!"
             if foo.id==obj.id:
                 self.remove_thing(foo)
                 self.add_thing(obj)
@@ -67,17 +70,21 @@ class NPCMind(BaseMind):
         """called once by world after object has been made
            send first tick operation to object"""
         #CHEAT!: add memory, etc... initialization (or some of it to __init__)
-        return Operation("look")+Operation("tick",to=self)
+        return Operation("look")+Operation("tick")
     def tick_operation(self, op):
         """periodically reasses situation"""
         self.tick_count=self.tick_count+1
-        opTick=Operation("tick",to=self)
-        opTick.time.sadd=const.basic_tick
+        opTick=Operation("tick")
+        opTick.setFutureSeconds(const.basic_tick)
         result=self.think()
         if self.message_queue:
             result = self.message_queue + result
             self.message_queue = None
         return opTick+result
+    def unseen_operation(self, op):
+        obsolete_id = op[0].id
+        print "Got unseen op for ", obsolete_id
+        self.map.delete(obsolete_id)
     ########## Persistance operations
     def save_operation(self, op):
         mind = Entity(self.id)
@@ -94,7 +101,7 @@ class NPCMind(BaseMind):
     def sight_create_operation(self, original_op, op):
         #BaseMind version overridden!
         obj=self.map.add(op[0], op.getSeconds())
-        if original_op.from_==self:
+        if original_op.from_==self.id:
             self.add_thing(obj)
     def sight_move_operation(self, original_op, op):
         """change position in out local map"""
@@ -102,8 +109,8 @@ class NPCMind(BaseMind):
         if obj.location.parent.id==self.id:
             self.add_thing(obj)
             if obj.type[0]=="coin":
-                self.money_transfers.append([op.from_.id, 1])
-                return Operation("action", Entity(self.id, action="sell"),to=self)
+                self.money_transfers.append([op.from_, 1])
+                return Operation("action", Entity(self.id, action="sell"))
     #replaced with dynamically added add_extinguish_fire -goal
     #ie: NPC is first teached that when it sees "sight_burn" it needs to
     #execute above goal, which then adds extinguish_fire goal and executes it
@@ -119,7 +126,7 @@ class NPCMind(BaseMind):
 ##         log.debug(2,"sight_burn_operation: add goal updating")
     ########## Talk operations
     def admin_sound(self, op):
-        return op[0].from_.id==self.id
+        return op[0].from_==self.id
 
     def interlinguish_warning(self, op, say, msg):
         log.debug(1,str(self.id)+" interlinguish_warning: "+str(msg)+\
@@ -132,12 +139,13 @@ class NPCMind(BaseMind):
             if not price:
                 return
             goal=mind.goals.common.misc_goal.transaction(object, op.from_, price)
+            who=self.map.get(op.from_)
             self.goals.insert(0,goal)
-            return Operation("talk", Entity(say=op.from_.name+" one "+object+" will be "+str(price)+" coins"))
+            return Operation("talk", Entity(say=who.name+" one "+object+" will be "+str(price)+" coins")) + self.face(who)
     def interlinguish_desire_verb3_operation(self, op, say):
         object=say[2:]
         verb=interlinguish.get_verb(object)
-        operation_method=self.find_operation(verb,"interlinguish_desire_verb3_",
+        operation_method=self.find_op_method(verb,"interlinguish_desire_verb3_",
                                              self.interlinguish_undefined_operation)
         res = Message()
         res = res + self.call_interlinguish_triggers(verb, "interlinguish_desire_verb3_", op, object)
@@ -166,12 +174,29 @@ class NPCMind(BaseMind):
             self.add_knowledge(predicate,subject,loc)
         else:
             self.add_knowledge(predicate,subject,object)
-    def interlinguish_price_verb1_operation(self, op, say):
-        if not self.admin_sound(op):
-            return self.interlinguish_warning(op,say,"You are not admin")
+    def interlinguish_tell_verb1_operation(self, op, say):
+        # Currently no checking for trus here.
+        # We are being liberal with interpretation of "subject" and "object"
         subject=say[1].word
-        cost=say[2].word
-        self.add_knowledge("price",subject,cost)
+        predicate=say[2].word
+        object=say[3].word
+        k=self.get_knowledge(predicate, object)
+        if k==None:
+            return Operation('talk',Entity(say="I know nothing about the "+predicate+" of "+object))
+        else:
+            k_type = type(k)
+            if k_type==type(Location()):
+                dist = distance_to(self.location, k)
+                dist.z = 0
+                distmag = dist.mag()
+                if distmag < 8:
+                    k = 'right here'
+                else:
+                    # Currently this assumes dist is relative to TLVE
+                    k='%f metres %s' % (distmag, vector_to_compass(dist))
+            elif k_type!=StringType:
+                k='difficult to explain'
+            return Operation('talk', Entity(say="The "+predicate+" of "+object+" is "+k)) + self.face(self.map.get(op.from_))
     def interlinguish_learn_verb1_operation(self, op, say):
         if not self.admin_sound(op):
             return self.interlinguish_warning(op,say,"You are not admin")
@@ -204,7 +229,7 @@ class NPCMind(BaseMind):
         if interlinguish.convert_english_to_interlinguish(self,talk_entity):
             say=talk_entity.interlinguish
             verb=interlinguish.get_verb(say)
-            operation_method=self.find_operation(verb,"interlinguish_",
+            operation_method=self.find_op_method(verb,"interlinguish_",
                                   self.interlinguish_undefined_operation)
             res = res + self.call_interlinguish_triggers(verb, "interlinguish_", original_op,say)
         else:
@@ -265,6 +290,9 @@ class NPCMind(BaseMind):
         if what=="location":
             #and reverse too
             self.reverse_knowledge.add("location",value,key)
+    def remove_knowledge(self,what,key):
+        """remove certain type of knowledge"""
+        self.knowledge.remove(what,key)
     ########## Importance: Knowledge about how things compare in urgency, etc..
     def add_importance(self, sub, cmp, obj):
         """add importance: both a>b and b<a"""
@@ -300,7 +328,10 @@ class NPCMind(BaseMind):
         what=thing.as_entity()
         ent = Entity(description=desc, what=what)
         self.send(Operation("thought",ent))
-        dictlist.add_value(self.things,thing.name,thing)
+        name=thing.name
+        if not name or name=='':
+            name=thing.type[0]
+        dictlist.add_value(self.things,name,thing)
         log.debug(3,"\tafter: "+str(self.things))
     def find_thing(self, thing):
         if StringType==type(thing):
@@ -373,3 +404,11 @@ class NPCMind(BaseMind):
             self.message_queue=Message(op)
         else:
             self.message_queue.append(op)
+    ########## turn to face other entity
+    def face(self, other):
+        vector = distance_to(self.location, other.location)
+        vector.z = 0
+        vector = vector.unit_vector()
+        newloc = Location(self.location.parent)
+        newloc.orientation = Quaternion(Vector3D(1,0,0), vector)
+        return Operation("move", Entity(self.id, location=newloc))
