@@ -2,9 +2,13 @@
 #Copyright (C) 1999 Aloril (See the file COPYING for details).
 #Al Riddoch - Added transport_something() goal
 
+from physics import *
 from mind.goals.common.common import *
 from mind.goals.common.move import *
-from whrandom import *
+try:
+  from random import *
+except ImportError:
+  from whrandom import *
 
 ######################## MAKE LOTS OF SOMETHING ###############################
 
@@ -40,7 +44,7 @@ class nyi(Goal):
         self.vars=["desc"]
     def not_yet_implemented(self, me):
         #CHEAT! (make it so that it asks other people for knowledge)
-        return Operation("NYI",Entity(description=self.desc))
+        return
 
 ############################ IMAGINARY TASK ###################################
 
@@ -54,7 +58,7 @@ class imaginary(Goal):
 ####################### GET KNOWLEDEGE ABOUT THING ############################
 
 class get_knowledge(Goal):
-    def __init__(self, me, what):
+    def __init__(self, what):
         Goal.__init__(self,"get knowledge about thing by name",
                       self.do_I_know_about_this,
                       [nyi("Try another task or try asking around!")])
@@ -63,7 +67,10 @@ class get_knowledge(Goal):
     def do_I_know_about_this(self, me):
         if me.things.has_key(self.what):
             return 1
-        return me.get_knowledge("place",self.what)
+        if me.get_knowledge("place",self.what):
+            return 1
+        return 0
+            
 
 ############################ BUY KNOWN THING ##################################
 
@@ -108,7 +115,7 @@ class acquire_known_thing(acquire):
         Goal.__init__(self,"acquire known thing by name",
                       self.is_it_in_my_inventory,
                       [buy_thing(me,what),
-                       move_it_into_me(me,what)])
+                       pick_up_possession(what)])
         self.what=what
         self.vars=["what"]
 
@@ -118,11 +125,10 @@ class acquire_thing(acquire):
     def __init__(self, me, what):
         Goal.__init__(self,"acquire thing by name",
                       self.is_it_in_my_inventory,
-                      [get_knowledge(me,what),
+                      [get_knowledge(what),
                        acquire_known_thing(me,what)])
         self.what=what
         self.vars=["what"]
-
         
 ############################ TASK WITH PLACE AND TOOL #########################
 
@@ -133,7 +139,7 @@ class task(Goal):
                       [acquire_thing(me,tool),
                        move_me_area(place),
                        spot_something(what),
-                       move_me(lambda me,what=what:me.things[what][0].location),
+                       move_me_to_focus(what),
                        self.do],
                       time)
         self.tool=tool
@@ -146,11 +152,14 @@ class task(Goal):
 
 class cut_something(task):
     def do(self,me):
-        if me.things.has_key(self.tool)==0: return
-        obj=me.find_thing(self.tool)[0]
         if me.things.has_key(self.what)==0: return
+        if me.things.has_key(self.tool)==0: return
+        if not hasattr(self,'wield') or not self.wield:
+            tool=me.find_thing(self.tool)[0]
+            self.wield=True
+            return Operation("wield", Entity(tool.id))
         what=me.find_thing(self.what)[0]
-        return Operation("cut",Entity(obj.id),Entity(what.id),to=obj)
+        return Operation("use",Entity(what.id))
 
 ############################ SPOT SOMETHING GOAL ########################
 
@@ -164,44 +173,53 @@ class spot_something(Goal):
         self.condition=condition
         self.vars=["what","range"]
     def do_I_have(self, me):
-        if me.things.has_key(self.what)==1:
-            for thing in me.find_thing(self.what):
-                locid = thing.location.parent.id
-                if locid!=me.location.parent.id and locid!=me.id:
-                    me.remove_thing(thing)
-        return me.things.has_key(self.what)==1
+        something=me.get_knowledge('focus', self.what)
+        if something and me.map.get(something)==None:
+            # No longer exists
+            me.remove_knowledge('focus', self.what)
+            something=None
+        return something!=None
     def do(self,me):
         thing_all=me.map.find_by_type(self.what)
         nearest=None
-        neardist=self.range
+        nearsqrdist=self.range*self.range
         for thing in thing_all:
-            distance = me.get_xyz().distance(thing.get_xyz())
-            if distance < neardist and thing.location.parent.id==me.location.parent.id:
+            sqr_dist = square_distance(me.location, thing.location)
+            # FIXME We need a more sophisticated check for parent. Perhaps just
+            # check its not in a persons inventory? Requires the ability to
+            # do decent type checks
+            if sqr_dist < nearsqrdist and thing.location.parent.id==me.location.parent.id:
                 if self.condition(thing):
                     nearest = thing
-                    neardist = distance
+                    nearsqrdist = nearsqrdist
         if nearest:
-            me.add_thing(nearest)
+            me.add_knowledge('focus', self.what, nearest.id)
                       
 ############################ FETCH SOMETHING GOAL ########################
 
-class fetch_something(acquire):
-    def __init__(self, me, what, where):
+class fetch_something(Goal):
+    def __init__(self, what, where):
         Goal.__init__(self, "fetch a thing",
                       self.is_it_in_my_inventory,
                       [move_me_area(where, 20),
                        spot_something(what),
-                       move_it_into_me(me, what)])
+                       pick_up_focus(what)])
         self.what=what
         self.vars=["what"]
+    def is_it_in_my_inventory(self,me):
+        if me.things.has_key(self.what):
+            if me.things[self.what][0].location.parent==me:
+                me.remove_knowledge('focus', self.what)
+                return 1
+        return 0
 
 ############################ MOVE SOMETHING GOAL ########################
 
 class transport_something(Goal):
-    def __init__(self, me, what, src, dest):
+    def __init__(self, what, src, dest):
         Goal.__init__(self,"move thing to place",
                       false,
-                      [fetch_something(me, what, src),
+                      [fetch_something(what, src),
                        move_me(dest),
                        move_it_outof_me(what)])
         self.what=what
@@ -214,12 +232,12 @@ class sit_down(Goal):
         Goal.__init__(self,"sit down",
                       self.am_i_sat,
                       [spot_something(where),
-                       move_me_to_it(where),
+                       move_me_to_focus(where),
                        sit])
     def am_i_sat(self,me):
         return me.mode=="sitting"
     def sit(self,me):
-        return Operation("set", Entity(me.id, mode="sitting"),to=me)
+        return Operation("set", Entity(me.id, mode="sitting"))
 
 ############################ FEED (FOR FOOD) ##################################
 
@@ -266,25 +284,48 @@ class meal(feed):
 ############################ FORAGE (FOR FOOD) ################################
 
 class forage(feed):
-    def __init__(self, me, what):
+    def __init__(self, what):
         Goal.__init__(self, "forage for food by name",
                       self.am_i_full,
                       [spot_something(what, range=5),
-                       move_it_into_me(me, what),
+                       pick_up_focus(what),
                        self.eat])
         self.what=what
         self.range=5
         self.full=0.2
         self.vars=["what","range"]
- 
-############################ BROWSE (FOR FOOD) ################################
+
+############################ PECK ###########################################
+
+class peck(feed):
+    def __init__(self) :
+        Goal.__init__(self, "forage for food by name and eat randomly",
+                      self.am_i_full,
+                      [self.do_peck])
+        self.what="world"
+        self.full=0.2
+        self.vars=["full"]
+
+    def do_peck(self, me):
+        #world = 
+        #ground = world.id 
+        #op = Operation("eat", ground, to=world)
+        target = Location(me.location.parent, me.location.coordinates)
+        target.coordinates = Vector3D(target.coordinates.x + uniform(-1.5,1.5), target.coordinates.y+ uniform(-1.5,1.5), target.coordinates.z)
+        target.velocity = Vector3D(1,0,0)
+        #op += Operation("move",  Entity(me.id, location=target))
+        return Operation("move",  Entity(me.id, location=target))
+                                      
+
+
+############################ BROWSE (FIND FOOD, EAT SOME, MOVE ON) ###########
 
 class browse(feed):
     def __init__(self, me, what, min_status):
         Goal.__init__(self, "browse for food by name",
                       self.am_i_full,
-		      [spot_something(what, range=20, condition=(lambda o,s=min_status:hasattr(o,"status") and o.status > s)),
-                       move_me_to_it(what),
+                      [spot_something(what, range=20, condition=(lambda o,s=min_status:hasattr(o,"status") and o.status > s)),
+                       move_me_to_focus(what),
                        self.eat])
         self.what=what
         self.range=20
@@ -342,11 +383,12 @@ class hunt(Goal):
         self.with=with
         self.what=what
         self.range=range
+        self.square_range=range*range
         self.vars=["with", "what", "range"]
     def none_in_range(self, me):
         thing_all=me.map.find_by_type(self.what)
         for thing in thing_all:
-            if me.get_xyz().distance(thing.get_xyz()) < self.range:
+            if square_distance(me.location, thing.location) < self.square_range:
                 return 0
         return 1
     def fight(self, me):
@@ -378,11 +420,12 @@ class defend(Goal):
         self.with=with
         self.what=what
         self.range=range
+        self.square_range=range*range
         self.vars=["with", "what", "range"]
     def none_in_range(self, me):
         thing_all=me.map.find_by_type(self.what)
         for thing in thing_all:
-            if me.get_xyz().distance(thing.get_xyz()) < self.range:
+            if square_distance(me.location, thing.location) < self.square_range:
                 return 0
         return 1
     def fight(self, me):
@@ -409,8 +452,7 @@ class buy_trade(get_thing):
         if self.ticks==5:
             self.ticks=0
             es=Entity(say="Sell your " + self.what + " here!")
-            return Operation("talk",es,from_=me,to=me)
-
+            return Operation("talk",es)
 
 ############################ SELL SOMETHING (AS TRADER) ########################
 
@@ -435,17 +477,17 @@ class sell_trade(Goal):
         if self.ticks==5:
             self.ticks=0
             es=Entity(say="Get your " + self.what + " here!")
-            ret = ret + Operation("talk",es,from_=me,to=me)
-            ret = ret + Operation("action", Entity(me.id, action="shout"), to=me)
+            ret = ret + Operation("talk",es)
+            ret = ret + Operation("action", Entity(me.id, action="shout"))
         else:
             if randint(0,4)==1:
-                ret = ret + Operation("action", Entity(me.id, action="wave"), to=me)
+                ret = ret + Operation("action", Entity(me.id, action="wave"))
         return ret
 
 ################ TRADE (BUY SOMETHING, USE TOOL, SELL PRODUCT) ################
 
 class trade(Goal):
-    def __init__(self, me, wbuy, tool, op, wsell, where, when=None):
+    def __init__(self, me, wbuy, tool, wsell, where, when=None):
         Goal.__init__(self, "trade at a market",
                       false,
                       [acquire_thing(me,tool),
@@ -454,15 +496,18 @@ class trade(Goal):
                       when)
         self.wbuy=wbuy
         self.tool=tool
-        self.op=op
         self.wsell=wsell
-        self.vars=["wbuy","tool","op","wsell"]
+        self.wield=False
+        self.vars=["wbuy","tool","wsell"]
     def process(self,me):
         if me.things.has_key(self.wbuy)==0: return
         if me.things.has_key(self.tool)==0: return
-        thing=me.find_thing(self.wbuy)[0]
         tool=me.find_thing(self.tool)[0]
-        return Operation(self.op,Entity(tool.id),Entity(thing.id),to=tool)
+        if not self.wield:
+            self.wield=True
+            return Operation("wield",Entity(tool.id))
+        thing=me.find_thing(self.wbuy)[0]
+        return Operation("use",Entity(thing.id, objtype="obj"))
 
 ############################# RUN MARKET STALL ##############################
 
@@ -474,7 +519,7 @@ class market(Goal):
     def set_it(self,me):
         if me.things.has_key(self.shop)==0: return
         shop=me.find_thing(self.shop)[0]
-        return Operation("set", Entity(shop.id, mode=self.state),to=me)
+        return Operation("set", Entity(shop.id, mode=self.state))
 
 class run_shop(market):
     def __init__(self, shop, updown, time):
@@ -482,6 +527,7 @@ class run_shop(market):
                       self.is_it,
                       [spot_something(shop),
                        self.set_it],time)
+        # FIXME This probably does not work, but I'll fix it when we need it
         self.shop=shop
         self.state=updown
         self.vars=["shop"]
@@ -551,7 +597,7 @@ class keep_on_me(Goal):
 
 class assemble(Goal):
     def __init__(self, me, what, fromwhat):
-	sgoals=[]
+        sgoals=[]
         for item in fromwhat:
             sgoals.append(spot_something(item))
             sgoals.append(move_it(item, me.location, 1))
@@ -562,14 +608,14 @@ class assemble(Goal):
         self.fromwhat=fromwhat
         self.vars=["what","fromwhat"]
     def build(self, me):
-	retops=Message()
+        retops=Message()
         for item in self.fromwhat:
             if me.things.has_key(item)==0: return
         for item in self.fromwhat:
             cmpnt=me.find_thing(item)[0]
-            retops = retops + Operation("set", Entity(cmpnt.id,status=-1), to=cmpnt)
-        retops = retops + Operation("create", Entity(name=self.what,parents=[self.what], location=me.location.copy()), to=me)
-        retops = retops + Operation("action", Entity(me.id, action="conjure"), to=me)
+            retops = retops + Operation("set", Entity(cmpnt.id,status=-1))
+        retops = retops + Operation("create", Entity(name=self.what,parents=[self.what], location=me.location.copy()))
+        retops = retops + Operation("action", Entity(me.id, action="conjure"))
         return retops
 
 ######################## TRANSACTION (Sell thing) #######################
@@ -594,7 +640,7 @@ class transaction(Goal):
         payed=self.payed
         if payed==-1: payed=0
         for item in me.money_transfers:
-            if item[0]==self.who.id:
+            if item[0]==self.who:
                 payed=payed+int(item[1])
                 me.money_transfers.remove(item)
         if payed != self.payed:
@@ -602,15 +648,16 @@ class transaction(Goal):
             return 0
         return 1
     def transact(self,me):
+        who=me.map.get(self.who)
         if me.things.has_key(self.what)==0:
             self.irrelevant=1
             return Operation("talk",Entity(say="I don't have any "+self.what+" left."))
         if self.payed < self.cost:
-            return Operation("talk",Entity(say=self.who.name+" you owe me "+str(self.cost-self.payed)+" coins."))
+            return Operation("talk",Entity(say=who.name+" you owe me "+str(self.cost-self.payed)+" coins."))
         thing=me.find_thing(self.what)[0]
         res=Message()
         me.remove_thing(thing)
-        res.append(Operation("move",Entity(thing.id, location=Location(self.who,Point3D(0,0,0)))))
+        res.append(Operation("move",Entity(thing.id, location=Location(who,Point3D(0,0,0)))))
         res.append(Operation("talk",Entity(say="Thankyou for your custom.")))
         self.irrelevant=1
         return res
