@@ -7,21 +7,21 @@ from world.objects.Thing import Thing
 from common import log,const
 from world import probability
 from misc import set_kw
-from world.physics.Vector3D import Vector3D
+
+if const.server_python:
+    from world.physics.Vector3D import Vector3D
+else:
+    from Vector3D import Vector3D
+    from Point3D import Point3D
 
 class House(Thing):
     """This base class for houses, building material is wood"""
-    def __init__(self, cppthing, **kw):
-    	self.base_init(cppthing, kw)
-    	set_kw(self,kw,"burn_speed",0.2)
-        set_kw(self,kw,"material","wood")
-        set_kw(self,kw,"mass", 5000.0)
     def tick_operation(self, op):
         """check if we should self-combust
            in any case send ourself next tick"""
         #print `self`,"Got tick operation:\n"
         opTick=Operation("tick",to=self)
-        opTick.time.sadd=const.basic_tick
+        opTick.setFutureSeconds(const.basic_tick)
         if probability.does_it_happen(probability.fire_probability):
             fireEntity=Entity(name='fire',type=['fire'],status=0.0,
                               location=Location(self,Point3D(0.0,0.0,0.0)))
