@@ -10,10 +10,6 @@ from Vector3D import Vector3D
 
 class Bow(Thing):
     """This is base class for bows, this one just ordinary bow"""
-    def __init__(self, cppthing, **kw):
-        self.base_init(cppthing, kw)
-        set_kw(self,kw,"mass",1.5)
-        set_kw(self,kw,"ammo","arrow")
     def shoot_operation(self, op):
         ammo = op[0].id
         to_ = op[1].id
@@ -27,7 +23,7 @@ class Bow(Thing):
         loc2.velocity = Vector3D(0,0,0)
         m1 = Operation("move",Entity(ammo,location=loc1),to=ammo)
         m2 = Operation("move",Entity(ammo,location=loc2),to=ammo)
-        m2.time.sadd=time
+        m2.setFutureSeconds(time)
         t = Operation("set",Entity(to_,status=-1),to=to_)
-        t.time.sadd=time
+        t.setFutureSeconds(time)
         return Message(m1,m2,t)
