@@ -2,7 +2,10 @@
 #Copyright (C) 1999 Aloril (See the file COPYING for details).
 
 from atlas import *
-from whrandom import *
+try:
+  from random import *
+except ImportError:
+  from whrandom import *
 from mind.panlingua import interlinguish
 il=interlinguish
 from world import probability
@@ -30,7 +33,12 @@ hall_xyz=(5,3,settlement_height)
 forest_xyz=(-20,-60,settlement_height)
 
 pig_sty_xyz=(8,8,settlement_height)
-butcher_stall_xyz=(-41.5,-6.3,settlement_height)
+butcher_stall_xyz=(155,140,settlement_height)
+butcher_xyz=(153,142,settlement_height)
+tool_stall_xyz=(150,138,settlement_height)
+tool_merchant_xyz=(150,140,settlement_height)
+
+lake_xyz=(-40,-70,0)
 
 knowledge=[('axe','place','smithy'),
            ('forest','location',forest_xyz),
@@ -68,26 +76,35 @@ keep=(il.keep,"keep()")
 sell=(il.sell,"sell_trade()")
 patrol=(il.patrol,"patrol()")
 
+toolprices = [('axe', 'price', '4'), 
+              ('shovel', 'price', '5'),
+              ('tinderbox', 'price', '8')]
+
+toolmerc_knowledge=[('market', 'location', tool_stall_xyz)]
+
 pig_goals=[(il.avoid,"avoid(['wolf','skeleton','crab'],10.0)"),
-           (il.forage,"forage(self, 'acorn')"),
-           (il.forage,"forage(self, 'apple')"),
-           (il.forage,"forage(self, 'mushroom')"),
+           (il.forage,"forage('acorn')"),
+           (il.forage,"forage('apple')"),
+           (il.forage,"forage('mushroom')"),
            (il.herd,"herd()")]
 
-deer_goals=[(il.avoid,"avoid(['settler','orc'],10.0)"),
-            (il.forage,"forage(self, 'apple')"),
-            (il.forage,"forage(self, 'mushroom')"),
+deer_goals=[(il.avoid,"avoid(['settler','orc'],30.0)"),
+            (il.avoid,"avoid(['fire'],10.0)"),
+            (il.forage,"forage('apple')"),
+            (il.forage,"forage('mushroom')"),
             (il.browse,"browse(self, 'fir', 0.8)"),
             (il.flock,"flock()")]
 
-chicken_goals=[(il.avoid,"avoid(['settler','orc','wolf'],10.0)"),
-               (il.flock,"flock()")]
+chicken_goals=[(il.avoid,"avoid(['orc','wolf'],10.0)"),
+               (il.avoid,"avoid(['settler'],1.0)"),
+               (il.flock,"flock()"),
+               (il.peck,"peck()")]
 
 squirrel_goals=[(il.avoid,"avoid(['wolf','crab'],10.0)"),
-                (il.forage,"forage(self, 'acorn')"),
-                (il.forage,"forage(self, 'pinekernel')")]
+                (il.forage,"forage('acorn')"),
+                (il.forage,"forage('pinekernel')")]
 
-wolf_goals=[(il.forage,"forage(self, 'ham')"),
+wolf_goals=[(il.forage,"forage('ham')"),
             (il.hunt,"predate(self,'pig',30.0)"),
             (il.hunt,"predate(self,'crab',20.0)"),
             (il.hunt,"predate(self,'squirrel',10.0)"),
@@ -105,8 +122,8 @@ directions = [[0,0,0.707,0.707],[0,0,0,1],[0,0,-0.707,0.707],[0,0,1,0],
               [0,0,0.387,0.921],[0,0,-0.387,0.921],[0,0,-0.921,0.387],[0,0,0.921,0.387]]
 
 forests = [
-           ('oak', 20, 20, 200, -100, 100, 20),
-           ('oak', 20, -100, 100, 20, 200, 20),
+           ('oak', 30, 20, 190, -200, 100, 20),
+           ('oak', 20, -200, -100, 0, 190, 20),
            ('fir', 200,  200,  300, -300, 300, 50),
            ('fir', 200, -300, 300,  200,  300, 50)
           ]
@@ -120,25 +137,35 @@ def default(mapeditor):
     world=m.look()
 
     points = { }
-    for i in range(-5, 6):
-        for j in range(-5, 6):
-            if i==5 or j==5:
+    for i in range(-6, 7):
+        for j in range(-6, 7):
+            if i>=5 or j>=5:
                 points['%ix%i'%(i,j)] = [i, j, uniform(100, 150)]
-            elif i==-5 or j == -5:
+            elif i<=-5 or j <= -5:
                 points['%ix%i'%(i,j)] = [i, j, uniform(-30, -10)]
             elif (i==2 or i==3) and (j==2 or j==3):
                 points['%ix%i'%(i,j)] = [i, j, uniform(20, 25)]
             elif i==4 or j==4:
                 points['%ix%i'%(i,j)] = [i, j, uniform(30, 80)]
             elif i==-4 or j==-4:
-                points['%ix%i'%(i,j)] = [i, j, uniform(-2, 5)]
+                points['%ix%i'%(i,j)] = [i, j, uniform(-5, 5)]
             else:
-                points['%ix%i'%(i,j)] = [i, j, 1+uniform(0, 8)*(abs(i)+abs(j))]
+                points['%ix%i'%(i,j)] = [i, j, 1+uniform(3, 11)*(abs(i)+abs(j))]
 
+    points['-4x-1'] = [-4, -1, 12.4]
+    points['-4x-2'] = [-4, -2, -8.3]
+    points['-3x-2'] = [-3, -2, -6.2]
+    points['-3x-1'] = [-3, -1, -5.3]
+    points['-2x-1'] = [-2, -1, -4.1]
     points['-1x-1'] = [-1, -1, -16.8]
     points['0x-1'] = [0, -1, -3.8]
     points['-1x0'] = [-1, 0, -2.8]
-    points['-1x1'] = [-1, 1, 12.8]
+    points['-1x1'] = [-1, 1, -1.8]
+    points['-1x2'] = [-1, 2, -1.7]
+    points['0x2'] = [0, 2, -1.6]
+    points['1x2'] = [1, 2, -1.3]
+    points['1x3'] = [1, 3, -1.1]
+    points['1x4'] = [1, 4, -0.6]
     points['1x-1'] = [1, -1, 15.8]
     points['0x0'] = [0, 0, 12.8]
     points['1x0'] = [1, 0, 23.1]
@@ -149,25 +176,56 @@ def default(mapeditor):
 
 # a wall around the world
 
-    m.make('boundary',type='boundary',xyz=(-257,-257,settlement_height),bbox=[2,514,256])
-    m.make('boundary',type='boundary',xyz=(-257,-257,settlement_height),bbox=[514,2,256])
-    m.make('boundary',type='boundary',xyz=(-257, 256,settlement_height),bbox=[514,2,256])
-    m.make('boundary',type='boundary',xyz=( 256,-257,settlement_height),bbox=[2,514,256])
+    m.make('boundary',type='boundary',xyz=(-321,-321,-20),bbox=[2,642,300],mode="fixed")
+    m.make('boundary',type='boundary',xyz=(-321,-321,-20),bbox=[642,2,300],mode="fixed")
+    m.make('boundary',type='boundary',xyz=(-321, 320,-20),bbox=[642,2,300],mode="fixed")
+    m.make('boundary',type='boundary',xyz=( 320,-321,-20),bbox=[2,642,300],mode="fixed")
 
     m.make('fir',type='fir',xyz=(-10,-0,settlement_height))
     m.make('fir',type='fir',xyz=(-0,-10,settlement_height))
     m.make('fir',type='fir',xyz=(0,10,settlement_height))
     m.make('fir',type='fir',xyz=(10,0,settlement_height))
 
+    path_area={'points' : [ [-26,-62], [-36,-31], [-26,-14], [2,-1], [22, 40], [132,122], [140,127], [144.5, 146.5], [169, 153], [169,155], [142.5,148.5], [138,129], [130,124], [18,40], [-2, 0], [-28,-12], [-38,-29], [-29,-62] ], 'layer' : 7}
+    m.make('path to village',type='path',xyz=(10, 20,settlement_height), area=path_area,bbox=[-38,-62,0,169,154,1])
+
+    m.make('tower',type='tower',xyz=(210,210,5))
+    m.make('gallows',type='gallows',xyz=(185,175,5))
+
+    m.make('house3',type='house3',xyz=(158,150,22),orientation=directions[1])
+    m.make('house3',type='house3',xyz=(158,158,22),orientation=directions[4])
+    m.make('house3',type='house3',xyz=(150,158,22),orientation=directions[0])
+    m.make('house3',type='house3',xyz=(142,158,22),orientation=directions[7])
+    m.make('house3',type='house3',xyz=(142,150,22),orientation=directions[3])
+    m.make('house3',type='house3',xyz=(142,142,22),orientation=directions[6])
+    m.make('house3',type='house3',xyz=(150,125,22),orientation=directions[2])
+    m.make('house3',type='house3',xyz=(171,142,22),orientation=directions[5])
+
+    m.make('field',type='ploughed_field',xyz=(120,170,30),status=1.0,area={'points' : [ [0,0], [0,20], [20,20], [20,0] ], 'layer' : 8}, bbox=[20,20,0])
+    m.make('field',type='ploughed_field',xyz=(142,170,30),status=1.0,area={'points' : [ [0,0], [0,20], [20,20], [20,0] ], 'layer' : 8}, bbox=[20,20,0])
+
+    village_square={'points': [[-10, -14], [15, -11], [13,18], [-8, 11]], 'layer':7 }
+    m.make('village_square', xyz=(150, 150, 22), type='path', area=village_square, bbox=[-10, -14, 0, 15, 18, 1])
+
     chickens=[]
-    xbase = uniform(12,20)
-    ybase = uniform(12,20)
+    xbase = uniform(140,160)
+    ybase = uniform(130,150)
     for i in range(0, 10):
         xpos = xbase + uniform(-5,5)
         ypos = ybase + uniform(-5,5)
         d=m.make('chicken', type='chicken', xyz=(xpos, ypos, settlement_height))
         chickens.append(d)
     m.learn(chickens,chicken_goals)
+
+    fish=[]
+    xbase = lake_xyz[0]
+    ybase = lake_xyz[1]
+    for i in range(0, 10):
+        xpos = xbase + uniform(-5,5)
+        ypos = ybase + uniform(-5,5)
+        zpos = uniform(-4,0)
+        d=m.make('fish', type='fish', xyz=(xpos, ypos, zpos))
+        fish.append(d)
     
     # m.make('sherwood',type='forest',xyz=(-50, 10,settlement_height),bbox=[40,40,40])
 
@@ -191,10 +249,10 @@ def default(mapeditor):
     # m.make('fire',type='fire',xyz=(0.7,0.7,0),parent=cfire.id)
 
     cfire=m.make('campfire',type='campfire',xyz=(3,9,settlement_height))
-    m.make('fire',type='fire',xyz=(0.7,0.7,0),parent=cfire.id)
+    m.make('fire',type='fire',xyz=(0,0,0),parent=cfire.id)
 
     cfire=m.make('campfire',type='campfire',xyz=(11,1,settlement_height))
-    m.make('fire',type='fire',xyz=(0.7,0.7,0),parent=cfire.id)
+    m.make('fire',type='fire',xyz=(0,0,0),parent=cfire.id)
 
     for i in range(0, 20):
         m.make('lumber',type='lumber',xyz=(uniform(-200,0),uniform(-200,0),settlement_height))
@@ -248,31 +306,74 @@ def default(mapeditor):
     #directions = [[0,1,0],[1,0,0],[0,-1,0],[-1,0,0],
                   #[0.7,0.7,0],[0.7,-0.7,0],[-0.7,-0.7,0],[-0.7,0.7,0]]
 
+    # An NPC settler
+    settler=m.make('settler',xyz=(1,1,0), sex='male')
+    axe=m.make('axe',type='axe',xyz=(0,0,0),parent=settler.id)
+    m.own(settler,axe)
+    m.know(settler,[('forest','location',(30,30,0))])
+    m.learn(settler,(il.trade,"harvest_resource(self,'lumber','oak','forest','axe')"))
+
+    # An NPC forester
+    settler=m.make('forester',type='settler',xyz=(0, 12, 0), sex='male')
+    trowel=m.make('trowel',xyz=(0,0,0),parent=settler.id)
+    m.own(settler, trowel)
+    m.know(settler,[('forest','location',(30,30,0))])
+    m.learn(settler,(il.trade,"plant_seeds(self,'acorn','oak','forest','trowel')"))
+
     home1_xyz=(90,-90,settlement_height)
+    
+    stall=m.make('Market Stall',type='stall',xyz=butcher_stall_xyz, orientation=directions[5])
+    m.make('wall',type='wall',parent=stall.id,xyz=(0,0,0),bbox=(-0.5,-1.5,0,0.5,1.5,0.8))
+    m.make('wall',type='wall',parent=stall.id,xyz=(2,0,0),bbox=(0,-1.5,0,0.5,1.5,2))
+
 
     butcher=m.make('Ulad Bargan',type='butcher',desc='the butcher',
-                 xyz=butcher_stall_xyz,age=probability.fertility_age,sex='male')
-    m.learn(butcher,(il.trade,"trade(self, 'pig', 'cleaver', 'cut', 'ham', 'market','day')"))
+                 xyz=butcher_xyz,age=probability.fertility_age,sex='male')
+    m.learn(butcher,(il.trade,"trade(self, 'pig', 'cleaver', 'ham', 'market')"))
     m.learn(butcher,(il.buy_livestock,"buy_livestock('pig', 1)"))
     m.learn(butcher,(il.market,"run_shop('mstall_freshmeat_1_se','open','dawn')"))
     m.learn(butcher,(il.market,"run_shop('mstall_freshmeat_1_se','closed','evening')"))
     m.know(butcher,bknowledge)
     m.know(butcher,bprices)
+    
     cleaver=m.make('cleaver', type='cleaver', desc='cleaver for cutting meat',
-                   place='market', xyz=(-41,-5,settlement_height))
+                   place='market', xyz=(0, 0, 0), parent=butcher.id)
     m.own(butcher,cleaver)
+    m.own(butcher, stall)
+    
     m.learn(butcher,(il.sell,"sell_trade('ham', 'market')"))
     coins=[]
     for i in range(0, 60):
         coins.append(m.make('coin',type='coin',xyz=(0,0,0),parent=butcher.id))
     m.own(butcher,coins)
     
+# 	tool-merchant
+    stall = m.make('Tool Stall', type='stall', xyz=tool_stall_xyz, orientation=directions[2])
+    m.make('wall',type='wall',parent=stall.id,xyz=(0,0,0),bbox=(-0.5,-1.5,0,0.5,1.5,0.8))
+    m.make('wall',type='wall',parent=stall.id,xyz=(2,0,0),bbox=(0,-1.5,0,0.5,1.5,2))
+
+    tmerchant=m.make('Blackun Decker',type='merchant',desc='the tool merchant',
+                 xyz=tool_merchant_xyz,age=probability.fertility_age,sex='male')
+
+    m.know(tmerchant, toolmerc_knowledge)
+    m.know(tmerchant,toolprices)
+
+    m.own(tmerchant, stall)
+
+    tools=[]
+    tooltypes=['shovel', 'axe', 'tinderbox']
+    
+    for i in range(0, 15):
+        tty = tooltypes[randint(0,2)]
+        tools.append(m.make(tty,type=tty,xyz=(uniform(-0.7, 0.7),uniform(-0.5,0.5),0.6),parent=stall.id,orientation=directions[randint(0,7)]))
+    
+    m.own(tmerchant,tools)
 
     home2_xyz=(80,80,settlement_height)
     merchant=m.make('Dyfed Searae',type='merchant',desc='the pig merchant',
                     xyz=pig_sty_xyz,age=probability.fertility_age,
-                    sex='male',orientation=Quaternion(Vector3D([1,0,0]),Vector3D([-1,0,0])).as_list())
-    sty=m.make('sty',type='sty',xyz=pig_sty_xyz,status=1.0,bbox=[5,5,3])
+                    sex='male',orientation=Quaternion(Vector3D([1,0,0]),Vector3D([0,-1,0])).as_list())
+    sty=m.make('sty',type='sty',xyz=pig_sty_xyz)
     m.know(merchant,mknowledge)
     m.know(merchant,village)
     m.know(merchant,mprices)
@@ -283,19 +384,33 @@ def default(mapeditor):
     m.learn(merchant,(il.lunch,"meal(self, 'ham','midday', 'inn')"))
     m.learn(merchant,(il.sup,"meal(self, 'beer', 'evening', 'inn')"))
     m.learn(merchant,(il.welcome,"welcome('Welcome to this our settlement','settler')"))
-    m.learn(merchant,(il.help,"add_help(['Thankyou for joining our remote settlement.','Our first task is to build some shelter, but while we are doing that we still need food.','You can help us out by raising pigs for slaughter.','If you want to buy a piglet to raise, let me know by saying you would like to buy one.','Pigs love to eat acorns from under the oak trees that are abundant in this area.'])"))
+    m.learn(merchant,(il.help,"add_help(['Thankyou for joining our remote settlement.','Our first task is to build some shelter, but while we are doing that we still need food.','You can help us out by raising pigs for slaughter.','If you want to buy a piglet to raise, let me know by saying you would like to buy one.','Pigs love to eat acorns from under the oak trees that are abundant in this area.'],['I would like to buy a pig', 'Pehaps I will buy one later'])"))
     piglets=[]
     for i in range(0, 6):
         piglets.append(m.make('pig',type='pig',xyz=(uniform(0,4),uniform(0,4),settlement_height),parent=sty.id,orientation=directions[randint(0,7)]))
     m.learn(piglets,pig_goals)
     m.own(merchant,piglets)
 
+    marshall=m.make('Gorun Iksa',type='marshall',desc='the duke\'s marshall',
+                    xyz=(14,12,settlement_height), sex='male')
+    m.know(marshall, [('deed','price','50')])
+    m.know(marshall, village)
+    m.learn(marshall,(il.help,"add_help(['On behalf of the Duke I would like to welcome you to moraf.','If you are new here I suggest talking to the pig seller.','He will tell you what you can do to help out.','If you have decide you would like to settle here I can assign you some land','but you will need to show that you are a useful citizen.','If you can raise 50 coins herding pigs, then a plot of land is yours.'],['I would like to buy a deed','I will come back when I have raised some pigs'])"))
+    plots=[]
+    for i in range(20, 200, 20):
+        for j in range(-100, 100, 20):
+            plots.append(m.make('deed',xyz=(0,0,0),parent=marshall.id,plot=(i,j)))
+    m.own(marshall,plots)
+
     # Warriors - the more adventurous types
 
     warriors=[]
     warrior=m.make('Vonaa Barile',type='mercenary',xyz=(uniform(-2,2),uniform(-2,2),settlement_height),sex='female',orientation=directions[randint(0,7)])
     bow=m.make('bow',type='bow',xyz=(0,0,0), parent=warrior.id)
     m.own(warrior,bow)
+    for i in range(0, 6):
+        arrow=m.make('arrow',type='arrow',xyz=(0,0,0), parent=warrior.id)
+        m.own(warrior,arrow)
     warriors.append(warrior)
 
     warrior=m.make('Lile Birloc', type='mercenary',xyz=(uniform(-2,2),uniform(-2,2),settlement_height),sex='female',orientation=directions[randint(0,7)])
@@ -310,8 +425,8 @@ def default(mapeditor):
     m.know(warriors,village)
 
     # Warriors enjoy their food and drink
-    m.price(warriors, [('services','5')])
-    m.learn(warriors,(il.help,"add_help(['The forest is a dangerous place.','If you need some help protecting your pigs,','I can help you out for a day or so.','I will need some gold for food and equipment.','For 5 coins I can work for you until sundown.','After sundown you should make sure your pigs are safe,','and get indoors yourself.','If you want to hire my services,','let me know by saying you would like to hire me.'])"))
+    m.know(warriors, [('services','price','5')])
+    m.learn(warriors,(il.help,"add_help(['The forest is a dangerous place.','If you need some help protecting your pigs,','I can help you out for a day or so.','I will need some gold for food and equipment.','For 5 coins I can work for you until sundown.','After sundown you should make sure your pigs are safe,','and get indoors yourself.','If you want to hire my services,','let me know by saying you would like to hire me.'],['I would like to hire your services','I can take care of my pigs alone'])"))
     m.learn(warriors,(il.hire,"hire_trade()"))
     m.learn(warriors,(il.lunch,"meal(self, 'ham','midday', 'inn')"))
     m.learn(warriors,(il.sup,"meal(self, 'beer', 'evening', 'inn')"))
@@ -370,6 +485,8 @@ def add_village(mapeditor):
     m.make('house3',type='house3',xyz=(150,142,22),orientation=directions[2])
     m.make('house3',type='house3',xyz=(158,142,22),orientation=directions[5])
 
+    m.make('field',type='ploughed_field',xyz=(120,170,30),status=1.0,area={'points' : [ [0,0], [0,20], [20,20], [20,0] ], 'layer' : 8})
+    m.make('field',type='ploughed_field',xyz=(142,170,30),status=1.0,area={'points' : [ [0,0], [0,20], [20,20], [20,0] ], 'layer' : 8})
 def test_pig(mapeditor):
 #   general things
 
@@ -397,6 +514,7 @@ def test_forest(mapeditor):
         for j in range(0, i[1]):
             m.make(i[0],type=i[0],xyz=(uniform(i[2],i[3]),uniform(i[4],i[5]),i[6]), orientation=directions[randint(0,7)])
 
+   
 def modify_terrain(mapeditor):
 #   general things
 
@@ -404,25 +522,35 @@ def modify_terrain(mapeditor):
 
     world=m.look()
     points = { }
-    for i in range(-5, 6):
-        for j in range(-5, 6):
-            if i==5 or j==5:
+    for i in range(-6, 7):
+        for j in range(-6, 7):
+            if i>=5 or j>=5:
                 points['%ix%i'%(i,j)] = [i, j, uniform(100, 150)]
-            elif i==-5 or j == -5:
+            elif i<=-5 or j <= -5:
                 points['%ix%i'%(i,j)] = [i, j, uniform(-30, -10)]
             elif (i==2 or i==3) and (j==2 or j==3):
                 points['%ix%i'%(i,j)] = [i, j, uniform(20, 25)]
             elif i==4 or j==4:
                 points['%ix%i'%(i,j)] = [i, j, uniform(30, 80)]
             elif i==-4 or j==-4:
-                points['%ix%i'%(i,j)] = [i, j, uniform(-2, 5)]
+                points['%ix%i'%(i,j)] = [i, j, uniform(-5, 5)]
             else:
-                points['%ix%i'%(i,j)] = [i, j, 1+uniform(0, 8)*(abs(i)+abs(j))]
+                points['%ix%i'%(i,j)] = [i, j, 1+uniform(3, 11)*(abs(i)+abs(j))]
 
+    points['-4x-1'] = [-4, -1, 12.4]
+    points['-4x-2'] = [-4, -2, -8.3]
+    points['-3x-2'] = [-3, -2, -6.2]
+    points['-3x-1'] = [-3, -1, -5.3]
+    points['-2x-1'] = [-2, -1, -4.1]
     points['-1x-1'] = [-1, -1, -16.8]
     points['0x-1'] = [0, -1, -3.8]
     points['-1x0'] = [-1, 0, -2.8]
-    points['-1x1'] = [-1, 1, 12.8]
+    points['-1x1'] = [-1, 1, -1.8]
+    points['-1x2'] = [-1, 2, -1.7]
+    points['0x2'] = [0, 2, -1.6]
+    points['1x2'] = [1, 2, -1.3]
+    points['1x3'] = [1, 3, -1.1]
+    points['1x4'] = [1, 4, -0.6]
     points['1x-1'] = [1, -1, 15.8]
     points['0x0'] = [0, 0, 12.8]
     points['1x0'] = [1, 0, 23.1]
@@ -437,12 +565,104 @@ def test_coll(mapeditor):
 
     sty=m.make('sty',type='sty',xyz=pig_sty_xyz,status=1.0,bbox=[5,5,3], orientation=directions[0])
 
-def test_deer(mapeditor):
+def test_butcher(mapeditor):
 
     m=editor(mapeditor)
 
-    deer=m.make('deer', type='deer', xyz=(10, 10, settlement_height))
-    m.learn(deer,deer_goals)
+    butcher=m.make('Ulad Bargan',type='butcher',desc='the butcher',
+                 xyz=(3,3,0),age=probability.fertility_age,sex='male')
+    cleaver=m.make('cleaver', type='cleaver', desc='cleaver for cutting meat',
+                   place='market', xyz=(3,2.5,settlement_height))
+    m.own(butcher,cleaver)
+    m.learn(butcher,(il.trade,"trade(self, 'pig', 'cleaver', 'cut', 'ham', 'market')"))
+    piglet = m.make('pig', type='pig', xyz=(3,2,0))
 
-    chicken=m.make('chicken', type='chicken', xyz=(5, 10, settlement_height))
-    m.learn(chicken,chicken_goals)
+def test_pig(mapeditor):
+
+    m=editor(mapeditor)
+
+    piglet = m.make('pig', type='pig', xyz=(-3,-1,settlement_height))
+    m.learn(piglet,pig_goals)
+
+def test_settler(mapeditor):
+
+    m=editor(mapeditor)
+    settler=m.make('settler',xyz=(1,1,0), sex='male')
+    axe=m.make('axe',type='axe',xyz=(0,0,0),parent=settler.id)
+    m.own(settler,axe)
+    m.know(settler,[('forest','location',(30,30,0))])
+    m.learn(settler,(il.trade,"harvest_resource(self,'lumber','oak','forest','axe')"))
+    m.make('oak',xyz=(32,32,0))
+
+def test_forester(mapeditor):
+
+    m=editor(mapeditor)
+    # An NPC forester
+    settler=m.make('forester',type='settler',xyz=(0, 12, 0), sex='male')
+    trowel=m.make('trowel',xyz=(0,0,0),parent=settler.id)
+    m.own(settler, trowel)
+    m.know(settler,[('forest','location',(30,30,0))])
+    m.learn(settler,(il.trade,"plant_seeds(self,'acorn','oak','forest','trowel')"))
+    m.make('oak',xyz=(32,32,0))
+
+def test_chicken(mapeditor):
+
+    m=editor(mapeditor)
+
+    chuck = m.make('chicken', type='chicken', xyz=(-3,-1,settlement_height))
+    m.learn(chuck,chicken_goals)
+
+def test_path(mapeditor):
+
+    m=editor(mapeditor)
+    path_area={'points' : [ [2,0], [22, 40], [132,122], [140,127], [144.5, 146.5], [169, 153], [169,155], [142.5,148.5], [138,129], [130,124], [18,40], [-2,-1] ], 'layer' : 7}
+    m.make('path to village',type='path',xyz=(10, 20,settlement_height), area=path_area,bbox=[169,154,1])
+   
+    
+graveyard_height = 0
+
+lych2_knowledge=[('w1','location',(-140,110,graveyard_height)),
+                ('w2','location',(-50,160,graveyard_height)),
+                ('w3','location',(-80,40,graveyard_height)),
+                ('w4','location',(-180,150,graveyard_height))]
+
+lych2_goals=[(il.assemble, "assemble(self, 'skeleton', ['skull', 'ribcage', 'arm', 'pelvis', 'thigh', 'shin'])"),
+            (il.patrol,"patrol(['w1', 'w2', 'w3', 'w4'])")]
+   
+def test_graveyard(mapeditor):
+    m = editor(mapeditor)
+    
+    m.make('mausoluem of harrington', type='mausoleum', xyz=(-160, 105, graveyard_height), orientation=directions[0])
+    m.make('steps',type='wall',xyz=(-160,100,graveyard_height),bbox=[4,4,2])
+    
+    graveyard_area={'points': [[-10, -8], [15, -11], [13,23], [-8, 8]], 'layer':7 }
+    m.make('graveyard', xyz=(-150, 110, graveyard_height), type='path', area=graveyard_area, bbox=[-10, -11,0, 15, 23, 1])
+    
+    path_area={'points' : [ [2, 0], [20, 20], [45, 60], [65, 70], [83, 40], [106, -15], [105, -12], [82, 36], [66, 67], [44, 57], [18, 16], [0, -4]], 'layer' : 7}
+    m.make('path to graveyard',type='path',xyz=(-150, 110, graveyard_height), area=path_area,bbox=[100,8,1])
+
+    m.make('gravestone1', type='gravestone_rounded', xyz=(-145, 107, graveyard_height), orientation=directions[1])
+    m.make('gravestone2', type='gravestone_simple', xyz=(-147, 104, graveyard_height), orientation=directions[2])
+    m.make('gravestone3', type='gravestone_stylish', xyz=(-148, 109, graveyard_height), orientation=directions[3])
+    m.make('gravestone4', type='gravestone_rounded', xyz=(-150, 117, graveyard_height), orientation=directions[2])
+    m.make('gravestone5', type='gravestone_rounded', xyz=(-143, 111, graveyard_height), orientation=directions[4])
+    m.make('gravestone6', type='gravestone_stylish', xyz=(-149, 110, graveyard_height), orientation=directions[3])
+    m.make('gravestone7', type='gravestone_stylish', xyz=(-147, 116, graveyard_height), orientation=directions[2])
+    
+    lych=m.make('lych', type='lych', xyz=(-140, 130, graveyard_height))
+    m.learn(lych,lych2_goals)
+    m.know(lych,lych2_knowledge)
+    m.tell_importance(lych,il.assemble,'>',il.patrol)
+
+def test_fire(mapeditor):
+    m = editor(mapeditor)
+
+    cfire=m.make('campfire',type='campfire',xyz=(3,9,settlement_height))
+    # m.make('fire',type='fire',xyz=(0,0,0),parent=cfire.id)
+
+def test_own(mapeditor):
+
+    m=editor(mapeditor)
+    settler=m.make('settler',xyz=(1,1,0), sex='male')
+    axe=m.make('axe',type='axe',xyz=(0,0,0),parent=settler.id)
+    m.own(settler,axe)
