@@ -12,14 +12,14 @@ class Fircone(Seed):
         ent=Entity(self.id,status=-1)
         res = Operation("set",ent,to=self)
         to_ = op.from_
-        nour=Entity(to_.id,mass=self.mass)
+        nour=Entity(to_,mass=self.mass)
         res = res + Operation("nourish",nour,to=to_)
         return res
     def setup_operation(self, op):
         # Decay a bit, we don't last forever
         # Of course, a real acorn would germinate after a bit
         opTick=Operation("tick",to=self)
-        opTick.time.sadd=1800
+        opTick.setFutureSeconds(1800)
         return opTick
     def tick_operation(self, op):
         ent=Entity(self.id,status=-1)
