@@ -3,7 +3,10 @@ from atlas import *
 from world.objects.Thing import Thing
 from misc import set_kw
 from common import log,const
-from whrandom import *
+try:
+  from random import *
+except ImportError:
+  from whrandom import *
 
 import atlas
 
@@ -16,12 +19,9 @@ Base class from which all fruit/seed entities are derived.
 """
 
 class Seed(Thing):
-    def __init__(self, cppthing, **kw):
-        self.base_init(cppthing, kw)
-        set_kw(self,kw,"mass",0.05)
     def tick_operation(self, op): pass
 #        opTick=Operation("tick",to=self)
-#        opTick.time.sadd=const.basic_tick*speed
+#        opTick.setFutureSeconds(const.basic_tick*speed)
 #
 #        result = atlas.Message(opTick)
 #
