@@ -28,9 +28,10 @@ class Lobby : public OOGThing {
         return m_accounts;
     }
 
-    virtual void operation(const RootOperation &, OpVector &);
+    virtual void operation(const Operation &, OpVector &);
 
-    virtual void addToMessage(MapType &) const;
+    virtual void addToMessage(Atlas::Message::MapType &) const;
+    virtual void addToEntity(const Atlas::Objects::Entity::RootEntity &) const;
 };
 
 #endif // SERVER_LOBBY_H
