@@ -5,15 +5,18 @@
 #ifndef SERVER_PERSISTANCE_H
 #define SERVER_PERSISTANCE_H
 
-#include "common/types.h"
-
 #include <Atlas/Message/Element.h>
 
 #include <string>
 
+class Entity;
 class Account;
 class Database;
 
+typedef std::map<std::string, Entity *> EntityDict;
+
+/// \brief Class for managing the required database tables for persisting
+/// in-game entities and server accounts
 class Persistance {
   private:
     Persistance();
@@ -35,7 +38,7 @@ class Persistance {
     void addCharacter(const Account &, const Entity &);
     void delCharacter(const std::string &);
     
-    bool getRules(MapType & m);
+    bool getRules(Atlas::Message::MapType & m);
     bool clearRules();
 };
 
