@@ -14,6 +14,14 @@ void PersistorConnection<T>::persist()
     m_p.persist(m_t);
 }
 
+template <class T>
+PersistantThingFactory<T>::~PersistantThingFactory()
+{
+    if (m_master) {
+        delete &m_p;
+    }
+}
+
 template <class T>
 T * PersistantThingFactory<T>::newThing(const std::string & id)
 {
@@ -34,4 +42,28 @@ FactoryBase * PersistantThingFactory<T>::duplicateFactory()
     return new PersistantThingFactory<T>(*this);
 }
 
+template <class T>
+ForbiddenThingFactory<T>::~ForbiddenThingFactory()
+{
+    delete &m_p;
+}
+
+template <class T>
+T * ForbiddenThingFactory<T>::newThing(const std::string &)
+{
+    return 0;
+}
+
+template <class T>
+T * ForbiddenThingFactory<T>::newPersistantThing(const std::string &, PersistorBase **)
+{
+    return 0;
+}
+
+template <class T>
+FactoryBase * ForbiddenThingFactory<T>::duplicateFactory()
+{
+    return 0;
+}
+
 #endif // SERVER_PERSISTANT_THING_FACTORY_IMPL_H
