@@ -7,10 +7,12 @@
 
 #include "Account.h"
 
+/// \brief This is a class for handling users who are regular players
 class Player : public Account {
   protected:
-    virtual int characterError(const Create &,
-                               const MapType &, OpVector &) const;
+    virtual int characterError(const Operation &,
+                               const Atlas::Objects::Entity::RootEntity &,
+                               OpVector &) const;
   public:
     Player(Connection * conn, const std::string & username,
                               const std::string & passwd,
@@ -19,7 +21,8 @@ class Player : public Account {
 
     virtual const char * getType() const;
 
-    virtual void addToMessage(MapType &) const;
+    virtual void addToMessage(Atlas::Message::MapType &) const;
+    virtual void addToEntity(const Atlas::Objects::Entity::RootEntity &) const;
 
     static std::set<std::string> playableTypes;
 };
