@@ -8,7 +8,6 @@
 #include "Account.h"
 
 #include "common/OOGThing.h"
-#include "common/serialno.h"
 
 class BaseWorld;
 class Lobby;
@@ -40,8 +39,6 @@ class ServerRouting : public OOGThing {
                   const std::string & name);
     ~ServerRouting();
 
-    bool idle();
-
     /// Add an OOG object to the server.
     void addObject(BaseEntity * obj) {
         m_objects[obj->getId()] = obj;
@@ -53,11 +50,6 @@ class ServerRouting : public OOGThing {
         addObject(a);
     }
 
-    /// Generate a new operation serial number.
-    int newSerialNo() {
-        return opSerialNo();
-    }
-
     /// Accessor for OOG objects map.
     const BaseDict & getObjects() const {
         return m_objects;
@@ -103,7 +95,8 @@ class ServerRouting : public OOGThing {
     const std::string & getName() const { return m_svrName; }
 
     /// Copy the attribute values of this object into an Atlas message.
-    virtual void addToMessage(MapType &) const;
+    virtual void addToMessage(Atlas::Message::MapType &) const;
+    virtual void addToEntity(const Atlas::Objects::Entity::RootEntity &) const;
 };
 
 #endif // SERVER_SERVER_ROUTING_H
