@@ -1,6 +1,6 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2000,2001 Alistair Riddoch
+// Copyright (C) 2000-2004 Alistair Riddoch
 
 #include "WorldRouter.h"
 
@@ -8,32 +8,70 @@
 
 #include "rulesets/World.h"
 
+#include "common/id.h"
 #include "common/log.h"
 #include "common/debug.h"
 #include "common/const.h"
 #include "common/globals.h"
-#include "common/stringstream.h"
-#include "common/globals.h"
 #include "common/Database.h"
 #include "common/random.h"
-#include "common/refno.h"
+#include "common/serialno.h"
+#include "common/compose.hpp"
 
 #include "common/Setup.h"
 
-#include <Atlas/Objects/Operation/Look.h>
-#include <Atlas/Objects/Operation/Sight.h>
+#include <Atlas/Objects/Operation.h>
+#include <Atlas/Objects/RootEntity.h>
+
+#include <sstream>
+#include <algorithm>
+
+using Atlas::Message::Element;
+using Atlas::Objects::Operation::Setup;
+using Atlas::Objects::Entity::RootEntity;
 
 static const bool debug_flag = false;
 
+struct OpQueEntry {
+    Operation op;
+    Entity & from;
+
+    explicit OpQueEntry(const Operation & o, Entity & f);
+    OpQueEntry(const OpQueEntry & o);
+    ~OpQueEntry();
+
+    const Operation & operator*() const {
+        return op;
+    }
+
+    Atlas::Objects::Operation::RootOperationData * operator->() const {
+        return op.get();
+    }
+};
+
+inline OpQueEntry::OpQueEntry(const Operation & o, Entity & f) : op(o), from(f)
+{
+    from.incRef();
+}
+
+inline OpQueEntry::OpQueEntry(const OpQueEntry & o) : op(o.op), from(o.from)
+{
+    from.incRef();
+}
+
+inline OpQueEntry::~OpQueEntry()
+{
+    from.decRef();
+}
+
+
 /// \brief Update the in-game time.
 ///
 /// Reads the system time, and applies the necessary offsets to calculate
 /// the in-game time. This is the stored, and can be accessed using getTime().
-void WorldRouter::updateTime()
+void WorldRouter::updateTime(int sec, int usec)
 {
-    struct timeval tv;
-    gettimeofday(&tv, NULL);
-    double tmp_time = (double)(tv.tv_sec + timeoffset - m_initTime) + (double)tv.tv_usec/1000000;
+    double tmp_time = (double)(sec + timeoffset - m_initTime) + (double)usec/1000000;
     m_realTime = tmp_time;
 }
 
@@ -45,10 +83,11 @@ void WorldRouter::updateTime()
 /// but I am not clear why. Need to look into why.
 WorldRouter::WorldRouter() : BaseWorld(*new World(consts::rootWorldId))
 {
-    // setId(consts::rootWorldId);
-    m_initTime = time(NULL);
-    updateTime();
-    // m_gameWorld.setId(getId());
+    struct timeval tv;
+    gettimeofday(&tv, NULL);
+    m_initTime = tv.tv_sec;
+    updateTime(tv.tv_sec, tv.tv_usec);
+    m_gameWorld.incRef();
     m_gameWorld.m_world = this;
     m_gameWorld.setType("world");
     m_eobjects[m_gameWorld.getId()] = &m_gameWorld;
@@ -64,26 +103,17 @@ WorldRouter::WorldRouter() : BaseWorld(*new World(consts::rootWorldId))
 /// the server, clears the operation queue
 WorldRouter::~WorldRouter()
 {
-    OpQueue::const_iterator I = m_operationQueue.begin();
-    { 
-        debug(std::cout << "Flushing op queue with " << m_operationQueue.size()
-                        << " ops" << std::endl << std::flush;);
-    }
-    for (; I != m_operationQueue.end(); I++) {
-        delete *I;
-    }
     { 
         debug(std::cout << "Flushing world with " << m_eobjects.size()
                         << " entities" << std::endl << std::flush;);
     }
-    m_eobjects.erase(m_gameWorld.getId());
-    EntityDict::const_iterator J = m_eobjects.begin();
-    for(; J != m_eobjects.end(); J++) {
-        delete J->second;
+    EntityDict::const_iterator Jend = m_eobjects.end();
+    for (EntityDict::const_iterator J = m_eobjects.begin(); J != Jend; ++J) {
+        J->second->decRef();
     }
     // This should be deleted here rather than in the base class because
     // we created it, and BaseWorld should not even know what it is.
-    delete &m_gameWorld;
+    m_gameWorld.decRef();
 }
 
 /// \brief Add an operation to the ordered op queue.
@@ -94,23 +124,21 @@ WorldRouter::~WorldRouter()
 /// the entity that is responsible for adding the operation to the
 /// queue, unless it is set to "cheat". This is used to spook
 /// operations when they come from an admin.
-void WorldRouter::addOperationToQueue(RootOperation & op,
-                         const Entity * obj)
+void WorldRouter::addOperationToQueue(const Operation & op, Entity & ent)
 {
-    if (op.getFrom() == "cheat") {
-        op.setFrom(op.getTo());
+    assert(op.isValid());
+    if (op->getFrom() == "cheat") {
+        op->setFrom(op->getTo());
     } else {
-        op.setFrom(obj->getId());
+        op->setFrom(ent.getId());
     }
-    updateTime();
-    double t = m_realTime;
-    t = t + op.getFutureSeconds();
-    op.setSeconds(t);
-    op.setFutureSeconds(0.0);
-    OpQueue::iterator I;
-    for(I = m_operationQueue.begin();
-        (I != m_operationQueue.end()) && ((*I)->getSeconds() <= t) ; I++);
-    m_operationQueue.insert(I, &op);
+    double t = m_realTime + op->getFutureSeconds();
+    op->setSeconds(t);
+    op->setFutureSeconds(0.0);
+    OpQueue::iterator I = m_operationQueue.begin();
+    OpQueue::iterator Iend = m_operationQueue.end();
+    for (; (I != Iend) && ((*I).op->getSeconds() <= t) ; ++I);
+    m_operationQueue.insert(I, OpQueEntry(op, ent));
 }
 
 /// \brief Get the next due operation from the queue.
@@ -118,27 +146,18 @@ void WorldRouter::addOperationToQueue(RootOperation & op,
 /// If the operation at the end of the queue is now due, return it.
 /// @return a pointer to the operation due for dispatch, or 0 if none
 /// is due.
-RootOperation * WorldRouter::getOperationFromQueue()
+Operation WorldRouter::getOperationFromQueue()
 {
-    std::list<RootOperation *>::const_iterator I = m_operationQueue.begin();
+    OpQueue::const_iterator I = m_operationQueue.begin();
     if ((I == m_operationQueue.end()) || ((*I)->getSeconds() > m_realTime)) {
         return NULL;
     }
     debug(std::cout << "pulled op off queue" << std::endl << std::flush;);
-    RootOperation * op = *I;
+    const Operation & op = (**I);
     m_operationQueue.pop_front();
     return op;
 }
 
-/// \brief Give an operation a new serial number.
-///
-/// Call newSerialNo() to get a new serial number,
-/// and assign it to the operation provided.
-inline void WorldRouter::setSerialnoOp(RootOperation & op)
-{
-    op.setSerialno(newSerialNo());
-}
-
 /// \brief Provide an adjusted heigh for the given entity.
 ///
 /// If the position has a parent which has an associated geometry
@@ -147,29 +166,42 @@ inline void WorldRouter::setSerialnoOp(RootOperation & op)
 /// This function recurses through the parents until it finds
 /// A parent which defines the height.
 /// @return the modified Z coord of the position.
-float WorldRouter::constrainHeight(Entity * parent, const Point3D & pos)
+float WorldRouter::constrainHeight(Entity * parent, const Point3D & pos,
+                                   const std::string & mode)
 {
     assert(parent != 0);
     World * wrld = dynamic_cast<World*>(parent);
     if (wrld != 0) {
-        float h = wrld->getHeight(pos.x(), pos.y());
+        float h;
+        h = wrld->getHeight(pos.x(), pos.y());
+        if (mode == "fixed") {
+            h = pos.z();
+        } else if (mode == "floating") {
+            h = 0;
+        } else if (mode == "swimming") {
+            h = std::max(h, std::min(0.f, pos.z()));
+        }
         debug(std::cout << "Fix height " << pos.z() << " to " << h
                         << std::endl << std::flush;);
         return h;
     } else {
+        static const Quaternion identity(Quaternion().identity());
         assert(parent->m_location.m_loc != 0);
         // FIXME take account of orientation
-        const Point3D & ppos = parent->m_location.m_pos;
+        const Point3D & ppos = parent->m_location.pos();
         debug(std::cout << "parent " << parent->getId() << " of type "
                         << parent->getType() << " pos " << ppos.z()
                         << " my pos " << pos.z()
                         << std::endl << std::flush;);
         float h;
-        // FIXME Is it safe to use m_orientation without checking it
-        h = constrainHeight(parent->m_location.m_loc,
-                            pos.toParentCoords(parent->m_location.m_pos,
-                                               parent->m_location.m_orientation)
-                           ) - ppos.z();
+        // FIXME Recently swapped argument order as it appears to fix
+        // a bug. Check carefully whether this is doing the right thing.
+        const Quaternion & parent_orientation = parent->m_location.orientation().isValid() ? parent->m_location.orientation() : identity;
+        h =  ppos.z() - constrainHeight(parent->m_location.m_loc,
+                            pos.toParentCoords(parent->m_location.pos(),
+                                               parent_orientation),
+                            mode
+                           );
         debug(std::cout << "Correcting height from " << pos.z() << " to " << h
                         << std::endl << std::flush;);
         return h;
@@ -184,101 +216,122 @@ float WorldRouter::constrainHeight(Entity * parent, const Point3D & pos)
 /// entity into the loc/contains tree maintained by the Entity
 /// class. Handle omnipresent entities, and sending a Setup op
 /// to the entity.
-Entity * WorldRouter::addObject(Entity * obj, bool setup)
+Entity * WorldRouter::addEntity(Entity * ent, bool setup)
 {
-    debug(std::cout << "WorldRouter::addObject(Entity *)" << std::endl
+    debug(std::cout << "WorldRouter::addEntity(Entity *)" << std::endl
                     << std::flush;);
-    m_eobjects[obj->getId()] = obj;
-    m_objectList.insert(obj);
-    assert(obj->m_location.isValid());
+    m_eobjects[ent->getId()] = ent;
+    m_objectList.insert(ent);
+    assert(ent->m_location.isValid());
 
-    if (!obj->m_location.isValid()) {
+    if (!ent->m_location.isValid()) {
         log(ERROR, "Entity added to world with invalid location!");
         debug(std::cout << "set loc " << &m_gameWorld  << std::endl
                         << std::flush;);
-        obj->m_location.m_loc = &m_gameWorld;
-        obj->m_location.m_pos = Point3D(uniform(-8,8), uniform(-8,8), 0);
-        debug(std::cout << "loc set with loc " << obj->m_location.m_loc->getId()
+        ent->m_location.m_loc = &m_gameWorld;
+        ent->m_location.m_pos = Point3D(uniform(-8,8), uniform(-8,8), 0);
+        debug(std::cout << "loc set with loc " << ent->m_location.m_loc->getId()
                         << std::endl << std::flush;);
     }
-    obj->m_location.m_pos.z() = constrainHeight(obj->m_location.m_loc,
-                                                obj->m_location.m_pos);
-    bool cont_change = obj->m_location.m_loc->m_contains.empty();
-    obj->m_location.m_loc->m_contains.insert(obj);
-    if (cont_change) {
-        obj->m_location.m_loc->m_update_flags |= a_cont;
-        obj->m_location.m_loc->updated.emit();
+    ent->m_location.update(getTime());
+    // FIXME
+    std::string mode;
+    if (ent->has("mode")) {
+        Element mode_attr;
+        ent->get("mode", mode_attr);
+        if (mode_attr.isString()) {
+            mode = mode_attr.String();
+        } else {
+            log(ERROR, String::compose("Mode on entity is a %1 in WorldRouter::addEntity", Element::typeName(mode_attr.getType())).c_str());
+        }
     }
-    debug(std::cout << "Entity loc " << obj->m_location << std::endl
+    ent->m_location.m_pos.z() = constrainHeight(ent->m_location.m_loc,
+                                                ent->m_location.pos(), mode);
+    bool cont_change = ent->m_location.m_loc->m_contains.empty();
+    ent->m_location.m_loc->m_contains.insert(ent);
+    ent->m_location.m_loc->incRef();
+    if (cont_change) {
+        ent->m_location.m_loc->m_update_flags |= a_cont;
+        ent->m_location.m_loc->updated.emit();
+    }
+    debug(std::cout << "Entity loc " << ent->m_location << std::endl
                     << std::flush;);
-    obj->m_world = this;
+    ent->m_world = this;
     if (consts::enable_omnipresence &&
-        (obj->getAttributes().find("omnipresent") !=
-         obj->getAttributes().end())) {
-        m_omnipresentList.insert(obj);
+        (ent->getAttributes().find("omnipresent") !=
+         ent->getAttributes().end())) {
+        m_omnipresentList.insert(ent);
     }
     if (setup) {
-        Setup * s = new Setup;
-        s->setTo(obj->getId());
+        Setup s;
+        s->setTo(ent->getId());
         s->setFutureSeconds(-0.1);
         s->setSerialno(newSerialNo());
-        message(*s, &m_gameWorld);
+        message(s, m_gameWorld);
     }
-    return (obj);
+    return ent;
 }
 
 /// \brief Create a new entity and add to the world.
 ///
 /// Construct a new entity using the entity description provided,
-/// and pass it to addObject().
+/// and pass it to addEntity().
 /// @return a pointer to the new entity.
-Entity * WorldRouter::addNewObject(const std::string & typestr,
-                                   const MapType & ent)
+Entity * WorldRouter::addNewEntity(const std::string & typestr,
+                                   const RootEntity & attrs)
 {
-    debug(std::cout << "WorldRouter::addNewObject(\"" << typestr << "\", ent)"
+    debug(std::cout << "WorldRouter::addNewEntity(\"" << typestr << "\", attrs)"
                     << std::endl << std::flush;);
     std::string id;
-    Database::instance()->newId(id);
-    Entity * obj = EntityFactory::instance()->newEntity(id, typestr, ent);
-    if (obj == 0) {
+    if (consts::enable_database) {
+        Database::instance()->newId(id);
+    } else {
+        newId(id);
+    }
+    assert(!id.empty());
+    Entity * ent = EntityFactory::instance()->newEntity(id, typestr, attrs);
+    if (ent == 0) {
         std::string msg = std::string("Attempt to create an entity of type \"")
-                          + typestr + "\" but type is unknown";
+                          + typestr + "\" but type is unknown or forbidden";
         log(ERROR, msg.c_str());
         return 0;
     }
-    return addObject(obj);
+    return addEntity(ent);
 }
 
 /// \brief Remove an entity from the world.
 ///
 /// Remove an entity from the various lists in which it is stored.
-/// The entity is not deleted, nor any attend made to handle
-/// the loc/contains. It would probably be a good idea to move
-/// some of this handling to this function.
-void WorldRouter::delObject(Entity * obj)
+/// The entity is removed from the LOC/CONTAINS tree, and the
+/// reference held by the world is decremented. There may still be
+/// a reference held by an operation in the queue from the removed
+/// entity.
+void WorldRouter::delEntity(Entity * ent)
 {
-    if (consts::enable_omnipresence) {
-        m_omnipresentList.erase(obj);
+    if (ent == &m_gameWorld) {
+        log(WARNING, "Attempt to delete game world");
+        return;
     }
-    m_perceptives.erase(obj);
-    m_objectList.erase(obj);
-    m_eobjects.erase(obj->getId());
+    if (consts::enable_omnipresence) {
+        m_omnipresentList.erase(ent);
+    }
+    m_perceptives.erase(ent);
+    m_objectList.erase(ent);
+    m_eobjects.erase(ent->getId());
+    ent->destroy();
+    ent->decRef();
 }
 
 /// \brief Pass an operation to the World.
 ///
 /// Pass an operation to addOperationToQueue()
 /// so it gets added to the queue for dispatch.
-void WorldRouter::message(RootOperation & op, const Entity * obj)
+void WorldRouter::message(const Operation & op, Entity & ent)
 {
+    addOperationToQueue(op, ent);
     debug(std::cout << "WorldRouter::message {"
-                    << op.getParents().front().asString() << ":"
-                    << op.getFrom() << ":" << op.getTo() << "}" << std::endl
-                    << std::flush;);
-    addOperationToQueue(op, obj);
-    debug(std::cout << "WorldRouter::message {"
-                    << op.getParents().front().asString() << ":"
-                    << op.getFrom() << ":" << op.getTo() << "}" << std::endl
+                    << op->getParents().front() << ":"
+                    << op->getFrom() << ":" << op->getTo() << "}" << std::endl
                     << std::flush;);
 }
 
@@ -290,21 +343,22 @@ void WorldRouter::message(RootOperation & op, const Entity * obj)
 /// This should probably go, as there is essentially no sane reason
 /// for broadcasting a random op to all entities.
 /// @return a reference to the list of entities to be used for braodcast.
-const EntitySet & WorldRouter::broadcastList(const RootOperation & op) const
+const EntitySet & WorldRouter::broadcastList(const Operation & op) const
 {
-    const ListType & parents = op.getParents();
-    if (!parents.empty() && (parents.front().isString())) {
-        const std::string & parent = parents.front().asString();
+    // FIXME Use numeric types
+    const std::list<std::string> & parents = op->getParents();
+    if (!parents.empty()) {
+        const std::string & parent = parents.front();
         if ((parent == "sight") || (parent == "sound") ||
             (parent == "appearance") || (parent == "disappearance")) {
             return m_perceptives;
         }
         std::string msg = std::string("Broadcasting ") + parent + " op from "
-                                                       + op.getFrom();
+                                                       + op->getFrom();
         log(WARNING, msg.c_str());
     } else {
         std::string msg = std::string("Broadcasting op with no parent from ")
-                                                       + op.getFrom();
+                                                       + op->getFrom();
         log(ERROR, msg.c_str());
     }
     return m_objectList;
@@ -315,66 +369,34 @@ const EntitySet & WorldRouter::broadcastList(const RootOperation & op) const
 /// Pass the operation to the target entity. The resulting operations
 /// have their ref numbers set, and are added to the queue for
 /// dispatch.
-void WorldRouter::deliverTo(const RootOperation & op, Entity * e)
+void WorldRouter::deliverTo(const Operation & op, Entity & ent)
 {
     OpVector res;
-    e->operation(op, res);
-    setRefno(res, op);
-    for(OpVector::const_iterator I = res.begin(); I != res.end(); I++) {
-        setSerialnoOp(**I);
-        message(**I, e);
-    }
-}
-
-/// \brief Special version of WorldRouter::deliverTo() for delete ops.
-///
-/// Delete is special. It causes the target to be removed, but
-/// we need to handle the responses first. To prevent a tight loop,
-/// we do not attempt to immediatly handle the response to a delete op
-/// if it is anothe delete op.
-void WorldRouter::deliverDeleteTo(const RootOperation & op, Entity * e)
-{
-    OpVector res;
-    e->operation(op, res);
-    setRefno(res, op);
-    for(OpVector::const_iterator I = res.begin(); I != res.end(); I++) {
-        RootOperation & newOp = **I;
-        setSerialnoOp(newOp);
-        if (newOp.getParents().front().asString() == "delete") {
-            // If this is a delete, queue as normal to avoid a recursive loop
-            debug(std::cerr << "Handling delete response to delete"
-                            << std::endl << std::flush;);
-            log(WARNING, "Response to Delete op is another Delete.");
-            message(newOp, e);
-        } else {
-            // Other ops we dispatch immediatly before deleting the source
-            debug(std::cerr << "Handling normal response to delete"
-                            << std::endl << std::flush;);
-            newOp.setFrom(e->getId());
-            operation(newOp);
-            delete &newOp;
+    ent.operation(op, res);
+    OpVector::const_iterator Iend = res.end();
+    for(OpVector::const_iterator I = res.begin(); I != Iend; ++I) {
+        if (op->getFrom() == (*I)->getTo()) {
+            (*I)->setRefno(op->getSerialno());
         }
+        (*I)->setSerialno(newSerialNo());
+        message(*I, ent);
     }
-    if (e == &m_gameWorld) {
-        log(WARNING, "Attempt to delete game world");
-        return;
-    }
-    delObject(e);
-    e->destroy();
-    delete e;
 }
 
-/// Main in-game operation dispatch function.
+/// \brief Main in-game operation dispatch function.
+///
 /// Operations are passed here when they are due for dispatch.
 /// Determine the target of the operation and deliver it directly,
 /// or broadcast if broadcast is required. This function implements
 /// sight ranges for perception operations.
-void WorldRouter::operation(const RootOperation & op)
+/// @param op operation to be dispatched to the world. This is non-const
+/// so that broadcast ops can have their TO set correctly for each target.
+void WorldRouter::operation(const Operation & op, Entity & from)
 {
-    const std::string & to = op.getTo();
+    const std::string & to = op->getTo();
     debug(std::cout << "WorldRouter::operation {"
-                    << op.getParents().front().asString() << ":"
-                    << op.getFrom() << ":" << to << "}" << std::endl
+                    << op->getParents().front() << ":"
+                    << op->getFrom() << ":" << to << "}" << std::endl
                     << std::flush;);
 
     if (!to.empty()) {
@@ -388,56 +410,37 @@ void WorldRouter::operation(const RootOperation & op)
         Entity * to_entity = I->second;
         assert(to_entity != 0);
 
-        if (op.getParents().front().asString() == "delete") {
-            deliverDeleteTo(op, to_entity);
-        } else {
-            deliverTo(op, to_entity);
+        deliverTo(op, *to_entity);
+        if (op->getParents().front() == "delete") { // FIXME numeric type
+            delEntity(to_entity);
         }
     } else {
-        RootOperation newop = op;
+        // Where broadcasts go depends on type of op
         const EntitySet & broadcast = broadcastList(op);
-        const std::string & from = newop.getFrom();
-        EntityDict::const_iterator J = m_eobjects.find(from);
-        if (from.empty()) {
-            log(ERROR, "WorldRouter::operation dispatching op with no from");
-            if (J != m_eobjects.end()) {
-                log(ERROR, "WorldRouter::operation empty id found in IG map");
-            }
-        }
-        if ((J == m_eobjects.end()) || (!consts::enable_ranges)) {
-            log(ERROR, "WorldRouter::operation broadcasting op with missing from");
-            std::cerr << op.getParents().front().asString() << " op from "
-                      << from << std::endl << std::flush;
-            EntitySet::const_iterator I;
-            for(I = broadcast.begin(); I != broadcast.end(); I++) {
-                newop.setTo((*I)->getId());
-                deliverTo(newop, *I);
+        assert(op->getFrom() == from.getId());
+        if (!consts::enable_ranges) {
+            EntitySet::const_iterator I = broadcast.begin();
+            EntitySet::const_iterator Iend = broadcast.end();
+            for (; I != Iend; ++I) {
+                op->setTo((*I)->getId());
+                deliverTo(op, **I);
             }
         } else {
-            Entity * fromEnt = J->second;
-            assert(fromEnt != NULL);
-            float fromSquSize = boxSquareSize(fromEnt->m_location.m_bBox);
-            EntitySet::const_iterator I;
-            for(I = broadcast.begin(); I != broadcast.end(); I++) {
+            float fromSquSize = boxSquareSize(from.m_location.bBox());
+            EntitySet::const_iterator I = broadcast.begin();
+            EntitySet::const_iterator Iend = broadcast.end();
+            for (; I != Iend; ++I) {
                 // Calculate square distance to target
-                float dist = squareDistance(fromEnt->m_location, (*I)->m_location);
+                float dist = squareDistance(from.m_location, (*I)->m_location);
                 float view_factor = fromSquSize / dist;
-#if 0
-                if (view_factor > consts::square_sight_factor) {
-                    std::cout << "Distance from " << fromEnt->getType() << " to "
-                              << (*I)->getType() << " is " << dist << " which gives "
-                              << fromSquSize << " / " << dist << " = "
-                              << view_factor << std::endl << std::flush;
-                }
-#endif
                 if (view_factor < consts::square_sight_factor) {
-                    debug(std::cout << "Op from " << from
+                    debug(std::cout << "Op from " << from.getId()
                                     << " cannot be seen by " << (*I)->getId()
                                     << std::endl << std::flush;);
                     continue;
                 }
-                newop.setTo((*I)->getId());
-                deliverTo(newop, *I);
+                op->setTo((*I)->getId());
+                deliverTo(op, **I);
             }
         }
     }
@@ -467,25 +470,31 @@ void WorldRouter::addPerceptive(const std::string & id)
 /// will call this function again as soon as possible rather than sleeping.
 /// This ensures that the maximum possible number of operations are dispatched
 /// without becoming unresponsive to client communications traffic.
-bool WorldRouter::idle()
+/// @param sec world time seconds component
+/// @param usec world time microseconds component
+bool WorldRouter::idle(int sec, int usec)
 {
-    updateTime();
+    updateTime(sec, usec);
     unsigned int op_count = 0;
-    RootOperation * op;
-    while ((++op_count < 10) && ((op = getOperationFromQueue()) != NULL)) {
-        Dispatching.emit(op);
+    OpQueue::iterator I = m_operationQueue.begin();
+    while ((++op_count < 10) && (I != m_operationQueue.end()) &&
+           ((*I)->getSeconds() <= m_realTime)) {
+        assert(I != m_operationQueue.end());
+        OpQueEntry & oqe = *I;
+        Dispatching.emit(oqe.op);
         try {
-            operation(*op);
+            operation(oqe.op, oqe.from);
         }
         catch (...) {
             std::string msg = std::string("Exception caught in world.idle()")
                             + " thrown while processing "
-                            + op->getParents().front().asString()
-                            + " operation sent to " + op->getTo()
-                            + " from " + op->getFrom() + ".";
+                            // + oqe->getParents().front()
+                            + " operation sent to " + oqe->getTo()
+                            + " from " + oqe->getFrom() + ".";
             log(ERROR, msg.c_str());
         }
-        delete op;
+        m_operationQueue.erase(I);
+        I = m_operationQueue.begin();
     }
     // If we have processed the maximum number for this call, return true
     // to tell the server not to sleep when polling clients. This ensures
@@ -502,8 +511,8 @@ bool WorldRouter::idle()
 /// instance with this name was not found.
 Entity * WorldRouter::findByName(const std::string & name)
 {
-    EntityDict::const_iterator I = m_eobjects.begin();
-    for(; I != m_eobjects.end(); ++I) {
+    EntityDict::const_iterator Iend = m_eobjects.end();
+    for (EntityDict::const_iterator I = m_eobjects.begin(); I != Iend; ++I) {
         if (I->second->getName() == name) {
             return I->second;
         }
@@ -519,8 +528,8 @@ Entity * WorldRouter::findByName(const std::string & name)
 /// instance was found.
 Entity * WorldRouter::findByType(const std::string & type)
 {
-    EntityDict::const_iterator I = m_eobjects.begin();
-    for(; I != m_eobjects.end(); ++I) {
+    EntityDict::const_iterator Iend = m_eobjects.end();
+    for(EntityDict::const_iterator I = m_eobjects.begin(); I != Iend; ++I) {
         if (I->second->getType() == type) {
             return I->second;
         }
