@@ -6,17 +6,37 @@
 #define SERVER_WORLD_ROUTER_H
 
 #include "common/BaseWorld.h"
-#include "common/globals.h"
-#include "common/serialno.h"
+
+#include <list>
 
 extern "C" {
     #include <sys/time.h>
     #include <unistd.h>
 }
 
-class WorldRouter;
 class Entity;
-class World;
+
+struct OpQueEntry;
+#if 0
+struct OpQueEntry {
+    const Operation & op;
+    Entity & from;
+
+    explicit OpQueEntry(const Operation & o, Entity & f);
+    OpQueEntry(const OpQueEntry & o);
+    ~OpQueEntry();
+
+    Operation & operator*() const {
+        return *op.get();
+    }
+
+    Atlas::Objects::Operation::RootOperationData * operator->() const {
+        return op.get();
+    }
+};
+#endif
+
+typedef std::list<OpQueEntry> OpQueue;
 
 /// \brief WorldRouter encapsulates the game world running in the server.
 ///
@@ -37,35 +57,27 @@ class WorldRouter : public BaseWorld {
     /// List of omnipresent entities. Obsolete.
     EntitySet m_omnipresentList;
 
-    /// Generate a new operation serial number.
-    int newSerialNo() {
-        return opSerialNo();
-    }
-
-    void addOperationToQueue(RootOperation & op, const Entity *);
-    RootOperation * getOperationFromQueue();
-    const EntitySet & broadcastList(const RootOperation & op) const;
-    void updateTime();
-    void deliverTo(const RootOperation & op, Entity * e);
-    void deliverDeleteTo(const RootOperation & op, Entity * e);
+    void addOperationToQueue(const Operation &, Entity &);
+    Operation getOperationFromQueue();
+    const EntitySet & broadcastList(const Operation &) const;
+    void updateTime(int sec, int usec);
+    void deliverTo(const Operation &, Entity &);
+    void delEntity(Entity * obj);
   public:
     explicit WorldRouter();
     virtual ~WorldRouter();
 
-    bool idle();
-    Entity * addObject(Entity * obj, bool setup = true);
-    Entity * addNewObject(const std::string &,
-                          const MapType &);
-    void delObject(Entity * obj);
-    void setSerialnoOp(RootOperation &);
+    bool idle(int, int);
+    Entity * addEntity(Entity * obj, bool setup = true);
+    Entity * addNewEntity(const std::string &, const Atlas::Objects::Entity::RootEntity &);
 
-    void operation(const RootOperation & op);
+    void operation(const Operation &, Entity &);
 
     virtual void addPerceptive(const std::string &);
-    virtual void message(RootOperation & op, const Entity * obj);
+    virtual void message(const Operation &, Entity &);
     virtual Entity * findByName(const std::string & name);
     virtual Entity * findByType(const std::string & type);
-    virtual float constrainHeight(Entity *, const Point3D &);
+    virtual float constrainHeight(Entity *, const Point3D &, const std::string &);
 };
 
 #endif // SERVER_WORLD_ROUTER_H
