@@ -1,31 +1,41 @@
 // This file may be redistributed and modified only under the terms of
 // the GNU General Public License (See COPYING for details).
-// Copyright (C) 2000,2001 Alistair Riddoch
+// Copyright (C) 2000-2004 Alistair Riddoch
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "CommServer.h"
 #include "CommListener.h"
+#include "CommPeerListener.h"
 #include "CommUnixListener.h"
 #include "CommPSQLSocket.h"
 #include "CommMetaClient.h"
+#include "CommMDNSPublisher.h"
 #include "ServerRouting.h"
 #include "EntityFactory.h"
 #include "Persistance.h"
 #include "Restoration.h"
 #include "WorldRouter.h"
+#include "Admin.h"
 
 #include "rulesets/Python_API.h"
 #include "rulesets/MindFactory.h"
 
+#include "common/id.h"
 #include "common/log.h"
+#include "common/const.h"
 #include "common/debug.h"
 #include "common/globals.h"
 #include "common/inheritance.h"
 #include "common/system.h"
 #include "common/nls.h"
-#include "common/stringstream.h"
 
 #include <varconf/Config.h>
 
+#include <sstream>
+
 static const bool debug_flag = false;
 
 int main(int argc, char ** argv)
@@ -52,26 +62,28 @@ int main(int argc, char ** argv)
     // Initialise the persistance subsystem. If we have been built with
     // database support, this will open the various databases used to
     // store server data.
-    int dbstatus = Persistance::init();
-    if (dbstatus < 0) {
-        log(CRITICAL, _("Critical error opening databases. Init failed."));
-        if (dbstatus == -2) {
-            log(INFO, "Database connection established, but unable to create required tables.");
-            log(INFO, "Please ensure that any obsolete database tables have been removed.");
-        } else {
-            log(INFO, "Unable to connect to the RDBMS.");
-            log(INFO, "Please ensure that the RDBMS is running, the cyphesis database exists and is accessible to the user running cyphesis.");
+    if (consts::enable_database) {
+        int dbstatus = Persistance::init();
+        if (dbstatus < 0) {
+            log(CRITICAL, _("Critical error opening databases. Init failed."));
+            if (dbstatus == -2) {
+                log(INFO, "Database connection established, but unable to create required tables.");
+                log(INFO, "Please ensure that any obsolete database tables have been removed.");
+            } else {
+                log(INFO, "Unable to connect to the RDBMS.");
+                log(INFO, "Please ensure that the RDBMS is running, the cyphesis database exists and is accessible to the user running cyphesis.");
+            }
+            return EXIT_DATABASE_ERROR;
         }
-        return EXIT_DATABASE_ERROR;
-    }
 
-    // If the restricted flag is set in the config file, then we
-    // don't allow connecting users to create accounts. Accounts must
-    // be created manually by the server administrator.
-    if (global_conf->findItem("cyphesis", "restricted")) {
-        restricted_flag = global_conf->getItem("cyphesis","restricted");
-        if (restricted_flag) {
-            log(INFO, "Setting restricted mode.");
+        // If the restricted flag is set in the config file, then we
+        // don't allow connecting users to create accounts. Accounts must
+        // be created manually by the server administrator.
+        if (global_conf->findItem("cyphesis", "restricted")) {
+            restricted_flag = global_conf->getItem("cyphesis","restricted");
+            if (restricted_flag) {
+                log(INFO, "Setting restricted mode.");
+            }
         }
     }
 
@@ -89,7 +101,6 @@ int main(int argc, char ** argv)
         mserver = global_conf->getItem("cyphesis", "metaserver").as_string();
     }
 
-
     std::string serverName;
     if (global_conf->findItem("cyphesis", "servername")) {
         serverName = global_conf->getItem("cyphesis","servername").as_string();
@@ -107,6 +118,8 @@ int main(int argc, char ** argv)
     // world object pair (World + WorldRouter), and initialise the admin
     // account. The primary ruleset name is passed in so it
     // can be stored and queried by clients.
+    Inheritance::instance();
+
     WorldRouter world;
 
     ServerRouting server(world, rulesets.front(), serverName);
@@ -119,43 +132,59 @@ int main(int argc, char ** argv)
     // not creating a new world using the contents of the database as a
     // template
 
-    log(INFO, _("Restoring world from database..."));
+    if (consts::enable_database) {
+        log(INFO, _("Restoring world from database..."));
 
-    Restoration restore(server);
-    if (restore.read() == 1) {
-        std::cout << "Bootstrapping world" << std::endl << std::flush;
-        EntityFactory::instance()->initWorld();
+        Restoration restore(server);
+        if (restore.read() == 1) {
+            debug(std::cout << "Bootstrapping world" << std::endl << std::flush;);
+            EntityFactory::instance()->initWorld();
+        }
+
+        log(INFO, _(" world restored"));
+
+        CommPSQLSocket * dbsocket = new CommPSQLSocket(commServer,
+                                        Persistance::instance()->m_connection);
+        commServer.addSocket(dbsocket);
+        commServer.addIdle(dbsocket);
+    } else {
+        std::string adminId;
+        newId(adminId);
+        assert(!adminId.empty());
+        Admin * admin = new Admin(0, "admin", "BAD_HASH", adminId);
+        server.addAccount(admin);
     }
 
-    log(INFO, _(" world restored"));
-
     CommListener * listener = new CommListener(commServer);
-    if (listener->setup(port_num) != 0) {
-        log(ERROR, "Could not create listen socket. Init failed.");
+    if (listener->setup(client_port_num) != 0) {
+        log(ERROR, "Could not create client listen socket. Init failed.");
         return EXIT_SOCKET_ERROR;
     }
-    commServer.add(listener);
+    commServer.addSocket(listener);
 
-    CommUnixListener * llistener = new CommUnixListener(commServer);
-    if (llistener->setup() != 0) {
-        std::stringstream str;
-        str << "Could not create local listen socket with address \"";
-        str << llistener->getPath() << "\".";
-        log(ERROR, str.str().c_str());
-        delete llistener;
+    CommPeerListener * peerListener = new CommPeerListener(commServer);
+    if (peerListener->setup(peer_port_num) != 0) {
+        log(ERROR, "Could not create peer listen socket.");
+        delete peerListener;
     } else {
-        commServer.add(llistener);
+        commServer.addSocket(peerListener);
     }
 
-    CommPSQLSocket * dbsocket = new CommPSQLSocket(commServer,
-                                        Persistance::instance()->m_connection);
-    commServer.add(dbsocket);
-    commServer.addIdle(dbsocket);
+    CommUnixListener * localListener = new CommUnixListener(commServer);
+    if (localListener->setup(client_socket_name) != 0) {
+        std::stringstream str;
+        str << "Could not create local listen socket with address \"";
+        str << localListener->getPath() << "\".";
+        log(ERROR, str.str().c_str());
+        delete localListener;
+    } else {
+        commServer.addSocket(localListener);
+    }
 
     if (useMetaserver) {
         CommMetaClient * cmc = new CommMetaClient(commServer);
         if (cmc->setup(mserver) == 0) {
-            commServer.add(cmc);
+            commServer.addSocket(cmc);
             commServer.addIdle(cmc);
         } else {
             log(ERROR, "Error creating metaserver comm channel.");
@@ -163,6 +192,18 @@ int main(int argc, char ** argv)
         }
     }
 
+#if defined(HAVE_LIBHOWL)
+
+    CommMDNSPublisher * cmdns = new CommMDNSPublisher(commServer);
+    if (cmdns->setup() == 0) {
+        commServer.addSocket(cmdns);
+    } else {
+        log(ERROR, "No MDNS Responder for me.");
+        delete cmdns;
+    }
+
+#endif // defined(HAVE_LIBHOWL)
+
     log(INFO, "Running");
 
     // Inform things that want to know that we are running.
@@ -172,7 +213,7 @@ int main(int argc, char ** argv)
     // the code easily.
     while (!exit_flag) {
         try {
-            commServer.loop();
+            commServer.poll();
         }
         catch (...) {
             // It is hoped that commonly thrown exception, particularly
