@@ -7,6 +7,8 @@
 
 #include "EntityExerciser.h"
 
+#include <Atlas/Message/Element.h>
+
 template <class EntityType>
 class IGEntityExerciser : public EntityExerciser<EntityType> {
   public:
@@ -18,10 +20,13 @@ class IGEntityExerciser : public EntityExerciser<EntityType> {
 template <class EntityType>
 inline bool IGEntityExerciser<EntityType>::checkAttributes(const std::set<std::string> & attr_names)
 {
-    Element null;
+    Atlas::Message::Element null;
     std::set<std::string>::const_iterator I = attr_names.begin();
-    for(; I != attr_names.end(); ++I) {
+    std::set<std::string>::const_iterator Iend = attr_names.end();
+    for (; I != Iend; ++I) {
         if (!this->m_ent.get(*I, null)) {
+            std::cerr << "Entity does not have \"" << *I << "\" attribute."
+                      << std::endl << std::flush;
             return false;
         }
     }
