@@ -9,6 +9,9 @@
 
 #include <cassert>
 
+using Atlas::Message::MapType;
+using Atlas::Message::ListType;
+
 int main()
 {
     Stackable e("testId");
@@ -21,7 +24,6 @@ int main()
     // Subscribe the entity to every class of op
     std::set<std::string> opNames;
     ee.addAllOperations(opNames);
-    ee.subscribeOperations(opNames);
 
     // Throw an op of every type at the entity again now it is subscribed
     ee.runOperations();
