@@ -9,20 +9,24 @@
 
 #include <cassert>
 
+using Atlas::Message::MapType;
+using Atlas::Message::ListType;
+
 class TestWorld : public BaseWorld {
   public:
     explicit TestWorld(Entity & gw) : BaseWorld(gw) { }
 
-    virtual bool idle() { return false; }
-    virtual Entity * addObject(Entity * obj, bool setup = true) { return 0; }
-    virtual Entity * addNewObject(const std::string &, const MapType &) {
+    virtual bool idle(int, int) { return false; }
+    virtual Entity * addEntity(Entity * ent, bool setup = true) { return 0; }
+    virtual Entity * addNewEntity(const std::string &,
+                                  const Atlas::Objects::Entity::RootEntity &) {
         return 0;
     }
-    virtual void setSerialnoOp(RootOperation &) { }
-    virtual void message(RootOperation & op, const Entity * obj) { }
+    virtual void message(const Operation & op, Entity & ent) { }
     virtual Entity * findByName(const std::string & name) { return 0; }
     virtual Entity * findByType(const std::string & type) { return 0; }
-    virtual float constrainHeight(Entity*, const Point3D&) { return 0.f; }
+    virtual float constrainHeight(Entity*, const Point3D&,
+                                  const std::string&) { return 0.f; }
     virtual void addPerceptive(const std::string &) { }
 };
 
@@ -40,7 +44,6 @@ int main()
     // Subscribe the entity to every class of op
     std::set<std::string> opNames;
     ee.addAllOperations(opNames);
-    ee.subscribeOperations(opNames);
 
     // Throw an op of every type at the entity again now it is subscribed
     ee.runOperations();
